/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.collections;

import java.text.Collator;
import java.util.Comparator;
import java.util.Locale;

/**
 * A concrete implementation of Comparator that compares two strings.  If a locale
 * is specified then the comparison will be performed using the locale specific
 * collating sequences.  If the locale is not specified then a binary comparison
 * will be performed.
 *
 * @version $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class StringComparator
implements
Comparator {

    private final Locale locale_;
    private final Collator collator_;
    private final boolean isAscending_;

    /**
     * Create a locale specific comparator.
     *
     * @param locale The locale to be used when determining sorting order.
     *        If locale is null then a binary comparison is performed.
     * @param collatorStrength The strength value to be used by the
     *        Collator.  If locale is null then this value is ignored.
     * @param isAscending True if we are sorting in ascending order, false
     *        otherwise.
     */
    public StringComparator(
                           final Locale locale,
                           final int collatorStrength,
                           final boolean isAscending ) {

        locale_ = locale;
        if( locale_ == null ) {
            collator_ = null;
        }
        else {
            collator_ = Collator.getInstance(locale_);
            collator_.setStrength( collatorStrength );
        }
        isAscending_ = isAscending;
    }

    /**
     * Create a locale specific comparator.
     *
     * @param locale The locale to be used when determining sorting order.
     *        If locale is null then a binary comparison is performed.
     */
    public StringComparator( final Locale locale ) {
        this( locale, Collator.PRIMARY, true );
    }

    /**
     * Compare the two strings.
     * @param object1 The first string.
     * @param object2 The second string.
     * @return a negative integer, zero, or a positive integer as the first
     * argument is less than, equal to, or greater than the second.
     */
    public int compare( final Object object1, final Object object2 ) {
        int rc;

        final String string1 = object1.toString();
        final String string2 = object2.toString();

        if( locale_ == null ) {
            // Perform a binary compare.  Individual characters are sorted
            //by their unicode values.
            rc = string1.compareTo(string2);
        }
        else {
            // Perform a locale specific compare.
            rc = collator_.compare( string1, string2 );
        }

        // If we're sorting in descending order then flip the result now.
        if( isAscending_ == false ) {
            rc *= -1;
        }

        return rc;
    }
}

