/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.util.Locale;
import javax.swing.SwingUtilities;

/**
 *  An abstract superclass for GUI controller classes.<p>
 *
 * Swing uses an architecture similar to Model-View-Controller (MVC) in which the model, view
 * and controller components are kept seperate from each other.  This class provides
 * some common behaviour that is useful for controller objects.  This includes support for:
 * <ul>
 *
 * <li>Starting and managing worker threads.  See {@link #startTask(WorkerTask)},
 * {@link #taskComplete(WorkerTask)}, {@link #taskSuccessful(WorkerTask)},
 * {@link #taskExceptionThrown(WorkerTask,Exception)},
 * {@link #taskErrorThrown(WorkerTask,Throwable)}
 * <br />&nbsp;</li>
 *
 * <li>Simple support for localization.  See {@link #setLocale(Locale)},
 * {@link #getLocale()}, {@link #localeChanged(Locale)}</li>
 *
 * </ul>
 *
 * @version  $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public abstract class AbstractUIController {
    private Locale locale_ = Locale.getDefault();


    private class TaskRunnable implements Runnable {
        private Throwable throwable_;
        private final WorkerTask task_;

        /**
         * Create an instance
         * @param task The task that will be run
         */
        public TaskRunnable( final WorkerTask task ) {
            task_ = task;
        }

        /** Run the task */
        public void run() {

            try {
                task_.runOnWorkerThread();
            }
            catch( final Throwable t ) {
                throwable_ = t;
            }
            SwingUtilities.invokeLater(
                new Runnable() {
                    public void run() {
                        try {
                            run2();
                        }
                        catch( final Throwable t ) {
                            t.printStackTrace();
                        }
                    }
                    public void run2() {
                        if( throwable_ != null ) {
                            if( throwable_ instanceof Exception ) {
                                taskExceptionThrown( task_,
                                        (Exception)throwable_ );
                            }
                            else {
                                taskErrorThrown( task_, throwable_ );
                            }
                        }
                        else {
                            try {
                                task_.runOnUIThread();
                                taskSuccessful( task_ );
                            }
                            catch( final Exception e ) {
                                taskExceptionThrown( task_, e );
                            }
                            catch( final Throwable t ) {
                                taskErrorThrown( task_, t );
                            }
                        }
                        taskComplete( task_ );
                    }
                } );
        }
    }


    /**
     *  Create a new controller.
     */
    public AbstractUIController() {
    }


    /**
     *  Set the current locale
     *
     * @param  locale the new locale
     */
    public final void setLocale( final Locale locale ) {
        assertNotNull("locale", locale);
        locale_ = locale;
        localeChanged( locale_ );
    }


    /**
     *  Return the current locale.
     *
     * @return  The current locale
     */
    public final Locale getLocale() {
        return locale_;
    }


    /**
     *  The main entry point into this controller
     */
    public final void run() {
        localeChanged( getLocale() );
        runImpl();
    }


    /**
     *  Subclasses will override this to provide the run logic
     */
    protected abstract void runImpl();


    /**
     *  Start a WorkerTask. This involves running some code on a background
     *  thread and then some more on the ui thread. See WorkerTask for more
     *  details. <p>
     *
     *  When the task has completed, one of the following callbacks will be
     *  called based on the success of the task.
     *  <ul>
     *    <li> If the task completed without error then {@link
     *    #taskSuccessful(com.gargoylesoftware.base.gui.WorkerTask)
     *    taskSuccessful()} will be called
     *    <li> If an exception was thrown during processing of the task then
     *    {@link #taskExceptionThrown(com.gargoylesoftware.base.gui.WorkerTask,Exception)
     *    taskExceptionThrown()} will be called
     *    <li> If an error was thrown during processing of the task then {@link
     *    #taskErrorThrown(com.gargoylesoftware.base.gui.WorkerTask,Throwable)
     *    taskErrorThrown()} will be called
     *  </ul>
     *  <br />
     *  Finally, the method {@link #taskComplete(com.gargoylesoftware.base.gui.WorkerTask)
     *  taskSuccessful()} will be called will be called to signal the completion
     *  of the task
     *
     * @param  task The WorkerTask that is about to execute.
     */
    protected final void startTask( final WorkerTask task ) {
        new Thread( new TaskRunnable(task) ).start();
    }


    /**
     *  A callback that will be invoked when an exception is thrown during the
     *  processing of a WorkerTask. Note that errors and other throwables will
     *  processed by the method {@link #taskErrorThrown(WorkerTask,Throwable)}.<p>
     *
     * The default behaviour is to print the stack trace of the caught exception
     * to System.out.  Override this method to provide custom error handling.
     *
     * @param  task The task that failed
     * @param  exception The exception that was caught.
     */
    protected void taskExceptionThrown( final WorkerTask task, final Exception exception ) {
        exception.printStackTrace();
    }


    /**
     *  A callback that will be invoked when a system error is thrown during the
     *  processing of a WorkerTask. System errors are any throwable objects not
     *  descended from Exception. Typically, only system level code will be
     *  concerned by the errors handled by this method. Application code should
     *  only be concerned with the errors handled by
     *  {@link #taskExceptionThrown(WorkerTask,Exception)}.
     *
     * The default behaviour is to print the stack trace of the caught error
     * to System.out.  Override this method to provide custom error handling.
     *
     * @param  task The task that failed
     * @param  throwable The throwable object that was caught
     */
    protected void taskErrorThrown( final WorkerTask task, final Throwable throwable ) {
        throwable.printStackTrace();
    }


    /**
     *  A callback that will be invoked when a task completed successfully.
     *  Override this method to provide custom handling on completion
     *  of a task.<p>
     *
     * The default behaviour is to do nothing.  Override this to provide custom
     * behaviour.
     *
     * @param  task The task that just finished.
     */
    protected void taskSuccessful( final WorkerTask task ) {
    }


    /**
     *  A callback that will be invoked when a task has completed whether it was
     *  successful or not. Override this method to provide custom handling on
     *  completion of a task.
     *
     * The default behaviour is to do nothing.  Override this to provide custom
     * behaviour.
     *
     * @param  task The task that just finished.
     */
    protected void taskComplete( final WorkerTask task ) {
    }


    /**
     *  The current locale has changed - update all locale specific information.
     *  All logic that sets locale sensitive information should be executed in
     *  this method.
     *
     * @param  locale The new locale
     */
    protected abstract void localeChanged( final Locale locale );


    /**
     * Throw an exception if the specified object is null
     * @param fieldName The name of the paremeter we are checking
     * @param object The value of the parameter we are checking
     */
    protected final void assertNotNull( final String fieldName, final Object object ) {
        if( object == null ) {
            throw new DetailedNullPointerException("fieldName");
        }
    }
}

