/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.awt.Component;
import javax.swing.JScrollPane;

/**
 * A default implementation of ComponentLoader that uses reflection
 * to instantiate components.
 *
 * @version $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DefaultComponentLoader implements ComponentLoader {

    private final Class clazz_;
    private final boolean wrapInJScrollPane_;


    /**
     * Create a new loader.
     * @param className The name of the class that will be used to create the new object.
     * This class must have a public no-arg constructor and must be an instance of
     * java.awt.Component.
     * @throws ClassNotFoundException If the class cannot be found.
     */
    public DefaultComponentLoader( final String className ) throws ClassNotFoundException {
        this( className == null ? null : Class.forName(className) );
    }

    /**
     * Create a new loader.
     *
     * @param clazz The class that will be used to create the new object.  This
     *        class must have a public no-arg constructor and must be an instance
     *        of java.awt.Component.
     */
    public DefaultComponentLoader( final Class clazz ) {
        this( clazz, false );
    }


    /**
     * Create a new loader.
     *
     * @param clazz The class that will be used to create the new object.  This
     *        class must have a public no-arg constructor and must be an instance
     *        of java.awt.Component.
     * @param wrapInJScrollPane If true then the returned component will be a
     *        JScrollPane which contains the loaded component.
     */
    public DefaultComponentLoader( final Class clazz, final boolean wrapInJScrollPane ) {
        assertNotNull("clazz", clazz);
        if( Component.class.isAssignableFrom(clazz) == false ) {
            throw new DetailedIllegalArgumentException("clazz", clazz, "Not instance of Component");
        }
        try {
            clazz.getConstructor( new Class[0] );
        }
        catch( final NoSuchMethodException e ) {
            throw new DetailedIllegalArgumentException("clazz", clazz, "Does not have a public default constructor");
        }
        clazz_ = clazz;

        wrapInJScrollPane_ = wrapInJScrollPane;
    }


    /**
     * Load the component.
     *
     * @return the loaded component.
     * @throws Exception If something went wrong during the creation of the component.
     */
    public Component loadComponent() throws Exception {
        final Component component = (Component)clazz_.newInstance();
        final Component result;
        if( wrapInJScrollPane_ ) {
            result = new JScrollPane(component);
        }
        else {
            result = component;
        }
        return result;
    }


    /**
     * Throw an Exception if the specified object is null.
     *
     * @param fieldName The name of the field that we are checking.
     * @param object The value of the field that we are checking
     */
    protected final void assertNotNull( final String fieldName, final Object object ) {
        if( object == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}

