/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.awt.BorderLayout;
import java.awt.Component;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.JScrollPane;

/**
 * A panel that supports delayed loading of its contents.  This is useful when
 * the component that will fill this panel will take a long time to load.  A "please
 * wait" message will be displayed while the component is being loaded.  When loading
 * is complete, the new component will be made a child of this panel so that it can
 * become visible.<p>
 *
 * The following sample will create a delayed loader with a default "please wait" message.
 * <pre>
 * final DelayedComponentLoaderPanel panel = new DelayedComponentLoaderPanel();
 * panel.setComponentLoader( new DefaultComponentLoader(MyExpensiveComponent.class) );
 * </pre>
 *
 * This sample has a custom message.
 * <pre>
 * final JLabel label = new JLabel("My custom wait message");
 * final DelayedComponentLoaderPanel panel = new DelayedComponentLoaderPanel(label);
 * panel.setComponentLoader( new DefaultComponentLoader(MyExpensiveComponent.class) );
 * </pre>
 *
 * @version $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DelayedComponentLoaderPanel extends JComponent {
	private static final long serialVersionUID = 1;
    private static final int LOADER_STARTED  = 1;
    private static final int LOADER_FINISHED = 2;

    private Set listeners_;
    private Component waitingComponent_;

    /**
     * Create a new panel that will display a simple "please wait" message
     * when it is loading.
     */
    public DelayedComponentLoaderPanel() {
        this( new JLabel("Loading ... Please wait.") );
        JLabel label = (JLabel)waitingComponent_;
        label.setIcon( UIManager.getIcon("OptionPane.informationIcon") );
        label.setHorizontalAlignment( JLabel.CENTER );
        label.setOpaque(true);
    }

    /**
     * Create a new panel that will display the specified waitingComponent
     * while it is loading.
     *
     * @param waitingComponent The component to display while loading is in progress.
     */
    public DelayedComponentLoaderPanel( final Component waitingComponent ) {
        assertNotNull("waitingComponent", waitingComponent);

        setLayout( new BorderLayout() );
        waitingComponent_ = waitingComponent;
        add( BorderLayout.CENTER, waitingComponent_ );
    }

    /**
     * Set the new component loader.  The waiting component will be displayed
     * immediately and component loading will start on a background thread.
     * If the new loader is null then no loading will take place and the waiting
     * component will be displayed indefinitely
     *
     * @param loader The new component loader.
     */
    public void setComponentLoader( final ComponentLoader loader ) {
        // Display the waiting component
        replaceComponent( waitingComponent_ );

        if( loader != null ) {
            fireComponentLoadingEvent(LOADER_STARTED, loader, null);
            new Thread() {
                public void run() {
                    try {
                        sleep(500);
                    }
                    catch( final InterruptedException e ) {
                        // Ignore this
                    }

                    Component loadedComponent;

                    try {
                        loadedComponent = loader.loadComponent();
                    }
                    catch( final Throwable t ) {
                        loadedComponent = new JScrollPane(new ThrowablePanel(t));
                        t.printStackTrace();
                    }

                    final Component finalLoadedComponent = loadedComponent;
                    SwingUtilities.invokeLater( new Runnable() {
                        public void run() {
                            replaceComponent( finalLoadedComponent );
                            fireComponentLoadingEvent(LOADER_FINISHED, loader,
                                                      finalLoadedComponent );
                        }
                    } );
                }
            }.start();
        }
    }

    /**
     * Replace the current child with the specified component
     * @param component The new component.
     */
    private void replaceComponent( final Component component ) {
        removeAll();
        add( BorderLayout.CENTER, component );
        revalidate();
    }

    /**
     * Add the specified listener.
     *
     * @param listener The new listener
     */
    public void addDelayedComponentLoaderListener( final DelayedComponentLoaderListener listener ) {

        assertNotNull("listener", listener);

        // Perform delayed loading of the listeners collection
        if( listeners_ == null ) {
            synchronized(this) {
                if( listeners_ == null ) {
                    listeners_ = Collections.synchronizedSet(new HashSet());
                }
            }
        }
        listeners_.add(listener);
    }

    /**
     * Remove the specified listener.
     * @param listener the listener to remove.
     */
    public void removeDelayedComponentLoaderListener(final DelayedComponentLoaderListener listener ) {

        assertNotNull("listener", listener);

        if( listeners_ != null ) {
            listeners_.remove(listener);
        }
    }

    /**
     * Fire the component loading event.
     * @param action The action id
     * @param loader The loader
     * @param loadedComponent The component that was just loaded.
     */
    private void fireComponentLoadingEvent(
            final int action,
            final ComponentLoader loader,
            final Component loadedComponent ) {

        if( listeners_ != null && listeners_.isEmpty() == false ) {
            DelayedComponentLoaderListener listenerArray[];
            synchronized(listeners_) {
                listenerArray = new DelayedComponentLoaderListener[listeners_.size()];
                listeners_.toArray(listenerArray);
            }

            final DelayedComponentLoaderEvent event
                = new DelayedComponentLoaderEvent(this, loader, loadedComponent);

            int i;
            for( i=0; i<listenerArray.length; i++ ) {
                switch( action ) {
                    case LOADER_STARTED:
                        listenerArray[i].componentLoadingStarted(event);
                        break;

                    case LOADER_FINISHED:
                        listenerArray[i].componentLoadingFinished(event);
                        break;

                    default:
                        throw new DetailedIllegalArgumentException("action", new Integer(action), "Unexpected value");
                }
            }
        }
    }


    /**
     * Throw an exception if the specified object is null
     * @param fieldName The name of the paremeter we are checking
     * @param object The value of the parameter we are checking
     */
    protected final void assertNotNull( final String fieldName, final Object object ) {
        if( object == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}
