/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import javax.swing.AbstractAction;
import java.awt.event.ActionEvent;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

/**
 * A swing "action" that uses reflection to find and invoke a given target
 * method.<p>
 *
 * ReflectedAction is a Swing "action" that uses reflection to call a specific target
 * method when the action is fired.  Actions are often implemented using inner classes which
 * can be very inefficient in terms of performance and deployment size.  This action is a tiny
 * bit slower when it is actually fired but is smaller and generally faster otherwise due to
 * the fact that those inner classes are no longer needed.
 *
 * <pre>
 * Action searchAction = new ReflectedAction(this, "performSearch");
 * </pre>
 *
 * @version $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class ReflectedAction extends AbstractAction {

    private static final long serialVersionUID = -3881811288492789994L;
	private final Object object_;
    private final String methodName_;


    /**
     * Create a reflected action with an object and the name of the method that
     * will be invoked on that object when the action is fired.
     *
     * @param object The object upon which we will invoke a method when
     * the action is fired
     * @param methodName The name of the method that we will invoke.  This method
     * must take either no parameters or one parameter which is an ActionEvent.
     * @throws IllegalArgumentException if there are no matching methods.
     */
    public ReflectedAction( final Object object, final String methodName )
        throws IllegalArgumentException {

        assertNotNull("object", object);
        assertNotNull("methodName", methodName);

        findMethodOnObject(object, methodName);

        object_ = object;
        methodName_ = methodName;
    }


    /**
     * The action has been fired so now we use reflection to invoke the method that
     * was specified previously.
     *
     * @param event The ActionEvent
     */
    public void actionPerformed( final ActionEvent event ) {
        final Method method = findMethodOnObject(object_, methodName_);
        final Object parameters[];
        if( method.getParameterTypes().length == 0 ) {
            parameters = new Object[0];
        }
        else {
            parameters = new Object[]{event};
        }

        try {
            method.invoke(object_, parameters);
        }
        catch( final IllegalAccessException e ) {
            exceptionThrown(e);
        }
        catch( final InvocationTargetException e ) {
            exceptionThrown(e);
        }
        catch( final IllegalArgumentException e ) {
            exceptionThrown(e);
        }
    }


    /**
     * Return the Method object that corresponds to the specified method name.
     * The method may take either no arguments or may take an ActionEvent.
     *
     * @param object The object that contains the method
     * @param methodName The name of the method that we're looking for
     * @return The method that matches the specified name.
     * @throws IllegalArgumentException If there aren't any methods that match.
     * @throws SecurityException If we do not have access to this method.
     */
    private Method findMethodOnObject( final Object object, final String methodName ) {
        final Class clazz = object.getClass();
        Method method = null;

        try {
            method = clazz.getDeclaredMethod(methodName, new Class[0]);
        }
        catch( final NoSuchMethodException e ) {
            try {
                method = clazz.getDeclaredMethod(methodName, new Class[]{ActionEvent.class});
            }
            catch( final NoSuchMethodException ex ) {
                throw new DetailedIllegalArgumentException("methodName", methodName, "No such method");
            }
        }

        // If we wouldn't normally have access to this method then
        // request access now.  If we are denied then a SecurityException
        // will be thrown.
        if( method.isAccessible() == false ) {
            method.setAccessible(true);
        }
        return method;
    }


    /**
     * Callback that will be invoked if an exception is thrown during the
     * reflected method call.<p>
     *
     * Override this to provide custom error handling. Default behaviour
     * is to print a stack trace.
     *
     * @param exception The exception
     */
    protected void exceptionThrown( final Exception exception ) {
        if( exception instanceof InvocationTargetException ) {
            ((InvocationTargetException)exception).getTargetException().printStackTrace();
        }
        else {
            exception.printStackTrace();
        }
    }


    /**
     * Throw an exception if the specified object is null
     * @param fieldName The name of the paremeter we are checking
     * @param object The value of the parameter we are checking
     */
    protected final void assertNotNull( final String fieldName, final Object object ) {
        if( object == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}
