/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.collections.NotificationList;
import com.gargoylesoftware.base.collections.NotificationListEvent;
import com.gargoylesoftware.base.collections.NotificationListListener;
import com.gargoylesoftware.base.trace.Trace;
import com.gargoylesoftware.base.trace.TraceChannel;
import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.table.AbstractTableModel;
import java.util.Iterator;

/**
 * A table model that uses reflection to retrieve values out of the row objects.
 * <p>
 * The sample below will create a JTable with one row of data and one column per property
 * in the Date class (Date has 10 properties in JDK1.3).
 * <pre>
 * final JTable table = new JTable();
 * final ReflectedTableModel model = new ReflectedTableModel(Date.class);
 * model.getRows().add( new Date() );
 * table.setModel(model);
 * </pre>
 *
 * This sample will only provide columns for month and year.
 *
 * <pre>
 * final JTable table = new JTable();
 * final ReflectedTableModel model = new ReflectedTableModel();
 * model.getRows().add( new Date() );
 * model.getColumns().add( new ReflectedTableModel.ColumnInfo("month") );
 * model.getColumns().add( new ReflectedTableModel.ColumnInfo("year") );
 * table.setModel(model);
 * </pre>
 * <b>Tip: </b>To enable debugging information call {@link #setTraceChannel(TraceChannel)}
 * with a non-null TraceChannel.
 * <pre>
 * model.setTraceChannel(Trace.out)
 * </pre>
 *
 * @version $Revision: 1.7 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class ReflectedTableModel extends AbstractTableModel {
//TODO: Register ourselves as PropertyChangeListeners on the row objects so
//      that we know when to fire a model changed event.
//TODO: Listen for changes to the row object list so we can fire model
//      changed.
//TODO: Listen for changes to the column objects so we can update the model.

    private static final long serialVersionUID = -7537209244601820035L;

	/**
     * This class contains information about one specific column in the table.
     */
    public static class ColumnInfo {
        private final String columnName_;
        private final String propertyName_;

        /**
         * Create a new ColumnInfo with the specified column name and property name.
         * @param columnName The name used by the table
         * @param propertyName The name of the property that we will get the data from.
         */
        public ColumnInfo( final String columnName, final String propertyName ) {
            if( columnName == null ) {
                throw new DetailedNullPointerException("columnName");
            }
            if( propertyName == null ) {
                throw new DetailedNullPointerException("propertyName");
            }

            columnName_ = columnName;
            propertyName_ = propertyName;
        }

        /**
         * Create a new ColumnInfo where the column name and property name are the same.
         * @param name The name.
         */
        public ColumnInfo( final String name ) {
            this( name, name );
        }

        /**
         * Return the column name.
         * @return The column name.
         */
        public String getColumnName() {
            return columnName_;
        }

        /**
         * Return the property name
         * @return The property name.
         */
        public String getPropertyName() {
            return propertyName_;
        }
    }

    /**
     * If the row list changes then fire the appropriate table event.
     */
    private NotificationListListener rowListener_
        = new NotificationListListener() {

        public void listElementsAdded( final NotificationListEvent event ) {
            addRowElements( event.getNewValues() );
            fireTableRowsInserted( event.getStartIndex(), event.getEndIndex() );
        }
        public void listElementsRemoved( final NotificationListEvent event ) {
            removeRowElements( event.getOldValues() );
            fireTableRowsDeleted( event.getStartIndex(), event.getEndIndex() );
        }
        public void listElementsChanged( final NotificationListEvent event ) {
            removeRowElements( event.getOldValues() );
            addRowElements( event.getNewValues() );
            fireTableRowsUpdated( event.getStartIndex(), event.getEndIndex() );
        }
    };

    /**
     * If any change occurs to the columns then fire a structure changed.
     */
    private NotificationListListener columnListener_
        = new NotificationListListener() {

        public void listElementsAdded( final NotificationListEvent event ) {
            fireTableStructureChanged();
        }
        public void listElementsRemoved( final NotificationListEvent event ) {
            fireTableStructureChanged();
        }
        public void listElementsChanged( final NotificationListEvent event ) {
            fireTableStructureChanged();
        }
    };

    /**
     *
     */
    private PropertyChangeListener propertyChangeListener_
        = new PropertyChangeListener() {

        public void propertyChange( final PropertyChangeEvent event ) {
            final int row = rows_.indexOf( event.getSource() );
            final int columnCount = columns_.size();
            final String propertyName = event.getPropertyName();
            ColumnInfo columnInfo;

            int i;
            for( i=0; i<columnCount; i++ ) {
                columnInfo = (ColumnInfo)columns_.get(i);
                if( columnInfo.getPropertyName().equals(propertyName) ) {
                    fireTableCellUpdated( row, i );
                    if( traceChannel_ != null ) {
                        Trace.println(traceChannel_,
                                      "ReflectedTableModel property changed:"
                                      + " property=" + propertyName
                                      + " row=" +row
                                      + " column=" + i);
                    }
                }
            }
        }
    };

    private class RowElementControlData {
        /** How many times has this element been placed in the list.  Used to ensure
         * that we only add/remove a PropertyChangeListener once per object.
         */
        public int instanceCounter_ = 0;

        /** This will only be true if the specified object has both add and remove
         * methods for PropertyChangeListeners.
         */
        public boolean supportsPropertyChangeEvents_ = false;
    }

    // Keys are the row elements, values are instances of RowElementControlData
    private final Map rowElementControlDatas_ = new HashMap();

    private final List rows_;
    private final List columns_;

    private static final Object[] EMPTY_OBJECT_ARRAY = new Object[0];

    // If non-null then tracing will be enabled.
    private TraceChannel traceChannel_ = null;

    /**
     * Create an empty model with no columns and no rows.
     */
    public ReflectedTableModel() {

        final NotificationList columnList = new NotificationList( new ArrayList() );
        columnList.addNotificationListListener(columnListener_);
        columns_ = Collections.synchronizedList( columnList );

        final NotificationList rowList = new NotificationList( new ArrayList() );
        rowList.addNotificationListListener(rowListener_);
        rows_ = Collections.synchronizedList( rowList );
    }

    /**
     * Create an empty model with no rows but the columns preset to match the
     * properties in the given class.
     *
     * @param clazz The class to get properties from.
     * @throws IntrospectionException If the Introspector is unable to get
     * the properties for this class.
     */
    public ReflectedTableModel( final Class clazz ) throws IntrospectionException {
        this();
        final BeanInfo beanInfo = Introspector.getBeanInfo( clazz );
        final PropertyDescriptor propertyDescriptors[]
        = beanInfo.getPropertyDescriptors();

        String propertyName;

        int i;
        for( i=0; i<propertyDescriptors.length; i++ ) {
            propertyName = propertyDescriptors[i].getName();

            if( traceChannel_ != null ) {
                Trace.println( traceChannel_,
                               "ReflectedTableModel(Class) adding column: ["
                               + propertyName +"]" );
            }
            columns_.add( new ColumnInfo( propertyName ) );
        }
    }

    /**
     * Return a list containing the objects that are used to create each row.  This
     * list is backed by the original store such that changes to this list will be
     * reflected in the table model.
     * @return The rows.
     */
    public List getRows() {
        return rows_;
    }

    /**
     * Return a list containing the ColumnInfo objects that are used to define each
     * column.  This list is backed by the original store such that changes to this
     * list will be reflected in the table model.
     * @return The columns.
     */
    public List getColumns() {
        return columns_;
    }

    /**
     * Return the number of columns.
     * @return the number of columns.
     */
    public int getColumnCount() {
        return columns_.size();
    }

    /**
     * Return the number of rows.
     * @return The number of rows.
     */
    public int getRowCount() {
        return rows_.size();
    }

    /**
     * Return the specified object.
     * @param rowIndex The row index
     * @param columnIndex The columnIndex
     * @return The object at the specified row and column.
     */
    public Object getValueAt( final int rowIndex, final int columnIndex ) {
        final Object rowObject = rows_.get(rowIndex);
        final ColumnInfo columnInfo = (ColumnInfo)columns_.get(columnIndex);
        final String propertyName = columnInfo.getPropertyName();

        try {
            final BeanInfo beanInfo = Introspector.getBeanInfo( rowObject.getClass() );
            final PropertyDescriptor propertyDescriptors[]
            = beanInfo.getPropertyDescriptors();

            int i;
            for( i=0; i<propertyDescriptors.length; i++ ) {
                if( propertyDescriptors[i].getName().equals( propertyName ) ) {
                    final Method readMethod = propertyDescriptors[i].getReadMethod();
                    return readMethod.invoke( rowObject, EMPTY_OBJECT_ARRAY );
                }
            }
        }
        catch( final Exception e ) {
            if( traceChannel_ != null ) {
                Trace.printStackTrace( traceChannel_, e );
            }
        }

        return null;
    }

    /**
     * Return the name of the column at the specified index.
     * @param index The index of the column.
     * @return The name of the column at the specified index.
     */
    public String getColumnName( final int index ) {
        return((ColumnInfo)columns_.get(index)).getColumnName();
    }

    /**
     * Set the channel to be used for tracing.
     *
     * @param channel The channel to be used for tracing or null if tracing is
     * to be disabled.
     */
    public void setTraceChannel( final TraceChannel channel ) {
        traceChannel_ = channel;
    }

    /**
     * Return the channel currently being used for tracing or null if tracing
     * is disabled.
     * @return The trace channel or null if a channel hasn't been set.
     */
    public TraceChannel getTraceChannel() {
        return traceChannel_;
    }

    /**
     * Add a row element
     * @param object the object that will be used to populate this row
     */
    private synchronized void addRowElement( final Object object ) {

        RowElementControlData data =
        (RowElementControlData)rowElementControlDatas_.get(object);

        if( data == null ) {
            data = new RowElementControlData();

            final Class clazz = object.getClass();
            final Class parms[] = { PropertyChangeListener.class};
            Method method;
            try {
                // We try the "remove" first just to ensure that there is one.
                method = clazz.getMethod("removePropertyChangeListener", parms);
                method = clazz.getMethod("addPropertyChangeListener", parms);
                method.invoke(object, new Object[]{propertyChangeListener_} );
            }
            catch( NoSuchMethodException e ) {
                data.supportsPropertyChangeEvents_ = false;
            }
            catch( IllegalAccessException e ) {
                if( traceChannel_ != null ) {
                    Trace.printStackTrace(traceChannel_, e);
                }
            }
            catch( InvocationTargetException e ) {
                if( traceChannel_ != null ) {
                    Trace.printStackTrace(traceChannel_, e.getTargetException());
                }
            }

            rowElementControlDatas_.put(object, data);
        }

        data.instanceCounter_++;
    }

    /**
     * @param list The list of objects that will be used to create the rows.
     */
    private void addRowElements( final List list ) {
        final Iterator iterator = list.iterator();
        while( iterator.hasNext() ) {
            addRowElement( iterator.next() );
        }
    }

    /**
     * @param list The list of object that will be removed from the model
     */
    private void removeRowElements( final List list ) {
        final Iterator iterator = list.iterator();
        while( iterator.hasNext() ) {
            removeRowElement( iterator.next() );
        }
    }

    /**
     * Remove one row
     * @param object The object that was used to create this row.
     */
    private synchronized void removeRowElement( final Object object ) {

        RowElementControlData data =
        (RowElementControlData)rowElementControlDatas_.get(object);

        if( data == null ) {
            throw new DetailedIllegalArgumentException("object", object, "Not in row list");
        }

        final Class clazz = object.getClass();
        final Class parms[] = { PropertyChangeListener.class};
        Method method;
        try {
            method = clazz.getMethod("removePropertyChangeListener", parms);
            method.invoke(object, new Object[]{propertyChangeListener_} );
        }
        catch( final NoSuchMethodException e ) {
            throw new IllegalStateException(
                "object doesn't have a removePropertyChange(): "+object);
        }
        catch( IllegalAccessException e ) {
            if( traceChannel_ != null ) {
                Trace.printStackTrace(traceChannel_, e);
            }
        }
        catch( InvocationTargetException e ) {
            if( traceChannel_ != null ) {
                Trace.printStackTrace(traceChannel_, e.getTargetException());
            }
        }

        data.instanceCounter_--;
        if( data.instanceCounter_ == 0 ) {
            rowElementControlDatas_.remove(object);
        }
    }


    /**
     * Throw an exception if the specified object is null
     * @param fieldName The name of the paremeter we are checking
     * @param object The value of the parameter we are checking
     */
    protected final void assertNotNull( final String fieldName, final Object object ) {
        if( object == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}
