/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.io.Serializable;

/**
 * This object contains all the constraints that apply to a specific
 * component inside a TableLayout.
 * <p>
 * We use the convention of specifying row before column to conform with the
 * swing standards.  This seems counterintuitive when specifying x,y
 * (actually y,x) co-ordinates however we felt it was more important to
 * conform to existing java layout standards.
 *
 * @version $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class TableLayoutConstraints implements Serializable {

    // Required for serialization - do not remove or modify.
    private static final long serialVersionUID = 4323958798646824406L;

    private boolean isImmutable_ = false;

    // Note that the default value for obeyMaximumSize is FALSE
    private boolean obeyMaximumSize_ = false;

    private boolean obeyMinimumSize_ = true;

    private int row_;
    private int column_;
    private int rowSpan_;
    private int columnSpan_;
    private int verticalAlignment_;
    private boolean verticalStretch_;
    private int horizontalAlignment_;
    private boolean horizontalStretch_;

    /**
     * Create a new instance.
     * @param row The starting row.
     * @param column The starting column.
     */
    public TableLayoutConstraints( final int row,
                                   final int column ) {

        setRow(row);
        setColumn(column);

        setRowSpan(1);
        setColumnSpan(1);
        setVerticalAlignment( TableLayout.CENTER );
        setHorizontalAlignment( TableLayout.LEFT );
        setVerticalStretch( false );
        setHorizontalStretch( false );
    }

    /**
     * Return a string representation of this object.
     * @return Return a string representation of this object.
     */
    public final String toString() {
        return getClass().getName()
        + "] row_=[" + row_
        + "] rowSpan_=[" + rowSpan_
        + "] column_=[" + column_
        + "] columnSpan_=[" + columnSpan_
        + "] verticalAlignment_=[" + verticalAlignment_
        + "] verticalStretch_=["+ verticalStretch_
        + "] horizontalAlignment_=[" + horizontalAlignment_
        + "] horizontalStretch_=["+ horizontalStretch_
        + "] isImmutable_=["+ isImmutable_
        + "] obeyMinimumSize_=["+ obeyMinimumSize_
        + "] obeyMaximumSize_=["+ obeyMaximumSize_
        + "]";

    }

    /**
     * Make this object immutable.
     */
    public final void setImmutable() {
        isImmutable_ = true;
    }

    /**
     * Return true if this object is immutable.
     * @return Return true if this object is immutable.
     */
    public final boolean isImmutable() {
        return isImmutable_;
    }

    /**
     * Check to make sure that we are allowed to modify this object.  If not, throw an
     * exception.
     * @throws IllegalArgumentException If the object is immutable
     */
    private void ensureMutable() throws IllegalArgumentException {
        if( isImmutable_ == true ) {
            throw new IllegalArgumentException("This object is now immutable and cannot be modified");
        }
    }

    /**
     * Set the row.
     * @param row The row.
     */
    public final void setRow( final int row ) {
        ensureMutable();
        if( row < 0 ) {
            throw new DetailedIllegalArgumentException("row", new Integer(row), "Is less than zero");
        }
        row_ = row;
    }

    /**
     * Return the row.
     * @return The row.
     */
    public final int getRow() {
        return row_;
    }

    /**
     * Set the column.
     * @param column The new column.
     */
    public final void setColumn( final int column ) {
        ensureMutable();
        if( column < 0 ) {
            throw new DetailedIllegalArgumentException("column", new Integer(column), "Is less than zero");
        }
        column_ = column;
    }

    /**
     * Return the column.
     * @return The column.
     */
    public final int getColumn() {
        return column_;
    }

    /**
     * Set the number of rows that this component will span.  The new span
     * must be greater than or equal to one.
     * @param span The new row span.
     */
    public final void setRowSpan( final int span ) {
        ensureMutable();
        if( span < 1 ) {
            throw new DetailedIllegalArgumentException("span", span, "Is less than one");
        }
        rowSpan_ = span;
    }

    /**
     * Return the number of rows this component will span.
     * @return The row span.
     */
    public final int getRowSpan() {
        return rowSpan_;
    }

    /**
     * Set the number of columns that this component will span.  The new span
     * must be greater than or equal to one.
     * @param span The new span.
     */
    public final void setColumnSpan( final int span ) {
        ensureMutable();
        if( span < 1 ) {
            throw new DetailedIllegalArgumentException("span", new Integer(span), "Is less than one");
        }
        columnSpan_ = span;
    }

    /**
     * Return the number of columns this component will span.
     * @return the column span.
     */
    public final int getColumnSpan() {
        return columnSpan_;
    }

    /**
     * Set the vertical alignment of the component within the specified area.
     * Possible values are
     * <ul>
     * <li>TableLayout.TOP
     * <li>TableLayout.BOTTOM
     * <li>TableLayout.CENTER
     * </ul>
     * @param alignment The new alignment.
     */
    public final void setVerticalAlignment( final int alignment ) {
        ensureMutable();
        switch( alignment ) {
        case TableLayout.TOP:
        case TableLayout.BOTTOM:
        case TableLayout.CENTER:
            // Legal values
            break;

        default:
            throw new DetailedIllegalArgumentException("alignment", alignment);
        }

        verticalAlignment_ = alignment;
    }

    /**
     * Return the vertical alignment.
     * @return Return the vertical alignment.
     */
    public final int getVerticalAlignment() {
        return verticalAlignment_;
    }

    /**
     * Set the horizontal alignment of the component within the specified
     * area. Possible values are
     * <ul>
     * <li>TableLayout.LEFT
     * <li>TableLayout.RIGHT
     * <li>TableLayout.CENTER
     * </ul>
     * @param alignment The new alignment.
     */
    public final void setHorizontalAlignment( final int alignment ) {
        ensureMutable();

        switch( alignment ) {
        case TableLayout.LEFT:
        case TableLayout.RIGHT:
        case TableLayout.CENTER:
            // Legal values
            break;

        default:
            throw new DetailedIllegalArgumentException("alignment", alignment);
        }

        horizontalAlignment_ = alignment;
    }

    /**
     * Return the horizontalAlignment.
     * @return Return the horizontalAlignment.
     */
    public final int getHorizontalAlignment() {
        return horizontalAlignment_;
    }

    /**
     * Set whether or not this component can be stretched vertically.
     * @param stretch true if the component will stretch vertically.
     */
    public final void setVerticalStretch( final boolean stretch ) {
        ensureMutable();
        verticalStretch_ = stretch;
    }

    /**
     * Return true if this component can be stretched vertically.
     * @return Return true if this component can be stretched vertically.
     */
    public final boolean getVerticalStretch() {
        return verticalStretch_;
    }

    /**
     * Set whether or not this component can be stretched horizontally.
     * @param stretch true if the component will stretch horizontally.
     */
    public final void setHorizontalStretch( final boolean stretch ) {
        ensureMutable();
        horizontalStretch_ = stretch;
    }

    /**
     * Return true if this component can be stretched horizontally.
     * @return Return true if this component can be stretched horizontally.
     */
    public final boolean getHorizontalStretch() {
        return horizontalStretch_;
    }

    /**
     * Set whether or not the component should obey it's maximum size.  Most components
     * do not provide sensible maximum values so always obeying these values, ends up
     * with very strange layouts.  The default value is false.
     * @param obey true if we are to obey the maximum size.
     */
    public final void setObeyMaximumSize( final boolean obey ) {
        ensureMutable();
        obeyMaximumSize_ = obey;
    }

    /**
     * Return true if we are obeying the maximum size.
     * @return Return true if we are obeying the maximum size.
     */
    public final boolean getObeyMaximumSize() {
        return obeyMaximumSize_;
    }

    /**
     * Set whether or not we should obey the minimum size of the component.  Default is true.
     * @param obey true if we are to obey the minimum size.
     */
    public final void setObeyMinimumSize( final boolean obey ) {
        ensureMutable();
        obeyMinimumSize_ = obey;
    }

    /**
     * Return true if we are obeying the minimum size.
     * @return Return true if we are obeying the minimum size.
     */
    public final boolean getObeyMinimumSize() {
        return obeyMinimumSize_;
    }

    /**
     * Create an instance of TableLayoutConstraints from the specified string.
     * Format: "1,1"
     * @param constraintString The constraints string.
     * @return A new TableLayoutConstraints object.
     */
    public static TableLayoutConstraints makeConstraints( final String constraintString ) {
        assertNotNull("constraintString", constraintString);
        if( constraintString.length() == 0 ) {
            throw new DetailedIllegalArgumentException("constraintString", constraintString, "May not be empty");
        }

        final int commaIndex = constraintString.indexOf(',');
        if( commaIndex == -1 ) {
            throw new DetailedIllegalArgumentException("constraintString", constraintString, "Missing a comma");
        }

        final TableLayoutConstraints constraints = new TableLayoutConstraints(0,0);
        parseConstraintString( constraints, constraintString.substring(0, commaIndex), true );
        parseConstraintString( constraints, constraintString.substring(commaIndex+1), false );

        return constraints;
    }

    /**
     * Parse the constraint string.
     * @param constraints The new constraints object
     * @param constraintString The constraints as an input string
     * @param isRow true if these constraints are for a row
     */
    private static void parseConstraintString( final TableLayoutConstraints constraints,
                                               final String constraintString,
                                               final boolean isRow ) {

        final int stringLength = constraintString.length();

        if( stringLength == 0 ) {
            throw new DetailedIllegalArgumentException("constraintString", constraintString, "May not be empty");
        }

        int index = 0;
        while( index < stringLength && Character.isDigit( constraintString.charAt(index) ) ) {
            index++;
        }

        final int number = Integer.parseInt( constraintString.substring(0,index) );
        if( isRow ) {
            constraints.setRow(number);
        }
        else {
            constraints.setColumn(number);
        }

        char nextChar;

        for( ;index < stringLength; index++ ) {
            nextChar = constraintString.charAt(index);

            switch(nextChar) {
                case 's':
                    if( isRow ) {
                        constraints.setVerticalStretch(true);
                    }
                    else {
                        constraints.setHorizontalStretch(true);
                    }
                    break;

                case '+':
                    index++;
                    final int startIndex = index;
                    while( index < stringLength && Character.isDigit( constraintString.charAt(index) ) ) {
                        index++;
                    }

                    final int span = Integer.parseInt( constraintString.substring(startIndex,index) );
                    if( isRow ) {
                        constraints.setRowSpan(span);
                    }
                    else {
                        constraints.setColumnSpan(span);
                    }
                    index--;
                    break;

                default:
                    throw new IllegalArgumentException("Unexpected command ["+nextChar+"]");
            }
        }
    }


    /**
     * Verify that the specified value is not null.  If it is then throw an exception
     *
     * @param fieldName The name of the field to check
     * @param fieldValue The value of the field to check
     * @exception DetailedNullPointerException If fieldValue is null
     */
    protected static final void assertNotNull( final String fieldName, final Object fieldValue )
        throws DetailedNullPointerException {

        if( fieldValue == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}

