/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.LayoutManager;
import javax.swing.JPanel;

/**
 * A special panel that when used with a TableLayout, will draw
 * all the table cell boundaries.  This is intended for use when
 * debugging only.
 *
 * @version $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class TableLayoutDebuggingPanel extends JPanel {
	private static final long serialVersionUID = 1;
    private Color lineColor_;
    private boolean gridLinesEnabled_ = true;


    /**
     * Create a new panel with a default line colour of red.
     */
    public TableLayoutDebuggingPanel() {
        setLineColor( Color.red );
    }


    /**
     * Create a new panel with the specified layout and a default line colour of red.
     * @param layout The new layout manager.
     */
    public TableLayoutDebuggingPanel( final LayoutManager layout ) {
        this();
        setLayout(layout);
    }


    /**
     * Set the line colour.
     * @param lineColor The new line colour.
     */
    public void setLineColor( final Color lineColor ) {
        assertNotNull("lineColor", lineColor);
        lineColor_ = lineColor;
    }


    /**
     * Return the current line colour.
     * @return the line colour.
     */
    public Color getLineColor() {
        return lineColor_;
    }


    /**
     * Override the painting logic to draw the gridlines.
     * @param g The graphics object.
     */
    public void paint( final Graphics g ) {
        super.paint(g);

        if( gridLinesEnabled_ == true ) {
            final LayoutManager layout = getLayout();
            if( layout instanceof TableLayout ) {
                g.setColor( lineColor_ );
                ((TableLayout)layout).drawOutlines(g);
            }
        }
    }


    /**
     * Set whether or not the grid lines are visible.  Default is true.
     * @param enabled True if the grid lines are to be shown.
     */
    public void setGridLinesEnabled( final boolean enabled ) {
        gridLinesEnabled_ = enabled;
    }


    /**
     * Return true if the grid lines are enabled.
     * @return true if the grid lines are enabled.
     */
    public boolean isGridLinesEnabled() {
        return gridLinesEnabled_;
    }


    /**
     * Verify that the specified value is not null.  If it is then throw an exception
     *
     * @param fieldName The name of the field to check
     * @param fieldValue The value of the field to check
     * @exception DetailedNullPointerException If fieldValue is null
     */
    protected final void assertNotNull( final String fieldName, final Object fieldValue )
        throws DetailedNullPointerException {

        if( fieldValue == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}

