/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.objectstore;

import java.util.Collections;
import java.util.Map;

/**
 * An object store that just dispatches commands to other object stores
 *
 * @version $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DispatchingObjectStore extends ObjectStore {

    private final Map dispatchMap_;
    private final Map resourceFactoryMap_;


    /**
     * Create an instance.  If you use this constructor then you must
     * override findStoreClass() since the dispatchMap has not been populated
     *
     * @param nameToResourceFactoryMap keys are names, values are the
     * resource factory objects that are represented by the name.  This map
     * will be passed into any newly created ObjectStore's via the method
     * setResourceFactoryMap()
     */
    protected DispatchingObjectStore(final Map nameToResourceFactoryMap) {
        dispatchMap_ = Collections.EMPTY_MAP;
        resourceFactoryMap_ = nameToResourceFactoryMap;
    }

    /**
     * Create an instance
     * @param commandToStoreMap A map containing command to store mappings.  Keys
     * and values must be class objects.
     * @param nameToResourceFactoryMap keys are names, values are the
     * resource factory objects that are represented by the name.  This map
     * will be passed into any newly created ObjectStore's via the method
     * setResourceFactoryMap()
     */
    public DispatchingObjectStore( final Map commandToStoreMap,
                                   final Map nameToResourceFactoryMap ) {
        assertNotNull("commandToStoreMap", commandToStoreMap);
        assertNotNull("nameToResourceFactoryMap", nameToResourceFactoryMap);

        dispatchMap_ = commandToStoreMap;
        resourceFactoryMap_ = nameToResourceFactoryMap;
    }


    /**
     * Perform the dispatching.
     *
     * @param command The command to dispatch
     * @return The result of the command after the other store has processed it
     */
    protected Object executeImpl(
            final ObjectStoreCommand command ) {

        return findStore(command).execute(command);
    }


    /**
     * Find the ObjectStore for the specified command
     *
     * @param command The command for which we are finding the store
     * @return The object store
     * @throws ObjectStoreCommandNotSupportedException If we cannot find a
     * store for the specified command
     * @throws ObjectStoreException if an error occurs when finding or
     * creating an object store
     */
    protected ObjectStore findStore(
            final ObjectStoreCommand command )
        throws
            ObjectStoreCommandNotSupportedException,
            ObjectStoreException {

        final Class storeClass = (Class)dispatchMap_.get(command.getClass());
        if( storeClass == null ) {
            throw new ObjectStoreCommandNotSupportedException(command);
        }

        try {
            final ObjectStore store = (ObjectStore)storeClass.newInstance();
            store.setResourceFactoryMap(resourceFactoryMap_);
            return store;
        }
        catch( IllegalAccessException e ) {
            throw new ObjectStoreException(e);
        }
        catch( final InstantiationException e ) {
            throw new ObjectStoreException(e);
        }
    }
}
