/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.objectstore;

import com.gargoylesoftware.base.resource.ManagedResource;
import com.gargoylesoftware.base.resource.ResourceFactory;
import com.gargoylesoftware.base.resource.ResourceManager;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.util.Iterator;
import java.util.Map;

/**
 *  This is a wrapper for the data layer in an application. Any code that
 *  accesses a database or some other form of data should be in a subclass of
 *  ObjectStore
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public abstract class ObjectStore {

    private ResourceManager resourceManager_ = null;


    /**
     *  Create an instance
     */
    protected ObjectStore() {
    }


    /**
     *  <p>Set the resource map. This defines what class is used when a specific
     *  resource factory is requested.</p>
     * 
     *  <p>It is recommended to use {@link #setResourceManager(ResourceManager)} instead</p>
     * @param  inputMap A map containing string/class pairs.
     */
    public final void setResourceFactoryMap( final Map inputMap ) {
        assertNotNull("inputMap", inputMap);

        Map.Entry entry;
        final ResourceManager newResourceManager = new ResourceManager( getClass().getName() );

        final Iterator iterator = inputMap.entrySet().iterator();
        while( iterator.hasNext() ) {
            entry = ( Map.Entry )iterator.next();
            newResourceManager.addFactory( ( String )entry.getKey(), ( ResourceFactory )entry.getValue() );
        }

        setResourceManager( newResourceManager );
    }


    /**
     *  Set the resource manager
     *
     * @param  resourceManager The new resource manager
     */
    public final void setResourceManager( final ResourceManager resourceManager ) {
        if( resourceManager_ != null ) {
            resourceManager_.releaseAllResources();
        }
        resourceManager_ = resourceManager;
    }


    /**
     *  Gets the resource manager
     *
     * @return  The resource manager or null if a resource manager has not been
     *      set.
     */
    public final ResourceManager getResourceManager() {
        return resourceManager_;
    }


    /**
     *  Perform the actions specified by the key and return a value. The
     *  subclasses will perform the actual work in overridden versions of
     *  executeImpl()
     *
     * @param  command The object that tells the object store what to do
     * @return  The results of the actions or null if there are no results
     * @exception  ObjectStoreCommandNotSupportedException If the command
     * is not supported by this store
     * @exception  ObjectStoreException If an error occurs during processing
     * of this command.
     * @see  #executeImpl(ObjectStoreCommand)
     */
    public final synchronized Object execute(
            final ObjectStoreCommand command )
        throws
            ObjectStoreCommandNotSupportedException,
            ObjectStoreException {

        Object result;
        try {
            result = executeImpl( command );
        }
        catch( final Exception e ) {
            result = handleException( e );
        }
        catch( final Error e ) {
            result = handleError( e );
        }
        catch( final Throwable practicallyImpossibleButRequiredByTheCompiler ) {
            practicallyImpossibleButRequiredByTheCompiler.printStackTrace();
            result = null;
        }
        finally {
            if( resourceManager_ != null ) {
                resourceManager_.releaseAllResources();
            }
        }

        return result;
    }


    /**
     *  Handle an exception that occured during the processing of executeImpl().
     *  Usually these exceptions will be rethrown after logging them to some
     *  error log<p>
     *
     *  The default behaviour is to rethrow any ObjectStoreExceptions or
     *  ObjectStoreCommandNotSupportedExceptions. All other exceptions are
     *  wrapped in a new ObjectStoreException and then that wrapper is thown
     *
     * @param  exception the exception that had been thrown
     * @return  The object to return from execute in those cases where an
     *      exception is not thrown out of this method
     * @exception  ObjectStoreException The exception to be thrown back out of
     *      execute()
     * @exception  ObjectStoreCommandNotSupportedException the exception to be
     *      thrown back out of execute()
     */
    protected Object handleException( final Exception exception )
        throws
            ObjectStoreException,
            ObjectStoreCommandNotSupportedException {

        if( exception instanceof ObjectStoreException ) {
            throw ( ObjectStoreException )exception;
        }
        if( exception instanceof ObjectStoreCommandNotSupportedException ) {
            throw ( ObjectStoreCommandNotSupportedException )exception;
        }

        throw new ObjectStoreException( exception );
    }


    /**
     *  Handle an exception that occured during the processing of executeImpl().
     *  Usually these exceptions will be rethrown after logging them to some
     *  error log<p>
     *
     *  The default behaviour is to rethrow the error
     *
     * @param  error The error that had been thrown
     * @return  The object to return from execute in those cases where an
     *      exception is not thrown out of this method
     */
    protected Object handleError( final Error error ) {
        throw error;
    }


    /**
     *  Return a resource from the specified factory
     *
     * @param  name The name of the factory
     * @return  The specified resource
     * @see  #setResourceFactoryMap(Map)
     */
    protected final Object getResource( final String name ) {
        return getResourceManagerOrDie().getResource( name );
    }


    /**
     *  Override this to provide the actual processing of the object store.
     *
     * @param  command The object that tells the object store what to do
     * @return  The results of the actions or null if there are no results
     * @exception  ObjectStoreCommandNotSupportedException If the specified
     *      command is not understood by the object store
     * @exception  Throwable If an error occurs
     * @see  #execute(ObjectStoreCommand)
     */
    protected abstract Object executeImpl(
            final ObjectStoreCommand command )
        throws
            Throwable,
            ObjectStoreCommandNotSupportedException;


    /**
     *  Release the specified resource
     *
     * @param  object The resource to release
     */
    protected final void releaseResource( final ManagedResource object ) {
        getResourceManagerOrDie().releaseResource( object );
    }


    /**
     *  Return the resource manager. If a resource manager has not been
     *  specified then throw an exception.
     *
     * @return  The resource manager
     */
    private ResourceManager getResourceManagerOrDie() {
        if( resourceManager_ == null ) {
            throw new IllegalStateException( "No ResourceManager has been specified" );
        }
        return resourceManager_;
    }


    /**
     * Throw an exception if the specified object is null
     * @param fieldName The name of the paremeter we are checking
     * @param object The value of the parameter we are checking
     */
    protected final void assertNotNull( final String fieldName, final Object object ) {
        if( object == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}

