/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.objectstore;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * An object store that allows configuration by reflection.  Commands are mapped to
 * method names such that when a specified command is received, it will be dispatched
 * via reflection to the specified method.
 *
 * @version  $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public abstract class ReflectedObjectStore extends ObjectStore {

    private final Map commandMap_ = new HashMap( 89 );


    /**
     *  Create an instance
     */
    public ReflectedObjectStore() {
    }


    /**
     * Execute the specified command
     *
     * @param  command The command to execute
     * @return An object
     * @exception  Throwable If an error occurs
     */
    protected final Object executeImpl( final ObjectStoreCommand command )
        throws Throwable {

        final Method method = (Method)commandMap_.get( command.getClass() );
        if( method == null ) {
            throw new ObjectStoreCommandNotSupportedException( command );
        }

        try {
            return method.invoke( this, new Object[]{command} );
        }
        catch( final InvocationTargetException e ) {
            throw e.getTargetException();
        }
    }


    /**
     * Register a command and the method that will be invoked when that command is
     * passed into execute()
     *
     * @param  commandClass The class of the command
     * @param  methodName The name of the method that will be executed
     */
    public void registerCommand( final Class commandClass, final String methodName ) {
        assertNotNull( "commandClass", commandClass );
        assertNotNull( "methodName", methodName );

        if( ObjectStoreCommand.class.isAssignableFrom( commandClass ) == false ) {
            throw new DetailedIllegalArgumentException(
                "commandClass", commandClass, "Must be an instance of ObjectStoreCommand" );
        }

        final Method method;
        try {
            method = getClass().getDeclaredMethod( methodName, new Class[]{commandClass} );
        }
        catch( final NoSuchMethodException e ) {
            throw new DetailedIllegalArgumentException(
                "methodName", methodName, "No method found on class " + commandClass.getName() );
        }

        attemptToSuppressAccessControl( method );

        commandMap_.put( commandClass, method );
    }


    private void attemptToSuppressAccessControl( final Method method ) {
        try {
            method.setAccessible( true );
        }
        catch( final SecurityException e ) {
            // Nothing we can do if this fails
        }
    }
}

