/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import java.util.LinkedList;
import java.util.List;

/**
 *  A resource factory that provides object pooling
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class PooledResourceFactory extends ResourceFactory {

    private final ResourceFactory sourceFactory_;
    private int preferredCacheSize_;
    private List cache_ = new LinkedList();


    /**
     *  Create an instance
     *
     * @param  sourceFactory The factory that will be used to actually create
     *      and destroy the pooled resources
     */
    public PooledResourceFactory( final ResourceFactory sourceFactory ) {
        if( sourceFactory == null ) {
            throw new NullPointerException( "sourceFactory" );
        }
        sourceFactory_ = sourceFactory;
        setPreferredCacheSize( 4 );
    }


    /**
     *  Set the preferredCacheSize
     *
     * @param  size The new size. May not be negative
     */
    public void setPreferredCacheSize( final int size ) {
        if( size < 0 ) {
            throw new DetailedIllegalArgumentException( "size", new Integer(size), "May not be negative" );
        }
        preferredCacheSize_ = size;
    }


    /**
     *  Return the preferredCacheSize
     *
     * @return  The size
     */
    public int getPreferredCacheSize() {
        return preferredCacheSize_;
    }


    /**
     *  Reinitialize the resource to a known state. This is required for
     *  resource pooling as all resources being returned from a pool must have
     *  been initialized to a known state.
     *
     * @param  resource the resource to reinitialize
     * @return  true if the resource was successfully reinitialized
     */
    public synchronized boolean reinitializeResourceIfPossible( final ManagedResource resource ) {
        return sourceFactory_.reinitializeResourceIfPossible( resource );
    }



    /**
     *  Get a resource
     *
     * @param  resourceManager The manager that owns this factory
     * @return  A resource
     * @exception  Exception If an error occurs
     */
    protected synchronized ManagedResource getResourceImpl(
            final ResourceManager resourceManager )
        throws
            Exception {

        ManagedResource resource;

        synchronized( cache_ ) {
            if( cache_.isEmpty() == true ) {
                resource = sourceFactory_.getResource( resourceManager );
            }
            else {
                resource = (ManagedResource)cache_.get( 0 );
                cache_.remove( 0 );
            }
        }

        return resource;
    }


    /**
     *  Release a resource
     *
     * @param  resource The resource to release
     * @param  resourceManager The manager that owns this factory
     * @exception  Exception If an error occurs
     */
    protected synchronized void releaseResourceImpl(
            final ResourceManager resourceManager,
            final ManagedResource resource )
        throws
            Exception {

        synchronized( cache_ ) {
            if( cache_.size() < getPreferredCacheSize()
                     && sourceFactory_.reinitializeResourceIfPossible( resource ) == true ) {
                cache_.add( resource );
            }
            else {
                sourceFactory_.releaseResource( resourceManager, resource );
            }
        }
    }
}

