/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 *  An object that manages the resources allocated by the resource factories
 *
 * @version  $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public final class ResourceManager {

    private Map resourceFactories_ = new HashMap( 89 );
    private final String name_;


    /**
     *  Create an instance
     */
    public ResourceManager() {
        this( "<unknown>" );
    }


    /**
     *  Create an instance
     *
     * @param  name The name of this object.
     */
    public ResourceManager( final String name ) {
        if( name == null ) {
            throw new NullPointerException( "name" );
        }
        name_ = name;
    }


    /**
     *  Return a resource from the specified factory
     *
     * @param  name The name of the factory
     * @return  The specified resource
     */
    public final Object getResource( final String name ) {
        if( name == null ) {
            throw new NullPointerException( "name" );
        }
        final ManagedResource resource = getResourceFactory( name ).getResource( this );
        resource.setResourceFactoryName( name );
        return resource;
    }


    /**
     *  Release the specified resource
     *
     * @param  object The resource to release
     */
    public final void releaseResource( final ManagedResource object ) {
        if( object == null ) {
            throw new NullPointerException( "object" );
        }
        getResourceFactory( object.getResourceFactoryName() ).releaseResource( this, object );
    }


    /**
     *  Release all resources
     */
    public void releaseAllResources() {
        final Iterator iterator = resourceFactories_.values().iterator();
        while( iterator.hasNext() ) {
            ( ( ResourceFactory )iterator.next() ).releaseAllResources( this );
        }
    }


    /**
     *  Adds a feature to the Factory attribute of the ResourceManager object
     *
     * @param  name The feature to be added to the Factory attribute
     * @param  factory The feature to be added to the Factory attribute
     */
    public void addFactory( final String name, final ResourceFactory factory ) {
        if( resourceFactories_.containsKey( name ) ) {
            throw new DetailedIllegalArgumentException( "name", name, "A factory already registered for this name" );
        }
        resourceFactories_.put( name, factory );
    }


    /**
     * @param  name Description of Parameter
     * @return  The resourceFactory value
     * @exception  ResourceFactoryNotFoundException Description of Exception
     */
    private final ResourceFactory getResourceFactory(
            final String name )
        throws
            ResourceFactoryNotFoundException {

        final ResourceFactory factory = ( ResourceFactory )resourceFactories_.get( name );
        if( factory == null ) {
            throw new ResourceFactoryNotFoundException( name );
        }

        return factory;
    }


    /**
     * Return a string representation of this object
     * @return A string respresentation of this object.
     */
    public String toString() {
        return "ResourceManager[" + name_ + "]";
    }
}

