/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import java.math.BigDecimal;
import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Ref;
import java.sql.SQLException;
import java.util.Calendar;

/**
 *  The interface used to execute SQL stored procedures. JDBC provides a stored
 *  procedure SQL escape syntax that allows stored procedures to be called in a
 *  standard way for all RDBMSs. This escape syntax has one form that includes a
 *  result parameter and one that does not. If used, the result parameter must
 *  be registered as an OUT parameter. The other parameters can be used for
 *  input, output or both. Parameters are referred to sequentially, by number,
 *  with the first parameter being 1. <P>
 *
 *  <blockquote><pre>
 *   {?= call &lt;procedure-name&gt;[&lt;arg1&gt;,&lt;arg2&gt;, ...]}
 *   {call &lt;procedure-name&gt;[&lt;arg1&gt;,&lt;arg2&gt;, ...]}
 * </pre></blockquote> <P>
 *
 *  IN parameter values are set using the set methods inherited from
 *  PreparedStatement. The type of all OUT parameters must be registered prior
 *  to executing the stored procedure; their values are retrieved after
 *  execution via the <code>get</code> methods provided here. <P>
 *
 *  A <code>CallableStatement</code> can return one ResultSet or multiple <code>ResultSet</code>
 *  objects. Multiple <code>ResultSet</code> objects are handled using
 *  operations inherited from Statement. <P>
 *
 *  For maximum portability, a call's <code>ResultSet</code> objects and update
 *  counts should be processed prior to getting the values of output parameters.
 *  <P>
 *
 *  Methods that are new in the JDBC 2.0 API are marked "Since 1.2."
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public final class CallableStatementWrapper
         extends
        PreparedStatementWrapper
         implements
        CallableStatement {

    private CallableStatement delegate_;


    /**
     *  Create a wrapper
     *
     * @param  statement The statement that is being wrapped
     */
    public CallableStatementWrapper( final CallableStatement statement ) {
        super( statement );
        delegate_ = statement;
    }


    /**
     *  Retrieves the value of a JDBC <code>CHAR</code>, <code>VARCHAR</code>,
     *  or <code>LONGVARCHAR</code> parameter as a <code>String</code> in the
     *  Java programming language. <p>
     *
     *  For the fixed-length type JDBC <code>CHAR</code>, the <code>String</code>
     *  object returned has exactly the same value the JDBC <code>CHAR</code>
     *  value had in the database, including any padding added by the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final String getString( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getString( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>BIT</code> parameter as a <code>boolean</code>
     *  in the Java programming language.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>false</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final boolean getBoolean( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getBoolean( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>TINYINT</code> parameter as a <code>byte</code>
     *  in the Java programming language.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is 0.
     * @exception  SQLException if a database access error occurs
     */
    public final byte getByte( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getByte( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>SMALLINT</code> parameter as a <code>short</code>
     *  in the Java programming language.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is 0.
     * @exception  SQLException if a database access error occurs
     */
    public final short getShort( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getShort( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>INTEGER</code> parameter as an <code>int</code>
     *  in the Java programming language.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is 0.
     * @exception  SQLException if a database access error occurs
     */
    public final int getInt( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getInt( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>BIGINT</code> parameter as a <code>long</code>
     *  in the Java programming language.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is 0.
     * @exception  SQLException if a database access error occurs
     */
    public final long getLong( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getLong( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>FLOAT</code> parameter as a <code>float</code>
     *  in the Java programming language.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is 0.
     * @exception  SQLException if a database access error occurs
     */
    public final float getFloat( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getFloat( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>DOUBLE</code> parameter as a <code>double</code>
     *  in the Java programming language.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is 0.
     * @exception  SQLException if a database access error occurs
     */
    public final double getDouble( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getDouble( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>NUMERIC</code> parameter as a <code>java.math.BigDecimal</code>
     *  object with scale digits to the right of the decimal point.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @param  scale the number of digits to the right of the decimal point
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     * @deprecated
     */
    public final BigDecimal getBigDecimal( int parameterIndex, int scale )
        throws SQLException {
        checkIsOpen();
        return delegate_.getBigDecimal( parameterIndex, scale );
    }


    /**
     *  Gets the value of a JDBC <code>BINARY</code> or <code>VARBINARY</code>
     *  parameter as an array of <code>byte</code> values in the Java
     *  programming language.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final byte[] getBytes( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getBytes( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>DATE</code> parameter as a <code>java.sql.Date</code>
     *  object.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final java.sql.Date getDate( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getDate( parameterIndex );
    }


    /**
     *  Get the value of a JDBC <code>TIME</code> parameter as a <code>java.sql.Time</code>
     *  object.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final java.sql.Time getTime( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getTime( parameterIndex );
    }


    /**
     *  Gets the value of a JDBC <code>TIMESTAMP</code> parameter as a <code>java.sql.Timestamp</code>
     *  object.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final java.sql.Timestamp getTimestamp( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getTimestamp( parameterIndex );
    }

    //----------------------------------------------------------------------
    // Advanced features:


    /**
     *  Gets the value of a parameter as an <code>Object</code> in the Java
     *  programming language. <p>
     *
     *  This method returns a Java object whose type corresponds to the JDBC
     *  type that was registered for this parameter using the method <code>registerOutParameter</code>
     *  . By registering the target JDBC type as <code>java.sql.Types.OTHER</code>
     *  , this method can be used to read database-specific abstract data types.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  A <code>java.lang.Object</code> holding the OUT parameter value.
     * @exception  SQLException if a database access error occurs
     */
    public final Object getObject( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getObject( parameterIndex );
    }


    //--------------------------JDBC 2.0-----------------------------

    /**
     *  Gets the value of a JDBC <code>NUMERIC</code> parameter as a <code>java.math.BigDecimal</code>
     *  object with as many digits to the right of the decimal point as the
     *  value contains.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @return  the parameter value in full precision. If the value is SQL
     *      <code>NULL</code>, the result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final BigDecimal getBigDecimal( int parameterIndex )
        throws SQLException {
        checkIsOpen();
        return delegate_.getBigDecimal( parameterIndex );
    }


    /**
     *  Returns an object representing the value of OUT parameter <code>i</code>
     *  and uses <code>map</code> for the custom mapping of the parameter value.
     *  <p>
     *
     *  This method returns a Java object whose type corresponds to the JDBC
     *  type that was registered for this parameter using the method <code>registerOutParameter</code>
     *  . By registering the target JDBC type as <code>java.sql.Types.OTHER</code>
     *  , this method can be used to read database-specific abstract data types.
     *
     * @param  i the first parameter is 1, the second is 2, and so on
     * @param  map the mapping from SQL type names to Java classes
     * @return  a <code>java.lang.Object</code> holding the OUT parameter value
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final Object getObject( int i, java.util.Map map )
        throws SQLException {
        checkIsOpen();
        return delegate_.getObject( i, map );
    }


    /**
     *  Gets the value of a JDBC <code>REF(&lt;structured-type&gt;)</code>
     *  parameter as a Ref object in the Java programming language.
     *
     * @param  i the first parameter is 1, the second is 2, and so on
     * @return  the parameter value as a <code>Ref</code> object in the Java
     *      programming language. If the value was SQL <code>NULL</code>, the
     *      value <code>null</code> is returned.
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final Ref getRef( int i )
        throws SQLException {
        checkIsOpen();
        return delegate_.getRef( i );
    }


    /**
     *  Gets the value of a JDBC <code>BLOB</code> parameter as a Blob\ object
     *  in the Java programming language.
     *
     * @param  i the first parameter is 1, the second is 2, and so on
     * @return  the parameter value as a <code>Blob</code> object in the Java
     *      programming language. If the value was SQL <code>NULL</code>, the
     *      value <code>null</code> is returned.
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final Blob getBlob( int i )
        throws SQLException {
        checkIsOpen();
        return delegate_.getBlob( i );
    }


    /**
     *  Gets the value of a JDBC <code>CLOB</code> parameter as a <code>Clob</code>
     *  object in the Java programming language.
     *
     * @param  i the first parameter is 1, the second is 2, and so on
     * @return  the parameter value as a <code>Clob</code> object in the Java
     *      programming language. If the value was SQL <code>NULL</code>, the
     *      value <code>null</code> is returned.
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final Clob getClob( int i )
        throws SQLException {
        checkIsOpen();
        return delegate_.getClob( i );
    }


    /**
     *  Gets the value of a JDBC <code>ARRAY</code> parameter as an Array object
     *  in the Java programming language.
     *
     * @param  i the first parameter is 1, the second is 2, and so on
     * @return  the parameter value as an <code>Array</code> object in the Java
     *      programming language. If the value was SQL <code>NULL</code>, the
     *      value <code>null</code> is returned.
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final Array getArray( int i )
        throws SQLException {
        checkIsOpen();
        return delegate_.getArray( i );
    }


    /**
     *  Gets the value of a JDBC <code>DATE</code> parameter as a <code>java.sql.Date</code>
     *  object, using the given <code>Calendar</code> object to construct the
     *  date. With a <code>Calendar</code> object, the driver can calculate the
     *  date taking into account a custom timezone and locale. If no <code>Calendar</code>
     *  object is specified, the driver uses the default timezone and locale.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @param  cal the <code>Calendar</code> object the driver will use to
     *      construct the date
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final java.sql.Date getDate( int parameterIndex, Calendar cal )
        throws SQLException {
        checkIsOpen();
        return delegate_.getDate( parameterIndex, cal );
    }


    /**
     *  Gets the value of a JDBC <code>TIME</code> parameter as a <code>java.sql.Time</code>
     *  object, using the given <code>Calendar</code> object to construct the
     *  time. With a <code>Calendar</code> object, the driver can calculate the
     *  time taking into account a custom timezone and locale. If no <code>Calendar</code>
     *  object is specified, the driver uses the default timezone and locale.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @param  cal the <code>Calendar</code> object the driver will use to
     *      construct the time
     * @return  the parameter value; if the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final java.sql.Time getTime( int parameterIndex, Calendar cal )
        throws SQLException {
        checkIsOpen();
        return delegate_.getTime( parameterIndex, cal );
    }


    /**
     *  Gets the value of a JDBC <code>TIMESTAMP</code> parameter as a <code>java.sql.Timestamp</code>
     *  object, using the given <code>Calendar</code> object to construct the
     *  <code>Timestamp</code> object. With a <code>Calendar</code> object, the
     *  driver can calculate the timestamp taking into account a custom timezone
     *  and locale. If no <code>Calendar</code> object is specified, the driver
     *  uses the default timezone and locale.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @param  cal the <code>Calendar</code> object the driver will use to
     *      construct the timestamp
     * @return  the parameter value. If the value is SQL <code>NULL</code>, the
     *      result is <code>null</code>.
     * @exception  SQLException if a database access error occurs
     */
    public final java.sql.Timestamp getTimestamp( int parameterIndex, Calendar cal )
        throws SQLException {
        checkIsOpen();
        return delegate_.getTimestamp( parameterIndex, cal );
    }


    /**
     *  Registers the OUT parameter in ordinal position <code>parameterIndex</code>
     *  to the JDBC type <code>sqlType</code>. All OUT parameters must be
     *  registered before a stored procedure is executed. <p>
     *
     *  The JDBC type specified by <code>sqlType</code> for an OUT parameter
     *  determines the Java type that must be used in the <code>get</code>
     *  method to read the value of that parameter. <p>
     *
     *  If the JDBC type expected to be returned to this output parameter is
     *  specific to this particular database, <code>sqlType</code> should be
     *  <code>java.sql.Types.OTHER</code>. The method getObject retrieves the
     *  value.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @param  sqlType the JDBC type code defined by <code>java.sql.Types</code>
     *      . If the parameter is of JDBC type <code>NUMERIC</code> or <code>DECIMAL</code>
     *      , the version of <code>registerOutParameter</code> that accepts a
     *      scale value should be used.
     * @exception  SQLException if a database access error occurs
     */
    public final void registerOutParameter( int parameterIndex, int sqlType )
        throws SQLException {
        checkIsOpen();
        delegate_.registerOutParameter( parameterIndex, sqlType );
    }


    /**
     *  Registers the parameter in ordinal position <code>parameterIndex</code>
     *  to be of JDBC type <code>sqlType</code>. This method must be called
     *  before a stored procedure is executed. <p>
     *
     *  The JDBC type specified by <code>sqlType</code> for an OUT parameter
     *  determines the Java type that must be used in the <code>get</code>
     *  method to read the value of that parameter. <p>
     *
     *  This version of <code>registerOutParameter</code> should be used when
     *  the parameter is of JDBC type <code>NUMERIC</code> or <code>DECIMAL</code>
     *  .
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, and so
     *      on
     * @param  sqlType SQL type code defined by <code>java.sql.Types</code>.
     * @param  scale the desired number of digits to the right of the decimal
     *      point. It must be greater than or equal to zero.
     * @exception  SQLException if a database access error occurs
     */
    public final void registerOutParameter( int parameterIndex, int sqlType, int scale )
        throws SQLException {
        checkIsOpen();
        delegate_.registerOutParameter( parameterIndex, sqlType, scale );
    }


    /**
     *  Indicates whether or not the last OUT parameter read had the value of
     *  SQL <code>NULL</code>. Note that this method should be called only after
     *  calling a <code>getXXX</code> method; otherwise, there is no value to
     *  use in determining whether it is <code>null</code> or not.
     *
     * @return  <code>true</code> if the last parameter read was SQL <code>NULL</code>
     *      ; <code>false</code> otherwise
     * @exception  SQLException if a database access error occurs
     */
    public final boolean wasNull()
        throws SQLException {
        checkIsOpen();
        return delegate_.wasNull();
    }


    /**
     *  Registers the designated output parameter. This version of the method
     *  <code>registerOutParameter</code> should be used for a user-named or REF
     *  output parameter. Examples of user-named types include: STRUCT,
     *  DISTINCT, JAVA_OBJECT, and named array types. Before executing a stored
     *  procedure call, you must explicitly call <code>registerOutParameter</code>
     *  to register the type from <code>java.sql.Types</code> for each OUT
     *  parameter. For a user-named parameter the fully-qualified SQL type name
     *  of the parameter should also be given, while a REF parameter requires
     *  that the fully-qualified type name of the referenced type be given. A
     *  JDBC driver that does not need the type code and type name information
     *  may ignore it. To be portable, however, applications should always
     *  provide these values for user-named and REF parameters. Although it is
     *  intended for user-named and REF parameters, this method may be used to
     *  register a parameter of any JDBC type. If the parameter does not have a
     *  user-named or REF type, the typeName parameter is ignored. <P>
     *
     *  <B>Note:</B> When reading the value of an out parameter, you must use
     *  the <code>getXXX</code> method whose Java type XXX corresponds to the
     *  parameter's registered SQL type.
     *
     * @param  paramIndex the first parameter is 1, the second is 2,...
     * @param  sqlType a value from java.sql.Types
     * @param  typeName the fully-qualified name of an SQL structured type
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void registerOutParameter( int paramIndex, int sqlType, String typeName )
        throws SQLException {
        checkIsOpen();
        delegate_.registerOutParameter( paramIndex, sqlType, typeName );
    }


    /**
     * Retrieves the value of the designated JDBC <code>DATALINK</code> parameter as a
     * <code>java.net.URL</code> object.
     *
     * @param parameterIndex the first parameter is 1, the second is 2,...
     * @return a <code>java.net.URL</code> object that represents the
     *         JDBC <code>DATALINK</code> value used as the designated
     *         parameter
     * @exception SQLException if a database access error occurs,
     *            or if the URL being returned is
     *            not a valid URL on the Java platform
     * @see #setURL
     * @since 1.4
     */
    public java.net.URL getURL(int parameterIndex) throws SQLException {
        checkIsOpen();
        return delegate_.getURL(parameterIndex);
    }


    /**
     * Sets the designated parameter to the given <code>java.net.URL</code> object.
     * The driver converts this to an SQL <code>DATALINK</code> value when
     * it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param val the parameter value
     * @exception SQLException if a database access error occurs,
     *            or if a URL is malformed
     * @since 1.4
     */
    public void setURL(String parameterName, java.net.URL val) throws SQLException {
        checkIsOpen();
        delegate_.setURL(parameterName, val);
    }


    /**
     * Sets the designated parameter to SQL <code>NULL</code>.
     *
     * <P><B>Note:</B> You must specify the parameter's SQL type.
     *
     * @param parameterName the name of the parameter
     * @param sqlType the SQL type code defined in <code>java.sql.Types</code>
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setNull(String parameterName, int sqlType) throws SQLException {
        checkIsOpen();
        delegate_.setNull(parameterName, sqlType);
    }


    /**
     * Sets the designated parameter to the given Java <code>boolean</code> value.
     * The driver converts this
     * to an SQL <code>BIT</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setBoolean(String parameterName, boolean x) throws SQLException {
        checkIsOpen();
        delegate_.setBoolean(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given Java <code>byte</code> value.
     * The driver converts this
     * to an SQL <code>TINYINT</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setByte(String parameterName, byte x) throws SQLException {
        checkIsOpen();
        delegate_.setByte(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given Java <code>short</code> value.
     * The driver converts this
     * to an SQL <code>SMALLINT</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setShort(String parameterName, short x) throws SQLException {
        checkIsOpen();
        delegate_.setShort( parameterName, x );
    }


    /**
     * Sets the designated parameter to the given Java <code>int</code> value.
     * The driver converts this
     * to an SQL <code>INTEGER</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setInt(String parameterName, int x) throws SQLException {
        checkIsOpen();
        delegate_.setInt(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given Java <code>long</code> value.
     * The driver converts this
     * to an SQL <code>BIGINT</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setLong(String parameterName, long x) throws SQLException {
        checkIsOpen();
        delegate_.setLong(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given Java <code>float</code> value.
     * The driver converts this
     * to an SQL <code>FLOAT</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setFloat(String parameterName, float x) throws SQLException {
        checkIsOpen();
        delegate_.setFloat(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given Java <code>double</code> value.
     * The driver converts this
     * to an SQL <code>DOUBLE</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setDouble(String parameterName, double x) throws SQLException {
        checkIsOpen();
        delegate_.setDouble(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given
     * <code>java.math.BigDecimal</code> value.
     * The driver converts this to an SQL <code>NUMERIC</code> value when
     * it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setBigDecimal(String parameterName, BigDecimal x) throws SQLException {
        checkIsOpen();
        delegate_.setBigDecimal(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given Java <code>String</code> value.
     * The driver converts this
     * to an SQL <code>VARCHAR</code> or <code>LONGVARCHAR</code> value
     * (depending on the argument's
     * size relative to the driver's limits on <code>VARCHAR</code> values)
     * when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setString(String parameterName, String x) throws SQLException {
        checkIsOpen();
        delegate_.setString(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given Java array of bytes.
     * The driver converts this to an SQL <code>VARBINARY</code> or
     * <code>LONGVARBINARY</code> (depending on the argument's size relative
     * to the driver's limits on <code>VARBINARY</code> values) when it sends
     * it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setBytes(String parameterName, byte x[]) throws SQLException {
        checkIsOpen();
        delegate_.setBytes(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given <code>java.sql.Date</code> value.
     * The driver converts this
     * to an SQL <code>DATE</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setDate(String parameterName, java.sql.Date x) throws SQLException {
        checkIsOpen();
        delegate_.setDate(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given <code>java.sql.Time</code> value.
     * The driver converts this
     * to an SQL <code>TIME</code> value when it sends it to the database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setTime(String parameterName, java.sql.Time x) throws SQLException {
        checkIsOpen();
        delegate_.setTime(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given <code>java.sql.Timestamp</code> value.
     * The driver
     * converts this to an SQL <code>TIMESTAMP</code> value when it sends it to the
     * database.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setTimestamp(String parameterName, java.sql.Timestamp x) throws SQLException {
        checkIsOpen();
        delegate_.setTimestamp(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given input stream, which will have
     * the specified number of bytes.
     * When a very large ASCII value is input to a <code>LONGVARCHAR</code>
     * parameter, it may be more practical to send it via a
     * <code>java.io.InputStream</code>. Data will be read from the stream
     * as needed until end-of-file is reached.  The JDBC driver will
     * do any necessary conversion from ASCII to the database char format.
     *
     * <P><B>Note:</B> This stream object can either be a standard
     * Java stream object or your own subclass that implements the
     * standard interface.
     *
     * @param parameterName the name of the parameter
     * @param x the Java input stream that contains the ASCII parameter value
     * @param length the number of bytes in the stream
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setAsciiStream(String parameterName, java.io.InputStream x, int length) throws SQLException {
        checkIsOpen();
        delegate_.setAsciiStream(parameterName, x, length);
    }


    /**
     * Sets the designated parameter to the given input stream, which will have
     * the specified number of bytes.
     * When a very large binary value is input to a <code>LONGVARBINARY</code>
     * parameter, it may be more practical to send it via a
     * <code>java.io.InputStream</code> object. The data will be read from the stream
     * as needed until end-of-file is reached.
     *
     * <P><B>Note:</B> This stream object can either be a standard
     * Java stream object or your own subclass that implements the
     * standard interface.
     *
     * @param parameterName the name of the parameter
     * @param x the java input stream which contains the binary parameter value
     * @param length the number of bytes in the stream
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setBinaryStream(String parameterName, java.io.InputStream x, int length) throws SQLException {
        checkIsOpen();
        delegate_.setBinaryStream(parameterName, x, length);
    }


    /**
     * Sets the value of the designated parameter with the given object. The second
     * argument must be an object type; for integral values, the
     * <code>java.lang</code> equivalent objects should be used.
     *
     * <p>The given Java object will be converted to the given targetSqlType
     * before being sent to the database.
     *
     * If the object has a custom mapping (is of a class implementing the
     * interface <code>SQLData</code>),
     * the JDBC driver should call the method <code>SQLData.writeSQL</code> to write it
     * to the SQL data stream.
     * If, on the other hand, the object is of a class implementing
     * <code>Ref</code>, <code>Blob</code>, <code>Clob</code>, <code>Struct</code>,
     * or <code>Array</code>, the driver should pass it to the database as a
     * value of the corresponding SQL type.
     * <P>
     * Note that this method may be used to pass datatabase-
     * specific abstract data types.
     *
     * @param parameterName the name of the parameter
     * @param x the object containing the input parameter value
     * @param targetSqlType the SQL type (as defined in java.sql.Types) to be
     * sent to the database. The scale argument may further qualify this type.
     * @param scale for java.sql.Types.DECIMAL or java.sql.Types.NUMERIC types,
     *          this is the number of digits after the decimal point.  For all other
     *          types, this value will be ignored.
     * @exception SQLException if a database access error occurs
     * @see java.sql.Types
     * @since 1.4
     */
    public void setObject(String parameterName, Object x, int targetSqlType, int scale) throws SQLException {
        checkIsOpen();
        delegate_.setObject(parameterName, x, targetSqlType, scale);
    }


    /**
     * Sets the value of the designated parameter with the given object.
     * This method is like the method <code>setObject</code>
     * above, except that it assumes a scale of zero.
     *
     * @param parameterName the name of the parameter
     * @param x the object containing the input parameter value
     * @param targetSqlType the SQL type (as defined in java.sql.Types) to be
     *                      sent to the database
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setObject(String parameterName, Object x, int targetSqlType) throws SQLException {
        checkIsOpen();
        delegate_.setObject(parameterName, x, targetSqlType);
    }


    /**
     * Sets the value of the designated parameter with the given object.
     * The second parameter must be of type <code>Object</code>; therefore, the
     * <code>java.lang</code> equivalent objects should be used for built-in types.
     *
     * <p>The JDBC specification specifies a standard mapping from
     * Java <code>Object</code> types to SQL types.  The given argument
     * will be converted to the corresponding SQL type before being
     * sent to the database.
     *
     * <p>Note that this method may be used to pass datatabase-
     * specific abstract data types, by using a driver-specific Java
     * type.
     *
     * If the object is of a class implementing the interface <code>SQLData</code>,
     * the JDBC driver should call the method <code>SQLData.writeSQL</code>
     * to write it to the SQL data stream.
     * If, on the other hand, the object is of a class implementing
     * <code>Ref</code>, <code>Blob</code>, <code>Clob</code>, <code>Struct</code>,
     * or <code>Array</code>, the driver should pass it to the database as a
     * value of the corresponding SQL type.
     * <P>
     * This method throws an exception if there is an ambiguity, for example, if the
     * object is of a class implementing more than one of the interfaces named above.
     *
     * @param parameterName the name of the parameter
     * @param x the object containing the input parameter value
     * @exception SQLException if a database access error occurs or if the given
     *            <code>Object</code> parameter is ambiguous
     * @since 1.4
     */
    public void setObject(String parameterName, Object x) throws SQLException {
        checkIsOpen();
        delegate_.setObject(parameterName, x);
    }


    /**
     * Sets the designated parameter to the given <code>Reader</code>
     * object, which is the given number of characters long.
     * When a very large UNICODE value is input to a <code>LONGVARCHAR</code>
     * parameter, it may be more practical to send it via a
     * <code>java.io.Reader</code> object. The data will be read from the stream
     * as needed until end-of-file is reached.  The JDBC driver will
     * do any necessary conversion from UNICODE to the database char format.
     *
     * <P><B>Note:</B> This stream object can either be a standard
     * Java stream object or your own subclass that implements the
     * standard interface.
     *
     * @param parameterName the name of the parameter
     * @param reader the <code>java.io.Reader</code> object that
     *        contains the UNICODE data used as the designated parameter
     * @param length the number of characters in the stream
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setCharacterStream(String parameterName, java.io.Reader reader, int length) throws SQLException {
        checkIsOpen();
        delegate_.setCharacterStream(parameterName, reader, length);
    }


    /**
     * Sets the designated parameter to the given <code>java.sql.Date</code> value,
     * using the given <code>Calendar</code> object.  The driver uses
     * the <code>Calendar</code> object to construct an SQL <code>DATE</code> value,
     * which the driver then sends to the database.  With a
     * a <code>Calendar</code> object, the driver can calculate the date
     * taking into account a custom timezone.  If no
     * <code>Calendar</code> object is specified, the driver uses the default
     * timezone, which is that of the virtual machine running the application.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @param cal the <code>Calendar</code> object the driver will use
     *            to construct the date
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setDate(String parameterName, java.sql.Date x, Calendar cal) throws SQLException {
        checkIsOpen();
        delegate_.setDate(parameterName, x, cal);
    }


    /**
     * Sets the designated parameter to the given <code>java.sql.Time</code> value,
     * using the given <code>Calendar</code> object.  The driver uses
     * the <code>Calendar</code> object to construct an SQL <code>TIME</code> value,
     * which the driver then sends to the database.  With a
     * a <code>Calendar</code> object, the driver can calculate the time
     * taking into account a custom timezone.  If no
     * <code>Calendar</code> object is specified, the driver uses the default
     * timezone, which is that of the virtual machine running the application.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @param cal the <code>Calendar</code> object the driver will use
     *            to construct the time
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setTime(String parameterName, java.sql.Time x, Calendar cal) throws SQLException {
        checkIsOpen();
        delegate_.setTime(parameterName, x, cal);
    }


    /**
     * Sets the designated parameter to the given <code>java.sql.Timestamp</code> value,
     * using the given <code>Calendar</code> object.  The driver uses
     * the <code>Calendar</code> object to construct an SQL <code>TIMESTAMP</code> value,
     * which the driver then sends to the database.  With a
     * a <code>Calendar</code> object, the driver can calculate the timestamp
     * taking into account a custom timezone.  If no
     * <code>Calendar</code> object is specified, the driver uses the default
     * timezone, which is that of the virtual machine running the application.
     *
     * @param parameterName the name of the parameter
     * @param x the parameter value
     * @param cal the <code>Calendar</code> object the driver will use
     *            to construct the timestamp
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setTimestamp(String parameterName, java.sql.Timestamp x, Calendar cal) throws SQLException {
        checkIsOpen();
        delegate_.setTimestamp(parameterName, x, cal);
    }


    /**
     * Sets the designated parameter to SQL <code>NULL</code>.
     * This version of the method <code>setNull</code> should
     * be used for user-defined types and REF type parameters.  Examples
     * of user-defined types include: STRUCT, DISTINCT, JAVA_OBJECT, and
     * named array types.
     *
     * <P><B>Note:</B> To be portable, applications must give the
     * SQL type code and the fully-qualified SQL type name when specifying
     * a NULL user-defined or REF parameter.  In the case of a user-defined type
     * the name is the type name of the parameter itself.  For a REF
     * parameter, the name is the type name of the referenced type.  If
     * a JDBC driver does not need the type code or type name information,
     * it may ignore it.
     *
     * Although it is intended for user-defined and Ref parameters,
     * this method may be used to set a null parameter of any JDBC type.
     * If the parameter does not have a user-defined or REF type, the given
     * typeName is ignored.
     *
     *
     * @param parameterName the name of the parameter
     * @param sqlType a value from <code>java.sql.Types</code>
     * @param typeName the fully-qualified name of an SQL user-defined type;
     *        ignored if the parameter is not a user-defined type or
     *        SQL <code>REF</code> value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setNull (String parameterName, int sqlType, String typeName) throws SQLException {
        checkIsOpen();
        delegate_.setNull(parameterName, sqlType, typeName);
    }


    /**
     * Retrieves the value of a JDBC <code>CHAR</code>, <code>VARCHAR</code>,
     * or <code>LONGVARCHAR</code> parameter as a <code>String</code> in
     * the Java programming language.
     * <p>
     * For the fixed-length type JDBC <code>CHAR</code>,
     * the <code>String</code> object
     * returned has exactly the same value the JDBC
     * <code>CHAR</code> value had in the
     * database, including any padding added by the database.
     * @param parameterName the name of the parameter
     * @return the parameter value. If the value is SQL <code>NULL</code>, the result
     * is <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @see #setString
     * @since 1.4
     */
    public String getString(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getString(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>BIT</code> parameter as a
     * <code>boolean</code> in the Java programming language.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>, the result
     * is <code>false</code>.
     * @exception SQLException if a database access error occurs
     * @see #setBoolean
     * @since 1.4
     */
    public boolean getBoolean(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getBoolean(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>TINYINT</code> parameter as a <code>byte</code>
     * in the Java programming language.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>, the result
     * is <code>0</code>.
     * @exception SQLException if a database access error occurs
     * @see #setByte
     * @since 1.4
     */
    public byte getByte(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getByte(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>SMALLINT</code> parameter as a <code>short</code>
     * in the Java programming language.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>, the result
     * is <code>0</code>.
     * @exception SQLException if a database access error occurs
     * @see #setShort
     * @since 1.4
     */
    public short getShort(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getShort(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>INTEGER</code> parameter as an <code>int</code>
     * in the Java programming language.
     *
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>,
     *         the result is <code>0</code>.
     * @exception SQLException if a database access error occurs
     * @see #setInt
     * @since 1.4
     */
    public int getInt(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getInt(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>BIGINT</code> parameter as a <code>long</code>
     * in the Java programming language.
     *
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>,
     *         the result is <code>0</code>.
     * @exception SQLException if a database access error occurs
     * @see #setLong
     * @since 1.4
     */
    public long getLong(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getLong(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>FLOAT</code> parameter as a <code>float</code>
     * in the Java programming language.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>,
     *         the result is <code>0</code>.
     * @exception SQLException if a database access error occurs
     * @see #setFloat
     * @since 1.4
     */
    public float getFloat(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getFloat(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>DOUBLE</code> parameter as a <code>double</code>
     * in the Java programming language.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>,
     *         the result is <code>0</code>.
     * @exception SQLException if a database access error occurs
     * @see #setDouble
     * @since 1.4
     */
    public double getDouble(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getDouble(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>BINARY</code> or <code>VARBINARY</code>
     * parameter as an array of <code>byte</code> values in the Java
     * programming language.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>, the result is
     *  <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @see #setBytes
     * @since 1.4
     */
    public byte[] getBytes(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getBytes(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>DATE</code> parameter as a
     * <code>java.sql.Date</code> object.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>, the result
     * is <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public java.sql.Date getDate(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getDate(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>TIME</code> parameter as a
     * <code>java.sql.Time</code> object.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>, the result
     * is <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public java.sql.Time getTime(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getTime(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>TIMESTAMP</code> parameter as a
     * <code>java.sql.Timestamp</code> object.
     * @param parameterName the name of the parameter
     * @return the parameter value.  If the value is SQL <code>NULL</code>, the result
     * is <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public java.sql.Timestamp getTimestamp(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getTimestamp(parameterName);
    }


    /**
     * Retrieves the value of a parameter as an <code>Object</code> in the Java
     * programming language. If the value is an SQL <code>NULL</code>, the
     * driver returns a Java <code>null</code>.
     * <p>
     * This method returns a Java object whose type corresponds to the JDBC
     * type that was registered for this parameter using the method
     * <code>registerOutParameter</code>.  By registering the target JDBC
     * type as <code>java.sql.Types.OTHER</code>, this method can be used
     * to read database-specific abstract data types.
     * @param parameterName the name of the parameter
     * @return A <code>java.lang.Object</code> holding the OUT parameter value.
     * @exception SQLException if a database access error occurs
     * @see java.sql.Types
     * @since 1.4
     */
    public Object getObject(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getObject(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>NUMERIC</code> parameter as a
     * <code>java.math.BigDecimal</code> object with as many digits to the
     * right of the decimal point as the value contains.
     * @param parameterName the name of the parameter
     * @return the parameter value in full precision.  If the value is
     * SQL <code>NULL</code>, the result is <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @see #setBigDecimal
     * @since 1.4
     */
    public BigDecimal getBigDecimal(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getBigDecimal(parameterName);
    }


    /**
     * Returns an object representing the value of OUT parameter
     * <code>i</code> and uses <code>map</code> for the custom
     * mapping of the parameter value.
     * <p>
     * This method returns a Java object whose type corresponds to the
     * JDBC type that was registered for this parameter using the method
     * <code>registerOutParameter</code>.  By registering the target
     * JDBC type as <code>java.sql.Types.OTHER</code>, this method can
     * be used to read database-specific abstract data types.
     * @param parameterName the name of the parameter
     * @param map the mapping from SQL type names to Java classes
     * @return a <code>java.lang.Object</code> holding the OUT parameter value
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public Object getObject(String parameterName, java.util.Map map) throws SQLException {
        checkIsOpen();
        return delegate_.getObject(parameterName, map);
    }


    /**
     * Retrieves the value of a JDBC <code>REF(&lt;structured-type&gt;)</code>
     * parameter as a {@link Ref} object in the Java programming language.
     *
     * @param parameterName the name of the parameter
     * @return the parameter value as a <code>Ref</code> object in the
     *         Java programming language.  If the value was SQL <code>NULL</code>,
     *         the value <code>null</code> is returned.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public Ref getRef (String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getRef(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>BLOB</code> parameter as a
     * {@link Blob} object in the Java programming language.
     *
     * @param parameterName the name of the parameter
     * @return the parameter value as a <code>Blob</code> object in the
     *         Java programming language.  If the value was SQL <code>NULL</code>,
     *         the value <code>null</code> is returned.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public Blob getBlob (String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getBlob(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>CLOB</code> parameter as a
     * <code>Clob</code> object in the Java programming language.
     * @param parameterName the name of the parameter
     * @return the parameter value as a <code>Clob</code> object in the
     *         Java programming language.  If the value was SQL <code>NULL</code>,
     *         the value <code>null</code> is returned.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public Clob getClob (String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getClob(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>ARRAY</code> parameter as an
     * {@link Array} object in the Java programming language.
     *
     * @param parameterName the name of the parameter
     * @return the parameter value as an <code>Array</code> object in
     *         Java programming language.  If the value was SQL <code>NULL</code>,
     *         the value <code>null</code> is returned.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public Array getArray (String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getArray(parameterName);
    }


    /**
     * Retrieves the value of a JDBC <code>DATE</code> parameter as a
     * <code>java.sql.Date</code> object, using
     * the given <code>Calendar</code> object
     * to construct the date.
     * With a <code>Calendar</code> object, the driver
     * can calculate the date taking into account a custom timezone and locale.
     * If no <code>Calendar</code> object is specified, the driver uses the
     * default timezone and locale.
     *
     * @param parameterName the name of the parameter
     * @param cal the <code>Calendar</code> object the driver will use
     *            to construct the date
     * @return the parameter value.  If the value is SQL <code>NULL</code>,
     * the result is <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public java.sql.Date getDate(String parameterName, Calendar cal) throws SQLException {
        checkIsOpen();
        return delegate_.getDate(parameterName, cal);
    }


    /**
     * Retrieves the value of a JDBC <code>TIME</code> parameter as a
     * <code>java.sql.Time</code> object, using
     * the given <code>Calendar</code> object
     * to construct the time.
     * With a <code>Calendar</code> object, the driver
     * can calculate the time taking into account a custom timezone and locale.
     * If no <code>Calendar</code> object is specified, the driver uses the
     * default timezone and locale.
     *
     * @param parameterName the name of the parameter
     * @param cal the <code>Calendar</code> object the driver will use
     *            to construct the time
     * @return the parameter value; if the value is SQL <code>NULL</code>, the result is
     * <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public java.sql.Time getTime(String parameterName, Calendar cal) throws SQLException {
        checkIsOpen();
        return delegate_.getTime(parameterName, cal);
    }


    /**
     * Retrieves the value of a JDBC <code>TIMESTAMP</code> parameter as a
     * <code>java.sql.Timestamp</code> object, using
     * the given <code>Calendar</code> object to construct
     * the <code>Timestamp</code> object.
     * With a <code>Calendar</code> object, the driver
     * can calculate the timestamp taking into account a custom timezone and locale.
     * If no <code>Calendar</code> object is specified, the driver uses the
     * default timezone and locale.
     *
     *
     * @param parameterName the name of the parameter
     * @param cal the <code>Calendar</code> object the driver will use
     *            to construct the timestamp
     * @return the parameter value.  If the value is SQL <code>NULL</code>, the result is
     * <code>null</code>.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public java.sql.Timestamp getTimestamp(String parameterName, Calendar cal) throws SQLException {
        checkIsOpen();
        return delegate_.getTimestamp(parameterName, cal);
    }


    /**
     * Retrieves the value of a JDBC <code>DATALINK</code> parameter as a
     * <code>java.net.URL</code> object.
     *
     * @param parameterName the name of the parameter
     * @return the parameter value as a <code>java.net.URL</code> object in the
     * Java programming language.  If the value was SQL <code>NULL</code>, the
     * value <code>null</code> is returned.
     * @exception SQLException if a database access error occurs,
     *            or if there is a problem with the URL
     * @see #setURL
     * @since 1.4
     */
    public java.net.URL getURL(String parameterName) throws SQLException {
        checkIsOpen();
        return delegate_.getURL(parameterName);
    }


    /**
     * Registers the OUT parameter named
     * <code>parameterName</code> to the JDBC type
     * <code>sqlType</code>.  All OUT parameters must be registered
     * before a stored procedure is executed.
     * <p>
     * The JDBC type specified by <code>sqlType</code> for an OUT
     * parameter determines the Java type that must be used
     * in the <code>get</code> method to read the value of that parameter.
     * <p>
     * If the JDBC type expected to be returned to this output parameter
     * is specific to this particular database, <code>sqlType</code>
     * should be <code>java.sql.Types.OTHER</code>.  The method
     * getObject retrieves the value.
     * @param parameterName the name of the parameter
     * @param sqlType the JDBC type code defined by <code>java.sql.Types</code>.
     * If the parameter is of JDBC type <code>NUMERIC</code>
     * or <code>DECIMAL</code>, the version of
     * <code>registerOutParameter</code> that accepts a scale value
     * should be used.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     * @see java.sql.Types
     */
    public void registerOutParameter(String parameterName, int sqlType) throws SQLException {
        checkIsOpen();
        delegate_.registerOutParameter(parameterName, sqlType);
    }


    /**
     * Registers the parameter named
     * <code>parameterName</code> to be of JDBC type
     * <code>sqlType</code>.  This method must be called
     * before a stored procedure is executed.
     * <p>
     * The JDBC type specified by <code>sqlType</code> for an OUT
     * parameter determines the Java type that must be used
     * in the <code>get</code> method to read the value of that parameter.
     * <p>
     * This version of <code>registerOutParameter</code> should be
     * used when the parameter is of JDBC type <code>NUMERIC</code>
     * or <code>DECIMAL</code>.
     * @param parameterName the name of the parameter
     * @param sqlType SQL type code defined by <code>java.sql.Types</code>.
     * @param scale the desired number of digits to the right of the
     * decimal point.  It must be greater than or equal to zero.
     * @exception SQLException if a database access error occurs
     * @since 1.4
     * @see java.sql.Types
     */
    public void registerOutParameter(String parameterName, int sqlType, int scale) throws SQLException {
        checkIsOpen();
        delegate_.registerOutParameter(parameterName, sqlType, scale);
    }


    /**
     * Registers the designated output parameter.  This version of
     * the method <code>registerOutParameter</code>
     * should be used for a user-named or REF output parameter.  Examples
     * of user-named types include: STRUCT, DISTINCT, JAVA_OBJECT, and
     * named array types.
     *
     * Before executing a stored procedure call, you must explicitly
     * call <code>registerOutParameter</code> to register the type from
     * <code>java.sql.Types</code> for each
     * OUT parameter.  For a user-named parameter the fully-qualified SQL
     * type name of the parameter should also be given, while a REF
     * parameter requires that the fully-qualified type name of the
     * referenced type be given.  A JDBC driver that does not need the
     * type code and type name information may ignore it.   To be portable,
     * however, applications should always provide these values for
     * user-named and REF parameters.
     *
     * Although it is intended for user-named and REF parameters,
     * this method may be used to register a parameter of any JDBC type.
     * If the parameter does not have a user-named or REF type, the
     * typeName parameter is ignored.
     *
     * <P><B>Note:</B> When reading the value of an out parameter, you
     * must use the <code>getXXX</code> method whose Java type XXX corresponds to the
     * parameter's registered SQL type.
     *
     * @param parameterName the name of the parameter
     * @param sqlType a value from {@link java.sql.Types}
     * @param typeName the fully-qualified name of an SQL structured type
     * @exception SQLException if a database access error occurs
     * @see java.sql.Types
     * @since 1.4
     */
    public void registerOutParameter (String parameterName, int sqlType, String typeName) throws SQLException {
        checkIsOpen();
        delegate_.registerOutParameter(parameterName, sqlType, typeName);
    }
}

