/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import com.gargoylesoftware.base.resource.ManagedResource;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.Savepoint;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 *  A wrapper for java.sql.Connection objects.<p>
 *
 *  The contract with JDBC says that result sets must be closed before
 *  statements and statements must be closed before connections but java does
 *  not enforce this contract. It is quite possible to close a connection while
 *  statements and result sets are still open. While some database drivers
 *  handle this condition nicely, others will start failing in undefined ways
 *  when this happens.<p>
 *
 *  This wrapper class is a solution to this problem. If the connection is only
 *  accessed through the wrapper then the wrapper will ensure that everything is
 *  closed in the correct order. It will also ensure that the various jdbc
 *  objects (connections, statements, result sets and metadata) cannot be used
 *  after they are closed.<p>
 *
 *  This class was created for the {@link JDBCResourceFactory} but can be used
 *  by itself.
 *
 * @version  $Revision: 1.4 $
 * @author  <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public final class ConnectionWrapper implements Connection, ManagedResource {

    private Connection delegate_ = null;
    private boolean isOpen_ = true;
    private String resourceFactoryName_;

    private final List openStatements_ = Collections.synchronizedList(new ArrayList());
    private final List openDatabaseMetaData_ = Collections.synchronizedList(new ArrayList());


    /**
     *  Create a new connection wrapper
     *
     * @param  connection The connection that we are wrapping
     */
    public ConnectionWrapper( final Connection connection ) {
        if( connection == null ) {
            throw new NullPointerException();
        }
        delegate_ = connection;
    }


    /**
     *  Set the name of the factory that allocated this connection
     *
     * @param  name The name of the factory
     */
    public final void setResourceFactoryName( final String name ) {
        resourceFactoryName_ = name;
    }


    /**
     *  Sets this connection's auto-commit mode. If a connection is in
     *  auto-commit mode, then all its SQL statements will be executed and
     *  committed as individual transactions. Otherwise, its SQL statements are
     *  grouped into transactions that are terminated by a call to either the
     *  method <code>commit</code> or the method <code>rollback</code>. By
     *  default, new connections are in auto-commit mode. <p>
     *
     *  The commit occurs when the statement completes or the next execute
     *  occurs, whichever comes first. In the case of statements returning a
     *  ResultSet, the statement completes when the last row of the ResultSet
     *  has been retrieved or the ResultSet has been closed. In advanced cases,
     *  a single statement may return multiple results as well as output
     *  parameter values. In these cases the commit occurs when all results and
     *  output parameter values have been retrieved.
     *
     * @param  autoCommit true enables auto-commit; false disables auto-commit.
     * @exception  SQLException if a database access error occurs
     */
    public void setAutoCommit( boolean autoCommit )
        throws SQLException {
        checkConnection();
        delegate_.setAutoCommit( autoCommit );
    }


    /**
     *  Puts this connection in read-only mode as a hint to enable database
     *  optimizations. <P>
     *
     *  <B>Note:</B> This method cannot be called while in the middle of a
     *  transaction.
     *
     * @param  readOnly true enables read-only mode; false disables read-only
     *      mode.
     * @exception  SQLException if a database access error occurs
     */
    public void setReadOnly( boolean readOnly )
        throws SQLException {
        checkConnection();
        delegate_.setReadOnly( readOnly );
    }


    /**
     *  Sets a catalog name in order to select a subspace of this Connection's
     *  database in which to work. If the driver does not support catalogs, it
     *  will silently ignore this request.
     *
     * @param  catalog The catalog name
     * @exception  SQLException if a database access error occurs
     */
    public void setCatalog( String catalog )
        throws SQLException {
        checkConnection();
        delegate_.setCatalog( catalog );
    }


    /**
     *  Attempts to change the transaction isolation level to the one given. The
     *  constants defined in the interface <code>Connection</code> are the
     *  possible transaction isolation levels. <P>
     *
     *  <B>Note:</B> This method cannot be called while in the middle of a
     *  transaction.
     *
     * @param  level one of the TRANSACTION_* isolation values with the
     *      exception of TRANSACTION_NONE; some databases may not support other
     *      values
     * @exception  SQLException if a database access error occurs
     */
    public void setTransactionIsolation( int level )
        throws SQLException {
        checkConnection();
        delegate_.setTransactionIsolation( level );
    }


    /**
     *  Installs the given type map as the type map for this connection. The
     *  type map will be used for the custom mapping of SQL structured types and
     *  distinct types.
     *
     * @param  map the <code>java.util.Map</code> object to install as the
     *      replacement for this <code>Connection</code> object's default type
     *      map
     * @exception  SQLException If a sql error occurs
     * @since  1.2
     */
    public void setTypeMap( final Map map )
        throws SQLException {
        checkConnection();
        delegate_.setTypeMap( map );
    }


    /**
     *  Return the name of the factory that allocated this connection
     *
     * @return  The name of the factory
     */
    public final String getResourceFactoryName() {
        return resourceFactoryName_;
    }


    /**
     *  Return the wrapped connection
     *
     * @return  The wrapped connection
     */
    public final Connection getDelegate() {
        return delegate_;
    }


    /**
     *  Gets the current auto-commit state.
     *
     * @return  the current state of auto-commit mode
     * @exception  SQLException if a database access error occurs
     * @see  #setAutoCommit
     */
    public boolean getAutoCommit()
        throws SQLException {
        checkConnection();
        return delegate_.getAutoCommit();
    }


    /**
     *  Tests to see if a Connection is closed.
     *
     * @return  true if the connection is closed; false if it's still open
     * @exception  SQLException if a database access error occurs
     */
    public boolean isClosed()
        throws SQLException {

        if( delegate_ == null ) {
            return true;
        }

        return delegate_.isClosed();
    }

    //======================================================================
    // Advanced features:

    /**
     *  Gets the metadata regarding this connection's database. A Connection's
     *  database is able to provide information describing its tables, its
     *  supported SQL grammar, its stored procedures, the capabilities of this
     *  connection, and so on. This information is made available through a
     *  DatabaseMetaData object.
     *
     * @return  a DatabaseMetaData object for this Connection
     * @exception  SQLException if a database access error occurs
     */
    public DatabaseMetaData getMetaData()
        throws SQLException {
        checkConnection();
        final DatabaseMetaDataWrapper wrapper
                 = new DatabaseMetaDataWrapper( delegate_.getMetaData() );
        wrapper.setConnection( this );
        openDatabaseMetaData_.add( wrapper );
        return wrapper;
    }


    /**
     *  Tests to see if the connection is in read-only mode.
     *
     * @return  true if connection is read-only and false otherwise
     * @exception  SQLException if a database access error occurs
     */
    public boolean isReadOnly()
        throws SQLException {
        checkConnection();
        return delegate_.isReadOnly();
    }


    /**
     *  Returns the Connection's current catalog name.
     *
     * @return  the current catalog name or null
     * @exception  SQLException if a database access error occurs
     */
    public String getCatalog()
        throws SQLException {
        checkConnection();
        return delegate_.getCatalog();
    }


    /**
     *  Gets this Connection's current transaction isolation level.
     *
     * @return  the current TRANSACTION_* mode value
     * @exception  SQLException if a database access error occurs
     */
    public int getTransactionIsolation()
        throws SQLException {
        checkConnection();
        return delegate_.getTransactionIsolation();
    }


    /**
     *  Returns the first warning reported by calls on this Connection. <P>
     *
     *  <B>Note:</B> Subsequent warnings will be chained to this SQLWarning.
     *
     * @return  the first SQLWarning or null
     * @exception  SQLException if a database access error occurs
     */
    public SQLWarning getWarnings()
        throws SQLException {
        checkConnection();
        return delegate_.getWarnings();
    }


    /**
     *  Gets the type map object associated with this connection. Unless the
     *  application has added an entry to the type map, the map returned will be
     *  empty.
     *
     * @return  the <code>java.util.Map</code> object associated with this
     *      <code>Connection</code> object
     * @exception  SQLException If a sql error occurs
     * @since  1.2
     */
    public Map getTypeMap()
        throws SQLException {
        checkConnection();
        return delegate_.getTypeMap();
    }


    /**
     *  Creates a <code>Statement</code> object for sending SQL statements to
     *  the database. SQL statements without parameters are normally executed
     *  using Statement objects. If the same SQL statement is executed many
     *  times, it is more efficient to use a <code>PreparedStatement</code>
     *  object. <P>
     *
     *  Result sets created using the returned <code>Statement</code> object
     *  will by default have forward-only type and read-only concurrency.
     *
     * @return  a new Statement object
     * @exception  SQLException if a database access error occurs
     */
    public Statement createStatement()
        throws SQLException {
        checkConnection();
        final StatementWrapper wrapper = new StatementWrapper( delegate_.createStatement() );
        wrapper.setConnection( this );
        openStatements_.add( wrapper );
        return wrapper;
    }


    /**
     *  Creates a <code>PreparedStatement</code> object for sending
     *  parameterized SQL statements to the database. A SQL statement with or
     *  without IN parameters can be pre-compiled and stored in a
     *  PreparedStatement object. This object can then be used to efficiently
     *  execute this statement multiple times. <P>
     *
     *  <B>Note:</B> This method is optimized for handling parametric SQL
     *  statements that benefit from precompilation. If the driver supports
     *  precompilation, the method <code>prepareStatement</code> will send the
     *  statement to the database for precompilation. Some drivers may not
     *  support precompilation. In this case, the statement may not be sent to
     *  the database until the <code>PreparedStatement</code> is executed. This
     *  has no direct effect on users; however, it does affect which method
     *  throws certain SQLExceptions. <p>
     *
     *  Result sets created using the returned PreparedStatement will have
     *  forward-only type and read-only concurrency, by default.
     *
     * @param  sql a SQL statement that may contain one or more '?' IN parameter
     *      placeholders
     * @return  a new PreparedStatement object containing the pre-compiled
     *      statement
     * @exception  SQLException if a database access error occurs
     */
    public PreparedStatement prepareStatement( String sql )
        throws SQLException {
        checkConnection();
        final PreparedStatement statement = new PreparedStatementWrapper( delegate_.prepareStatement( sql ) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     *  Creates a <code>CallableStatement</code> object for calling database
     *  stored procedures. The <code>CallableStatement</code> object provides
     *  methods for setting up its IN and OUT parameters, and methods for
     *  executing the call to a stored procedure. <P>
     *
     *  <B>Note:</B> This method is optimized for handling stored procedure call
     *  statements. Some drivers may send the call statement to the database
     *  when the method <code>prepareCall</code> is done; others may wait until
     *  the <code>CallableStatement</code> object is executed. This has no
     *  direct effect on users; however, it does affect which method throws
     *  certain SQLExceptions. Result sets created using the returned
     *  CallableStatement will have forward-only type and read-only concurrency,
     *  by default.
     *
     * @param  sql a SQL statement that may contain one or more '?' parameter
     *      placeholders. Typically this statement is a JDBC function call
     *      escape string.
     * @return  a new CallableStatement object containing the pre-compiled SQL
     *      statement
     * @exception  SQLException if a database access error occurs
     */
    public CallableStatement prepareCall( String sql )
        throws SQLException {
        checkConnection();
        final CallableStatement statement
                 = new CallableStatementWrapper( delegate_.prepareCall( sql ) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     *  Converts the given SQL statement into the system's native SQL grammar. A
     *  driver may convert the JDBC sql grammar into its system's native SQL
     *  grammar prior to sending it; this method returns the native form of the
     *  statement that the driver would have sent.
     *
     * @param  sql a SQL statement that may contain one or more '?' parameter
     *      placeholders
     * @return  the native form of this statement
     * @exception  SQLException if a database access error occurs
     */
    public String nativeSQL( String sql )
        throws SQLException {
        checkConnection();
        return delegate_.nativeSQL( sql );
    }


    /**
     *  Makes all changes made since the previous commit/rollback permanent and
     *  releases any database locks currently held by the Connection. This
     *  method should be used only when auto-commit mode has been disabled.
     *
     * @exception  SQLException if a database access error occurs
     * @see  #setAutoCommit
     */
    public void commit()
        throws SQLException {
        checkConnection();
        delegate_.commit();
    }


    /**
     *  Drops all changes made since the previous commit/rollback and releases
     *  any database locks currently held by this Connection. This method should
     *  be used only when auto- commit has been disabled.
     *
     * @exception  SQLException if a database access error occurs
     * @see  #setAutoCommit
     */
    public void rollback()
        throws SQLException {
        checkConnection();
        delegate_.rollback();
    }


    /**
     *  Releases a Connection's database and JDBC resources immediately instead
     *  of waiting for them to be automatically released. <P>
     *
     *  <B>Note:</B> A Connection is automatically closed when it is garbage
     *  collected. Certain fatal errors also result in a closed Connection.
     *
     * @exception  SQLException if a database access error occurs
     */
    public void close()
        throws SQLException {
        if( isOpen_ ) {
            closeAnyOpenStatements();
            closeAnyOpenMetaDatas();

            delegate_.close();
            delegate_ = null;
        }
        else {
            throw new AlreadyClosedException( "Connection is closed" );
        }
        isOpen_ = false;
    }


    /**
     *  Close any open statements
     *
     * @exception  SQLException If an error occurs
     */
    public void closeAnyOpenStatements()
        throws SQLException {
        StatementWrapper statement;
        Iterator iterator = openStatements_.iterator();
        while( iterator.hasNext() ) {
            statement = ( StatementWrapper )iterator.next();
            if( statement.isClosed() == false ) {
                statement.close();
            }
        }
        openStatements_.clear();
    }


    /**
     *  Close any open DatabaseMetaData objects
     *
     * @exception  SQLException If an error occurs
     */
    public void closeAnyOpenMetaDatas()
        throws SQLException {
        final Iterator iterator = openDatabaseMetaData_.iterator();
        while( iterator.hasNext() ) {
            ( ( DatabaseMetaDataWrapper )iterator.next() ).close();
        }
        openDatabaseMetaData_.clear();
    }


    /**
     *  Clears all warnings reported for this <code>Connection</code> object.
     *  After a call to this method, the method <code>getWarnings</code> returns
     *  null until a new warning is reported for this Connection.
     *
     * @exception  SQLException if a database access error occurs
     */
    public void clearWarnings()
        throws SQLException {
        checkConnection();
        delegate_.clearWarnings();
    }


    //--------------------------JDBC 2.0-----------------------------

    /**
     *  Creates a <code>Statement</code> object that will generate <code>ResultSet</code>
     *  objects with the given type and concurrency. This method is the same as
     *  the <code>createStatement</code> method above, but it allows the default
     *  result set type and result set concurrency type to be overridden.
     *
     * @param  resultSetType a result set type; see ResultSet.TYPE_XXX
     * @param  resultSetConcurrency a concurrency type; see ResultSet.CONCUR_XXX
     * @return  a new Statement object
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public Statement createStatement( int resultSetType, int resultSetConcurrency )
        throws
            SQLException {

        checkConnection();
        final StatementWrapper wrapper
            = new StatementWrapper( delegate_.createStatement( resultSetType, resultSetConcurrency ) );
        openStatements_.add( wrapper );
        return wrapper;
    }


    /**
     *  Creates a <code>PreparedStatement</code> object that will generate
     *  <code>ResultSet</code> objects with the given type and concurrency. This
     *  method is the same as the <code>prepareStatement</code> method above,
     *  but it allows the default result set type and result set concurrency
     *  type to be overridden.
     *
     * @param  sql a SQL statement that may contain one or more '?' parameter
     *      placeholders. Typically this statement is a JDBC function call
     *      escape string.
     * @param  resultSetType a result set type; see ResultSet.TYPE_XXX
     * @param  resultSetConcurrency a concurrency type; see ResultSet.CONCUR_XXX
     * @return  a new PreparedStatement object containing the pre-compiled SQL
     *      statement
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public PreparedStatement prepareStatement(
            final String sql,
            final int resultSetType,
            final int resultSetConcurrency )
        throws
            SQLException {

        checkConnection();
        final PreparedStatement statement
            = new PreparedStatementWrapper(
                delegate_.prepareStatement( sql, resultSetType, resultSetConcurrency ) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     *  Creates a <code>CallableStatement</code> object that will generate
     *  <code>ResultSet</code> objects with the given type and concurrency. This
     *  method is the same as the <code>prepareCall</code> method above, but it
     *  allows the default result set type and result set concurrency type to be
     *  overridden.
     *
     * @param  sql a SQL statement that may contain one or more '?' parameter
     *      placeholders. Typically this statement is a JDBC function call
     *      escape string.
     * @param  resultSetType a result set type; see ResultSet.TYPE_XXX
     * @param  resultSetConcurrency a concurrency type; see ResultSet.CONCUR_XXX
     * @return  a new CallableStatement object containing the pre-compiled SQL
     *      statement
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public CallableStatement prepareCall( String sql, int resultSetType,
            int resultSetConcurrency )
        throws SQLException {
        checkConnection();
        final CallableStatement statement
                 = new CallableStatementWrapper( delegate_.prepareCall( sql, resultSetType, resultSetConcurrency ) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     *  Check to see if the connection is still open. If not, throw an
     *  exception.
     *
     * @exception  SQLException
     */
    private void checkConnection()
        throws SQLException {
        if( isOpen_ == false ) {
            throw new SQLException( "Methods cannot be invoked on a closed connection" );
        }
    }


    /**
     * Return the number of statements that are currently open.  Useful for debugging purposes.
     *
     * @return The number of statements that are currently open.
     */
    public int getOpenStatementCount() {
        synchronized(openStatements_) {
            int count = 0;
            final Iterator iterator = openStatements_.iterator();
            while( iterator.hasNext() ) {
                final StatementWrapper statement = (StatementWrapper)iterator.next();
                if( statement.isClosed() ) {
                    iterator.remove();
                }
                else {
                    count++;
                }
            }

            return count;
        }
    }


    /**
     * Changes the holdability of <code>ResultSet</code> objects
     * created using this <code>Connection</code> object to the given
     * holdability.
     *
     * @param holdability a <code>ResultSet</code> holdability constant; one of
     *        <code>ResultSet.HOLD_CURSORS_OVER_COMMIT</code> or
     *        <code>ResultSet.CLOSE_CURSORS_AT_COMMIT</code>
     * @throws SQLException if a database access occurs, the given parameter
     *         is not a <code>ResultSet</code> constant indicating holdability,
     *         or the given holdability is not supported
     * @see #getHoldability
     * @see java.sql.ResultSet
     * @since 1.4
     */
    public void setHoldability(int holdability) throws SQLException {
        checkConnection();
        delegate_.setHoldability(holdability);
    }


    /**
     * Retrieves the current holdability of <code>ResultSet</code> objects
     * created using this <code>Connection</code> object.
     *
     * @return the holdability, one of
     *        <code>ResultSet.HOLD_CURSORS_OVER_COMMIT</code> or
     *        <code>ResultSet.CLOSE_CURSORS_AT_COMMIT</code>
     * @throws SQLException if a database access occurs
     * @see #setHoldability
     * @see java.sql.ResultSet
     * @since 1.4
     */
    public int getHoldability() throws SQLException {
        checkConnection();
        return delegate_.getHoldability();
    }


    /**
     * Creates an unnamed savepoint in the current transaction and
     * returns the new <code>Savepoint</code> object that represents it.
     *
     * @return the new <code>Savepoint</code> object
     * @exception SQLException if a database access error occurs
     *            or this <code>Connection</code> object is currently in
     *            auto-commit mode
     * @see Savepoint
     * @since 1.4
     */
    public Savepoint setSavepoint() throws SQLException {
        checkConnection();
        return delegate_.setSavepoint();
    }


    /**
     * Creates a savepoint with the given name in the current transaction
     * and returns the new <code>Savepoint</code> object that represents it.
     *
     * @param name a <code>String</code> containing the name of the savepoint
     * @return the new <code>Savepoint</code> object
     * @exception SQLException if a database access error occurs
     *            or this <code>Connection</code> object is currently in
     *            auto-commit mode
     * @see Savepoint
     * @since 1.4
     */
    public Savepoint setSavepoint(String name) throws SQLException {
        checkConnection();
        return delegate_.setSavepoint(name);
    }


    /**
     * Undoes all changes made after the given <code>Savepoint</code> object
     * was set.
     * <P>
     * This method should be used only when auto-commit has been disabled.
     *
     * @param savepoint the <code>Savepoint</code> object to roll back to
     * @exception SQLException if a database access error occurs,
     *            the <code>Savepoint</code> object is no longer valid,
     *            or this <code>Connection</code> object is currently in
     *            auto-commit mode
     * @see Savepoint
     * @since 1.4
     */
    public void rollback(Savepoint savepoint) throws SQLException {
        checkConnection();
        delegate_.rollback(savepoint);
    }


    /**
     * Removes the given <code>Savepoint</code> object from the current
     * transaction. Any reference to the savepoint after it have been removed
     * will cause an <code>SQLException</code> to be thrown.
     *
     * @param savepoint the <code>Savepoint</code> object to be removed
     * @exception SQLException if a database access error occurs or
     *            the given <code>Savepoint</code> object is not a valid
     *            savepoint in the current transaction
     * @since 1.4
     */
    public void releaseSavepoint(Savepoint savepoint) throws SQLException {
        checkConnection();
        delegate_.releaseSavepoint(savepoint);
    }


    /**
     * Creates a <code>Statement</code> object that will generate
     * <code>ResultSet</code> objects with the given type, concurrency,
     * and holdability.
     * This method is the same as the <code>createStatement</code> method
     * above, but it allows the default result set
     * type, concurrency, and holdability to be overridden.
     *
     * @param resultSetType one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.TYPE_FORWARD_ONLY</code>,
     *         <code>ResultSet.TYPE_SCROLL_INSENSITIVE</code>, or
     *         <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
     * @param resultSetConcurrency one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.CONCUR_READ_ONLY</code> or
     *         <code>ResultSet.CONCUR_UPDATABLE</code>
     * @param resultSetHoldability one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.HOLD_CURSORS_OVER_COMMIT</code> or
     *         <code>ResultSet.CLOSE_CURSORS_AT_COMMIT</code>
     * @return a new <code>Statement</code> object that will generate
     *         <code>ResultSet</code> objects with the given type,
     *         concurrency, and holdability
     * @exception SQLException if a database access error occurs
     *            or the given parameters are not <code>ResultSet</code>
     *            constants indicating type, concurrency, and holdability
     * @see java.sql.ResultSet
     * @since 1.4
     */
    public Statement createStatement(int resultSetType, int resultSetConcurrency,
        int resultSetHoldability) throws SQLException {
        checkConnection();

        final Statement statement = new StatementWrapper(
            delegate_.createStatement( resultSetType, resultSetConcurrency, resultSetHoldability ) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     * Creates a <code>PreparedStatement</code> object that will generate
     * <code>ResultSet</code> objects with the given type, concurrency,
     * and holdability.
     * <P>
     * This method is the same as the <code>prepareStatement</code> method
     * above, but it allows the default result set
     * type, concurrency, and holdability to be overridden.
     *
     * @param sql a <code>String</code> object that is the SQL statement to
     *            be sent to the database; may contain one or more ? IN
     *            parameters
     * @param resultSetType one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.TYPE_FORWARD_ONLY</code>,
     *         <code>ResultSet.TYPE_SCROLL_INSENSITIVE</code>, or
     *         <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
     * @param resultSetConcurrency one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.CONCUR_READ_ONLY</code> or
     *         <code>ResultSet.CONCUR_UPDATABLE</code>
     * @param resultSetHoldability one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.HOLD_CURSORS_OVER_COMMIT</code> or
     *         <code>ResultSet.CLOSE_CURSORS_AT_COMMIT</code>
     * @return a new <code>PreparedStatement</code> object, containing the
     *         pre-compiled SQL statement, that will generate
     *         <code>ResultSet</code> objects with the given type,
     *         concurrency, and holdability
     * @exception SQLException if a database access error occurs
     *            or the given parameters are not <code>ResultSet</code>
     *            constants indicating type, concurrency, and holdability
     * @see java.sql.ResultSet
     * @since 1.4
     */
    public PreparedStatement prepareStatement(String sql, int resultSetType,
        int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        checkConnection();

        final PreparedStatement statement = new PreparedStatementWrapper(
            delegate_.prepareStatement( sql, resultSetType, resultSetConcurrency, resultSetHoldability ) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     * Creates a <code>CallableStatement</code> object that will generate
     * <code>ResultSet</code> objects with the given type and concurrency.
     * This method is the same as the <code>prepareCall</code> method
     * above, but it allows the default result set
     * type, result set concurrency type and holdability to be overridden.
     *
     * @param sql a <code>String</code> object that is the SQL statement to
     *            be sent to the database; may contain on or more ? parameters
     * @param resultSetType one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.TYPE_FORWARD_ONLY</code>,
     *         <code>ResultSet.TYPE_SCROLL_INSENSITIVE</code>, or
     *         <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
     * @param resultSetConcurrency one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.CONCUR_READ_ONLY</code> or
     *         <code>ResultSet.CONCUR_UPDATABLE</code>
     * @param resultSetHoldability one of the following <code>ResultSet</code>
     *        constants:
     *         <code>ResultSet.HOLD_CURSORS_OVER_COMMIT</code> or
     *         <code>ResultSet.CLOSE_CURSORS_AT_COMMIT</code>
     * @return a new <code>CallableStatement</code> object, containing the
     *         pre-compiled SQL statement, that will generate
     *         <code>ResultSet</code> objects with the given type,
     *         concurrency, and holdability
     * @exception SQLException if a database access error occurs
     *            or the given parameters are not <code>ResultSet</code>
     *            constants indicating type, concurrency, and holdability
     * @see java.sql.ResultSet
     * @since 1.4
     */
    public CallableStatement prepareCall(String sql, int resultSetType,
        int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        checkConnection();

        final CallableStatement statement = new CallableStatementWrapper(
            delegate_.prepareCall( sql, resultSetType, resultSetConcurrency, resultSetHoldability) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     * Creates a default <code>PreparedStatement</code> object that has
     * the capability to retrieve auto-generated keys. The given constant
     * tells the driver whether it should make auto-generated keys
     * available for retrieval.  This parameter is ignored if the SQL
     * statement is not an <code>INSERT</code> statement.
     * <P>
     * <B>Note:</B> This method is optimized for handling
     * parametric SQL statements that benefit from precompilation. If
     * the driver supports precompilation,
     * the method <code>prepareStatement</code> will send
     * the statement to the database for precompilation. Some drivers
     * may not support precompilation. In this case, the statement may
     * not be sent to the database until the <code>PreparedStatement</code>
     * object is executed.  This has no direct effect on users; however, it does
     * affect which methods throw certain SQLExceptions.
     * <P>
     * Result sets created using the returned <code>PreparedStatement</code>
     * object will by default be type <code>TYPE_FORWARD_ONLY</code>
     * and have a concurrency level of <code>CONCUR_READ_ONLY</code>.
     *
     * @param sql an SQL statement that may contain one or more '?' IN
     *        parameter placeholders
     * @param autoGeneratedKeys a flag indicating whether auto-generated keys
     *        should be returned; one of
     *        <code>Statement.RETURN_GENERATED_KEYS</code> or
     *        <code>Statement.NO_GENERATED_KEYS</code>
     * @return a new <code>PreparedStatement</code> object, containing the
     *         pre-compiled SQL statement, that will have the capability of
     *         returning auto-generated keys
     * @exception SQLException if a database access error occurs
     *         or the given parameter is not a <code>Statement</code>
     *         constant indicating whether auto-generated keys should be
     *         returned
     * @since 1.4
     */
    public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
        checkConnection();

       final PreparedStatement statement
            = new PreparedStatementWrapper( delegate_.prepareStatement( sql, autoGeneratedKeys ) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     * Creates a default <code>PreparedStatement</code> object capable
     * of returning the auto-generated keys designated by the given array.
     * This array contains the indexes of the columns in the target
     * table that contain the auto-generated keys that should be made
     * available. This array is ignored if the SQL
     * statement is not an <code>INSERT</code> statement.
     * <P>
     * An SQL statement with or without IN parameters can be
     * pre-compiled and stored in a <code>PreparedStatement</code> object. This
     * object can then be used to efficiently execute this statement
     * multiple times.
     * <P>
     * <B>Note:</B> This method is optimized for handling
     * parametric SQL statements that benefit from precompilation. If
     * the driver supports precompilation,
     * the method <code>prepareStatement</code> will send
     * the statement to the database for precompilation. Some drivers
     * may not support precompilation. In this case, the statement may
     * not be sent to the database until the <code>PreparedStatement</code>
     * object is executed.  This has no direct effect on users; however, it does
     * affect which methods throw certain SQLExceptions.
     * <P>
     * Result sets created using the returned <code>PreparedStatement</code>
     * object will by default be type <code>TYPE_FORWARD_ONLY</code>
     * and have a concurrency level of <code>CONCUR_READ_ONLY</code>.
     *
     * @param sql an SQL statement that may contain one or more '?' IN
     *        parameter placeholders
     * @param columnIndexes an array of column indexes indicating the columns
     *        that should be returned from the inserted row or rows
     * @return a new <code>PreparedStatement</code> object, containing the
     *         pre-compiled statement, that is capable of returning the
     *         auto-generated keys designated by the given array of column
     *         indexes
     * @exception SQLException if a database access error occurs
     *
     * @since 1.4
     */
    public PreparedStatement prepareStatement(String sql, int columnIndexes[]) throws SQLException {
        checkConnection();

       final PreparedStatement statement
            = new PreparedStatementWrapper( delegate_.prepareStatement( sql, columnIndexes ) );
        openStatements_.add( statement );
        return statement;
    }


    /**
     * Creates a default <code>PreparedStatement</code> object capable
     * of returning the auto-generated keys designated by the given array.
     * This array contains the names of the columns in the target
     * table that contain the auto-generated keys that should be returned.
     * This array is ignored if the SQL
     * statement is not an <code>INSERT</code> statement.
     * <P>
     * An SQL statement with or without IN parameters can be
     * pre-compiled and stored in a <code>PreparedStatement</code> object. This
     * object can then be used to efficiently execute this statement
     * multiple times.
     * <P>
     * <B>Note:</B> This method is optimized for handling
     * parametric SQL statements that benefit from precompilation. If
     * the driver supports precompilation,
     * the method <code>prepareStatement</code> will send
     * the statement to the database for precompilation. Some drivers
     * may not support precompilation. In this case, the statement may
     * not be sent to the database until the <code>PreparedStatement</code>
     * object is executed.  This has no direct effect on users; however, it does
     * affect which methods throw certain SQLExceptions.
     * <P>
     * Result sets created using the returned <code>PreparedStatement</code>
     * object will by default be type <code>TYPE_FORWARD_ONLY</code>
     * and have a concurrency level of <code>CONCUR_READ_ONLY</code>.
     *
     * @param sql an SQL statement that may contain one or more '?' IN
     *        parameter placeholders
     * @param columnNames an array of column names indicating the columns
     *        that should be returned from the inserted row or rows
     * @return a new <code>PreparedStatement</code> object, containing the
     *         pre-compiled statement, that is capable of returning the
     *         auto-generated keys designated by the given array of column
     *         names
     * @exception SQLException if a database access error occurs
     *
     * @since 1.4
     */
    public PreparedStatement prepareStatement(String sql, String columnNames[]) throws SQLException {
        checkConnection();

       final PreparedStatement statement
            = new PreparedStatementWrapper( delegate_.prepareStatement( sql, columnNames ) );
        openStatements_.add( statement );
        return statement;
    }
}

