/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.ParameterMetaData;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;

/**
 *  A wrapper for a PreparedStatement
 *
 * @version  $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */

public class PreparedStatementWrapper
         extends
        StatementWrapper
         implements
        PreparedStatement {

    /**
     */
    private final PreparedStatement delegate_;


    /**
     *  Create a PreparedStatementWrapper
     *
     * @param  statement The PreparedStatement that we are wrapping
     */
    public PreparedStatementWrapper( final PreparedStatement statement ) {
        super( statement );
        delegate_ = statement;
    }


    /**
     *  Sets the designated parameter to SQL <code>NULL</code>. <P>
     *
     *  <B>Note:</B> You must specify the parameter's SQL type.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  sqlType the SQL type code defined in <code>java.sql.Types</code>
     * @exception  SQLException if a database access error occurs
     */
    public final void setNull( int parameterIndex, int sqlType )
        throws SQLException {
        checkIsOpen();
        delegate_.setNull( parameterIndex, sqlType );
    }


    /**
     *  Sets the designated parameter to a Java <code>boolean</code> value. The
     *  driver converts this to an SQL <code>BIT</code> value when it sends it
     *  to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setBoolean( int parameterIndex, boolean x )
        throws SQLException {
        checkIsOpen();
        delegate_.setBoolean( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a Java <code>byte</code> value. The
     *  driver converts this to an SQL <code>TINYINT</code> value when it sends
     *  it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setByte( int parameterIndex, byte x )
        throws SQLException {
        checkIsOpen();
        delegate_.setByte( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a Java <code>short</code> value. The
     *  driver converts this to an SQL <code>SMALLINT</code> value when it sends
     *  it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setShort( int parameterIndex, short x )
        throws SQLException {
        checkIsOpen();
        delegate_.setShort( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a Java <code>int</code> value. The
     *  driver converts this to an SQL <code>INTEGER</code> value when it sends
     *  it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setInt( int parameterIndex, int x )
        throws SQLException {
        checkIsOpen();
        delegate_.setInt( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a Java <code>long</code> value. The
     *  driver converts this to an SQL <code>BIGINT</code> value when it sends
     *  it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setLong( int parameterIndex, long x )
        throws SQLException {
        checkIsOpen();
        delegate_.setLong( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a Java <code>float</code> value. The
     *  driver converts this to an SQL <code>FLOAT</code> value when it sends it
     *  to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setFloat( int parameterIndex, float x )
        throws SQLException {
        checkIsOpen();
        delegate_.setFloat( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a Java <code>double</code> value. The
     *  driver converts this to an SQL <code>DOUBLE</code> value when it sends
     *  it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setDouble( int parameterIndex, double x )
        throws SQLException {
        checkIsOpen();
        delegate_.setDouble( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a <code>java.math.BigDecimal</code>
     *  value. The driver converts this to an SQL <code>NUMERIC</code> value
     *  when it sends it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setBigDecimal( int parameterIndex, BigDecimal x )
        throws SQLException {
        checkIsOpen();
        delegate_.setBigDecimal( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a Java <code>String</code> value. The
     *  driver converts this to an SQL <code>VARCHAR</code> or <code>LONGVARCHAR</code>
     *  value (depending on the argument's size relative to the driver's limits
     *  on <code>VARCHAR</code> values) when it sends it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setString( int parameterIndex, String x )
        throws SQLException {
        checkIsOpen();
        delegate_.setString( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a Java array of bytes. The driver
     *  converts this to an SQL <code>VARBINARY</code> or <code>LONGVARBINARY</code>
     *  (depending on the argument's size relative to the driver's limits on
     *  <code>VARBINARY</code> values) when it sends it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setBytes( int parameterIndex, byte x[] )
        throws SQLException {
        checkIsOpen();
        delegate_.setBytes( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a <code<java.sql.Date</code> value. The
     *  driver converts this to an SQL <code>DATE</code> value when it sends it
     *  to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setDate( int parameterIndex, Date x )
        throws SQLException {
        checkIsOpen();
        delegate_.setDate( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a <code>java.sql.Time</code> value. The
     *  driver converts this to an SQL <code>TIME</code> value when it sends it
     *  to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setTime( int parameterIndex, java.sql.Time x )
        throws SQLException {
        checkIsOpen();
        delegate_.setTime( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to a <code>java.sql.Timestamp</code>
     *  value. The driver converts this to an SQL <code>TIMESTAMP</code> value
     *  when it sends it to the database.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setTimestamp( int parameterIndex, Timestamp x )
        throws SQLException {
        checkIsOpen();
        delegate_.setTimestamp( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to the given input stream, which will have
     *  the specified number of bytes. When a very large ASCII value is input to
     *  a <code>LONGVARCHAR</code> parameter, it may be more practical to send
     *  it via a <code>java.io.InputStream</code>. Data will be read from the
     *  stream as needed until end-of-file is reached. The JDBC driver will do
     *  any necessary conversion from ASCII to the database char format. <P>
     *
     *  <B>Note:</B> This stream object can either be a standard Java stream
     *  object or your own subclass that implements the standard interface.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the Java input stream that contains the ASCII parameter value
     * @param  length the number of bytes in the stream
     * @exception  SQLException if a database access error occurs
     */
    public final void setAsciiStream( int parameterIndex, InputStream x, int length )
        throws SQLException {
        checkIsOpen();
        delegate_.setAsciiStream( parameterIndex, x, length );
    }


    /**
     *  Sets the designated parameter to the given input stream, which will have
     *  the specified number of bytes. When a very large UNICODE value is input
     *  to a <code>LONGVARCHAR</code> parameter, it may be more practical to
     *  send it via a <code>java.io.InputStream</code> object. The data will be
     *  read from the stream as needed until end-of-file is reached. The JDBC
     *  driver will do any necessary conversion from UNICODE to the database
     *  char format. The byte format of the Unicode stream must be Java UTF-8,
     *  as defined in the Java Virtual Machine Specification. <P>
     *
     *  <B>Note:</B> This stream object can either be a standard Java stream
     *  object or your own subclass that implements the standard interface.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the java input stream which contains the UNICODE parameter
     *      value
     * @param  length the number of bytes in the stream
     * @exception  SQLException if a database access error occurs
     * @deprecated
     */
    public final void setUnicodeStream( int parameterIndex, InputStream x,
            int length )
        throws SQLException {
        checkIsOpen();
        delegate_.setUnicodeStream( parameterIndex, x, length );
    }


    /**
     *  Sets the designated parameter to the given input stream, which will have
     *  the specified number of bytes. When a very large binary value is input
     *  to a <code>LONGVARBINARY</code> parameter, it may be more practical to
     *  send it via a <code>java.io.InputStream</code> object. The data will be
     *  read from the stream as needed until end-of-file is reached. <P>
     *
     *  <B>Note:</B> This stream object can either be a standard Java stream
     *  object or your own subclass that implements the standard interface.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the java input stream which contains the binary parameter value
     * @param  length the number of bytes in the stream
     * @exception  SQLException if a database access error occurs
     */
    public final void setBinaryStream( int parameterIndex, InputStream x,
            int length )
        throws SQLException {
        checkIsOpen();
        delegate_.setBinaryStream( parameterIndex, x, length );
    }

    //----------------------------------------------------------------------
    // Advanced features:

    /**
     *  <p>
     *
     *  Sets the value of the designated parameter with the given object. The
     *  second argument must be an object type; for integral values, the <code>java.lang</code>
     *  equivalent objects should be used. <p>
     *
     *  The given Java object will be converted to the given targetSqlType
     *  before being sent to the database. If the object has a custom mapping
     *  (is of a class implementing the interface <code>SQLData</code>), the
     *  JDBC driver should call the method <code>SQLData.writeSQL</code> to
     *  write it to the SQL data stream. If, on the other hand, the object is of
     *  a class implementing Ref, Blob, Clob, Struct, or Array, the driver
     *  should pass it to the database as a value of the corresponding SQL type.
     *  <p>
     *
     *  Note that this method may be used to pass datatabase- specific abstract
     *  data types.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the object containing the input parameter value
     * @param  targetSqlType the SQL type (as defined in java.sql.Types) to be
     *      sent to the database. The scale argument may further qualify this
     *      type.
     * @param  scale for java.sql.Types.DECIMAL or java.sql.Types.NUMERIC types,
     *      this is the number of digits after the decimal point. For all other
     *      types, this value will be ignored.
     * @exception  SQLException if a database access error occurs
     */
    public final void setObject( int parameterIndex, Object x, int targetSqlType, int scale )
        throws SQLException {
        checkIsOpen();
        delegate_.setObject( parameterIndex, x, targetSqlType, scale );
    }


    /**
     *  Sets the value of the designated parameter with the given object. This
     *  method is like the method <code>setObject</code> above, except that it
     *  assumes a scale of zero.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the object containing the input parameter value
     * @param  targetSqlType the SQL type (as defined in java.sql.Types) to be
     *      sent to the database
     * @exception  SQLException if a database access error occurs
     */
    public final void setObject( int parameterIndex, Object x, int targetSqlType )
        throws SQLException {
        checkIsOpen();
        delegate_.setObject( parameterIndex, x, targetSqlType );
    }


    /**
     *  <p>
     *
     *  Sets the value of the designated parameter using the given object. The
     *  second parameter must be of type <code>Object</code>; therefore, the
     *  <code>java.lang</code> equivalent objects should be used for built-in
     *  types. <p>
     *
     *  The JDBC specification specifies a standard mapping from Java <code>Object</code>
     *  types to SQL types. The given argument will be converted to the
     *  corresponding SQL type before being sent to the database. <p>
     *
     *  Note that this method may be used to pass datatabase- specific abstract
     *  data types, by using a driver-specific Java type. If the object is of a
     *  class implementing the interface <code>SQLData</code>, the JDBC driver
     *  should call the method <code>SQLData.writeSQL</code> to write it to the
     *  SQL data stream. If, on the other hand, the object is of a class
     *  implementing Ref, Blob, Clob, Struct, or Array, then the driver should
     *  pass it to the database as a value of the corresponding SQL type. This
     *  method throws an exception if there is an ambiguity, for example, if the
     *  object is of a class implementing more than one of the interfaces named
     *  above.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the object containing the input parameter value
     * @exception  SQLException if a database access error occurs
     */
    public final void setObject( int parameterIndex, Object x )
        throws SQLException {
        checkIsOpen();
        delegate_.setObject( parameterIndex, x );
    }


    /**
     *  Sets the designated parameter to the given <code>Reader</code> object,
     *  which is the given number of characters long. When a very large UNICODE
     *  value is input to a <code>LONGVARCHAR</code> parameter, it may be more
     *  practical to send it via a <code>java.io.Reader</code> object. The data
     *  will be read from the stream as needed until end-of-file is reached. The
     *  JDBC driver will do any necessary conversion from UNICODE to the
     *  database char format. <P>
     *
     *  <B>Note:</B> This stream object can either be a standard Java stream
     *  object or your own subclass that implements the standard interface.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  reader the java reader which contains the UNICODE data
     * @param  length the number of characters in the stream
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setCharacterStream( int parameterIndex, Reader reader,
            int length )
        throws SQLException {
        checkIsOpen();
        delegate_.setCharacterStream( parameterIndex, reader, length );
    }


    /**
     *  Sets the designated parameter to the given <code>REF(&lt;structured-type&gt;)</code>
     *  value.
     *
     * @param  i the first parameter is 1, the second is 2, ...
     * @param  x an SQL <code>REF</code> value
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setRef( int i, Ref x )
        throws SQLException {
        checkIsOpen();
        delegate_.setRef( i, x );
    }


    /**
     *  Sets the designated parameter to the given <code>Blob</code> object.
     *
     * @param  i the first parameter is 1, the second is 2, ...
     * @param  x a <code>Blob</code> object that maps an SQL <code>BLOB</code>
     *      value
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setBlob( int i, Blob x )
        throws SQLException {
        checkIsOpen();
        delegate_.setBlob( i, x );
    }


    /**
     *  Sets the designated parameter to the given <code>Clob</code> object.
     *
     * @param  i the first parameter is 1, the second is 2, ...
     * @param  x a <code>Clob</code> object that maps an SQL <code>CLOB</code>
     *      value
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setClob( int i, Clob x )
        throws SQLException {
        checkIsOpen();
        delegate_.setClob( i, x );
    }


    /**
     *  Sets the designated parameter to the given <code>Array</code> object.
     *  Sets an Array parameter.
     *
     * @param  i the first parameter is 1, the second is 2, ...
     * @param  x an <code>Array</code> object that maps an SQL <code>ARRAY</code>
     *      value
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setArray( int i, Array x )
        throws SQLException {
        checkIsOpen();
        delegate_.setArray( i, x );
    }


    /**
     *  Sets the designated parameter to the given <code>java.sql.Date</code>
     *  value, using the given <code>Calendar</code> object. The driver uses the
     *  <code>Calendar</code> object to construct an SQL <code>DATE</code>
     *  value, which the driver then sends to the database. With a a <code>Calendar</code>
     *  object, the driver can calculate the date taking into account a custom
     *  timezone. If no <code>Calendar</code> object is specified, the driver
     *  uses the default timezone, which is that of the virtual machine running
     *  the application.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @param  cal the <code>Calendar</code> object the driver will use to
     *      construct the date
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setDate( int parameterIndex, Date x, Calendar cal )
        throws SQLException {
        checkIsOpen();
        delegate_.setDate( parameterIndex, x, cal );
    }


    /**
     *  Sets the designated parameter to the given <code>java.sql.Time</code>
     *  value, using the given <code>Calendar</code> object. The driver uses the
     *  <code>Calendar</code> object to construct an SQL <code>TIME</code>
     *  value, which the driver then sends to the database. With a a <code>Calendar</code>
     *  object, the driver can calculate the time taking into account a custom
     *  timezone. If no <code>Calendar</code> object is specified, the driver
     *  uses the default timezone, which is that of the virtual machine running
     *  the application.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @param  cal the <code>Calendar</code> object the driver will use to
     *      construct the time
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setTime( int parameterIndex, Time x, Calendar cal )
        throws SQLException {
        checkIsOpen();
        delegate_.setTime( parameterIndex, x, cal );
    }


    /**
     *  Sets the designated parameter to the given <code>java.sql.Timestamp</code>
     *  value, using the given <code>Calendar</code> object. The driver uses the
     *  <code>Calendar</code> object to construct an SQL <code>TIMESTAMP</code>
     *  value, which the driver then sends to the database. With a a <code>Calendar</code>
     *  object, the driver can calculate the timestamp taking into account a
     *  custom timezone. If no <code>Calendar</code> object is specified, the
     *  driver uses the default timezone, which is that of the virtual machine
     *  running the application.
     *
     * @param  parameterIndex the first parameter is 1, the second is 2, ...
     * @param  x the parameter value
     * @param  cal the <code>Calendar</code> object the driver will use to
     *      construct the timestamp
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setTimestamp( int parameterIndex, Timestamp x, Calendar cal )
        throws SQLException {
        checkIsOpen();
        delegate_.setTimestamp( parameterIndex, x, cal );
    }


    /**
     *  Sets the designated parameter to SQL <code>NULL</code>. This version of
     *  the method <code>setNull</code> should be used for user-defined types
     *  and REF type parameters. Examples of user-defined types include: STRUCT,
     *  DISTINCT, JAVA_OBJECT, and named array types. <P>
     *
     *  <B>Note:</B> To be portable, applications must give the SQL type code
     *  and the fully-qualified SQL type name when specifying a NULL
     *  user-defined or REF parameter. In the case of a user-defined type the
     *  name is the type name of the parameter itself. For a REF parameter, the
     *  name is the type name of the referenced type. If a JDBC driver does not
     *  need the type code or type name information, it may ignore it. Although
     *  it is intended for user-defined and Ref parameters, this method may be
     *  used to set a null parameter of any JDBC type. If the parameter does not
     *  have a user-defined or REF type, the given typeName is ignored.
     *
     * @param  paramIndex the first parameter is 1, the second is 2, ...
     * @param  sqlType a value from <code>java.sql.Types</code>
     * @param  typeName the fully-qualified name of an SQL user-defined type;
     *      ignored if the parameter is not a user-defined type or REF
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final void setNull( int paramIndex, int sqlType, String typeName )
        throws SQLException {
        checkIsOpen();
        delegate_.setNull( paramIndex, sqlType, typeName );
    }


    /**
     *  Gets the number, types and properties of a <code>ResultSet</code>
     *  object's columns.
     *
     * @return  the description of a <code>ResultSet</code> object's columns
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final ResultSetMetaData getMetaData()
        throws SQLException {
        checkIsOpen();
        return delegate_.getMetaData();
    }


    /**
     *  Executes the SQL query in this <code>PreparedStatement</code> object and
     *  returns the result set generated by the query.
     *
     * @return  a <code>ResultSet</code> object that contains the data produced
     *      by the query; never <code>null</code>
     * @exception  SQLException if a database access error occurs
     */
    public final ResultSet executeQuery()
        throws SQLException {
        checkIsOpen();
        return wrapResultSet( delegate_.executeQuery() );
    }


    /**
     *  Executes the SQL INSERT, UPDATE or DELETE statement in this <code>PreparedStatement</code>
     *  object. In addition, SQL statements that return nothing, such as SQL DDL
     *  statements, can be executed.
     *
     * @return  either the row count for INSERT, UPDATE or DELETE statements; or
     *      0 for SQL statements that return nothing
     * @exception  SQLException if a database access error occurs
     */
    public final int executeUpdate()
        throws SQLException {
        checkIsOpen();
        return delegate_.executeUpdate();
    }


    /**
     *  Clears the current parameter values immediately. <P>
     *
     *  In general, parameter values remain in force for repeated use of a
     *  statement. Setting a parameter value automatically clears its previous
     *  value. However, in some cases it is useful to immediately release the
     *  resources used by the current parameter values; this can be done by
     *  calling the method <code>clearParameters</code>.
     *
     * @exception  SQLException if a database access error occurs
     */
    public final void clearParameters()
        throws SQLException {
        checkIsOpen();
        delegate_.clearParameters();
    }


    /**
     *  Executes any kind of SQL statement. Some prepared statements return
     *  multiple results; the <code>execute</code> method handles these complex
     *  statements as well as the simpler form of statements handled by the
     *  methods <code>executeQuery</code> and <code>executeUpdate</code>.
     *
     * @return  Unknown
     * @exception  SQLException if a database access error occurs
     */
    public final boolean execute()
        throws SQLException {
        checkIsOpen();
        return delegate_.execute();
    }

    //--------------------------JDBC 2.0-----------------------------

    /**
     *  Adds a set of parameters to this <code>PreparedStatement</code> object's
     *  batch of commands.
     *
     * @exception  SQLException if a database access error occurs
     */
    public final void addBatch()
        throws SQLException {
        checkIsOpen();
        delegate_.addBatch();
    }


    /**
     * Sets the designated parameter to the given <code>java.net.URL</code> value.
     * The driver converts this to an SQL <code>DATALINK</code> value
     * when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the <code>java.net.URL</code> object to be set
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public void setURL(int parameterIndex, java.net.URL x) throws SQLException {
        checkIsOpen();
        delegate_.setURL(parameterIndex, x);
    }

    /**
     * Retrieves the number, types and properties of this
     * <code>PreparedStatement</code> object's parameters.
     *
     * @return a <code>ParameterMetaData</code> object that contains information
     *         about the number, types and properties of this
     *         <code>PreparedStatement</code> object's parameters
     * @exception SQLException if a database access error occurs
     * @see ParameterMetaData
     * @since 1.4
     */
    public ParameterMetaData getParameterMetaData() throws SQLException {
        checkIsOpen();
        return delegate_.getParameterMetaData();
    }
}




