/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 *  <P>
 *
 *  The object used for executing a static SQL statement and obtaining the
 *  results produced by it. <P>
 *
 *  Only one <code>ResultSet</code> object per <code>Statement</code> object can
 *  be open at any point in time. Therefore, if the reading of one <code>ResultSet</code>
 *  object is interleaved with the reading of another, each must have been
 *  generated by different <code>Statement</code> objects. All statement <code>execute</code>
 *  methods implicitly close a statment's current <code>ResultSet</code> object
 *  if an open one exists.
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class StatementWrapper implements Statement {

    private Statement delegate_;
    private Connection connection_;

    private boolean isOpen_ = true;
    private final List openResultSets_ = new ArrayList();


    /**
     *  Create a new wrapper
     *
     * @param  statement The statement that we are wrapping
     */
    public StatementWrapper( final Statement statement ) {
        if( statement == null ) {
            throw new NullPointerException( "statement" );
        }

        delegate_ = statement;
    }


    /**
     *  Set the connection that created this statement
     *
     * @param  connection The connection
     */
    public final void setConnection( final Connection connection ) {
        if( connection == null ) {
            throw new NullPointerException();
        }
        connection_ = connection;
    }


    /**
     *  Sets the limit for the maximum number of bytes in a column to the given
     *  number of bytes. This is the maximum number of bytes that can be
     *  returned for any column value. This limit applies only to <code>BINARY</code>
     *  , <code>VARBINARY</code>, <code>LONGVARBINARY</code>, <code>CHAR</code>,
     *  <code>VARCHAR</code>, and <code>LONGVARCHAR</code> fields. If the limit
     *  is exceeded, the excess data is silently discarded. For maximum
     *  portability, use values greater than 256.
     *
     * @param  max the new max column size limit; zero means unlimited
     * @exception  SQLException if a database access error occurs
     */
    public final void setMaxFieldSize( int max )
        throws SQLException {
        checkIsOpen();
        delegate_.setMaxFieldSize( max );
    }


    /**
     *  Sets the limit for the maximum number of rows that any <code>ResultSet</code>
     *  object can contain to the given number. If the limit is exceeded, the
     *  excess rows are silently dropped.
     *
     * @param  max the new max rows limit; zero means unlimited
     * @exception  SQLException if a database access error occurs
     */
    public final void setMaxRows( int max )
        throws SQLException {
        checkIsOpen();
        delegate_.setMaxRows( max );
    }


    /**
     *  Sets escape processing on or off. If escape scanning is on (the
     *  default), the driver will do escape substitution before sending the SQL
     *  to the database. Note: Since prepared statements have usually been
     *  parsed prior to making this call, disabling escape processing for
     *  prepared statements will have no effect.
     *
     * @param  enable <code>true</code> to enable; <code>false</code> to disable
     * @exception  SQLException if a database access error occurs
     */
    public final void setEscapeProcessing( boolean enable )
        throws SQLException {
        checkIsOpen();
        delegate_.setEscapeProcessing( enable );
    }


    /**
     *  Sets the number of seconds the driver will wait for a <code>Statement</code>
     *  object to execute to the given number of seconds. If the limit is
     *  exceeded, an <code>SQLException</code> is thrown.
     *
     * @param  seconds the new query timeout limit in seconds; zero means
     *      unlimited
     * @exception  SQLException if a database access error occurs
     */
    public final void setQueryTimeout( int seconds )
        throws SQLException {
        checkIsOpen();
        delegate_.setQueryTimeout( seconds );
    }


    /**
     *  Defines the SQL cursor name that will be used by subsequent <code>Statement</code>
     *  object <code>execute</code> methods. This name can then be used in SQL
     *  positioned update/delete statements to identify the current row in the
     *  <code>ResultSet</code> object generated by this statement. If the
     *  database doesn't support positioned update/delete, this method is a
     *  noop. To insure that a cursor has the proper isolation level to support
     *  updates, the cursor's <code>SELECT</code> statement should be of the
     *  form 'select for update ...'. If the 'for update' phrase is omitted,
     *  positioned updates may fail. <P>
     *
     *  <B>Note:</B> By definition, positioned update/delete execution must be
     *  done by a different <code>Statement</code> object than the one which
     *  generated the <code>ResultSet</code> object being used for positioning.
     *  Also, cursor names must be unique within a connection.
     *
     * @param  name the new cursor name, which must be unique within a
     *      connection
     * @exception  SQLException if a database access error occurs
     */
    public final void setCursorName( String name )
        throws SQLException {
        checkIsOpen();
        delegate_.setCursorName( name );
    }


    //--------------------------JDBC 2.0-----------------------------


    /**
     *  Gives the driver a hint as to the direction in which the rows in a
     *  result set will be processed. The hint applies only to result sets
     *  created using this <code>Statement</code> object. The default value is
     *  <code>ResultSet.FETCH_FORWARD</code>. <p>
     *
     *  Note that this method sets the default fetch direction for result sets
     *  generated by this <code>Statement</code> object. Each result set has its
     *  own methods for getting and setting its own fetch direction.
     *
     * @param  direction the initial direction for processing rows
     * @exception  SQLException if a database access error occurs or the given
     *      direction is not one of <code>ResultSet.FETCH_FORWARD</code>, <code>ResultSet.FETCH_REVERSE</code>
     *      , or <code>ResultSet.FETCH_UNKNOWN</code>
     * @since  1.2
     */
    public final void setFetchDirection( int direction )
        throws SQLException {
        checkIsOpen();
        delegate_.setFetchDirection( direction );
    }


    /**
     *  Gives the JDBC driver a hint as to the number of rows that should be
     *  fetched from the database when more rows are needed. The number of rows
     *  specified affects only result sets created using this statement. If the
     *  value specified is zero, then the hint is ignored. The default value is
     *  zero.
     *
     * @param  rows the number of rows to fetch
     * @exception  SQLException if a database access error occurs, or the
     *      condition 0 <= rows <= this.getMaxRows() is not satisfied.
     * @since  1.2
     */
    public final void setFetchSize( int rows )
        throws SQLException {
        checkIsOpen();
        delegate_.setFetchSize( rows );
    }


    /**
     *  Return the statement that is wrapped
     *
     * @return  The wrapped statement
     */
    public final Statement getDelegate() {
        return delegate_;
    }


    /**
     *  Return true if this statement has been closed
     *
     * @return  true if this statement has been closed
     */
    public final boolean isClosed() {
        return isOpen_ == false;
    }

    //----------------------------------------------------------------------

    /**
     *  Returns the maximum number of bytes allowed for any column value. This
     *  limit is the maximum number of bytes that can be returned for any column
     *  value. The limit applies only to <code>BINARY</code>, <code>VARBINARY</code>
     *  , <code>LONGVARBINARY</code>, <code>CHAR</code>, <code>VARCHAR</code>,
     *  and <code>LONGVARCHAR</code> columns. If the limit is exceeded, the
     *  excess data is silently discarded.
     *
     * @return  the current max column size limit; zero means unlimited
     * @exception  SQLException if a database access error occurs
     */
    public final int getMaxFieldSize()
        throws SQLException {
        checkIsOpen();
        return delegate_.getMaxFieldSize();
    }


    /**
     *  Retrieves the maximum number of rows that a <code>ResultSet</code>
     *  object can contain. If the limit is exceeded, the excess rows are
     *  silently dropped.
     *
     * @return  the current max row limit; zero means unlimited
     * @exception  SQLException if a database access error occurs
     */
    public final int getMaxRows()
        throws SQLException {
        checkIsOpen();
        return delegate_.getMaxRows();
    }


    /**
     *  Retrieves the number of seconds the driver will wait for a <code>Statement</code>
     *  object to execute. If the limit is exceeded, a <code>SQLException</code>
     *  is thrown.
     *
     * @return  the current query timeout limit in seconds; zero means unlimited
     * @exception  SQLException if a database access error occurs
     */
    public final int getQueryTimeout()
        throws SQLException {
        checkIsOpen();
        return delegate_.getQueryTimeout();
    }


    /**
     *  Retrieves the first warning reported by calls on this <code>Statement</code>
     *  object. Subsequent <code>Statement</code> object warnings will be
     *  chained to this <code>SQLWarning</code> object. <p>
     *
     *  The warning chain is automatically cleared each time a statement is
     *  (re)executed. <P>
     *
     *  <B>Note:</B> If you are processing a <code>ResultSet</code> object, any
     *  warnings associated with reads on that <code>ResultSet</code> object
     *  will be chained on it.
     *
     * @return  the first <code>SQLWarning</code> object or <code>null</code>
     * @exception  SQLException if a database access error occurs
     */
    public final SQLWarning getWarnings()
        throws SQLException {
        checkIsOpen();
        return delegate_.getWarnings();
    }


    /**
     *  Returns the current result as a <code>ResultSet</code> object. This
     *  method should be called only once per result.
     *
     * @return  the current result as a <code>ResultSet</code> object; <code>null</code>
     *      if the result is an update count or there are no more results
     * @exception  SQLException if a database access error occurs
     */
    public final ResultSet getResultSet()
        throws SQLException {
        checkIsOpen();
        return wrapResultSet( delegate_.getResultSet() );
    }


    /**
     *  Returns the current result as an update count; if the result is a <code>ResultSet</code>
     *  object or there are no more results, -1 is returned. This method should
     *  be called only once per result.
     *
     * @return  the current result as an update count; -1 if the current result
     *      is a <code>ResultSet</code> object or there are no more results
     * @exception  SQLException if a database access error occurs
     */
    public final int getUpdateCount()
        throws SQLException {
        checkIsOpen();
        return delegate_.getUpdateCount();
    }


    /**
     *  Moves to a <code>Statement</code> object's next result. It returns
     *  <code>true</code> if this result is a <code>ResultSet</code> object.
     *  This method also implicitly closes any current <code>ResultSet</code>
     *  object obtained with the method <code>getResultSet</code>. <P>
     *
     *  There are no more results when the following is true: <PRE>
     *      <code>(!getMoreResults() && (getUpdateCount() == -1)</code> </PRE>
     *
     * @return  <code>true</code> if the next result is a <code>ResultSet</code>
     *      object; <code>false</code> if it is an update count or there are no
     *      more results
     * @exception  SQLException if a database access error occurs
     */
    public final boolean getMoreResults()
        throws SQLException {
        checkIsOpen();
        return delegate_.getMoreResults();
    }


    /**
     *  Retrieves the direction for fetching rows from database tables that is
     *  the default for result sets generated from this <code>Statement</code>
     *  object. If this <code>Statement</code> object has not set a fetch
     *  direction by calling the method <code>setFetchDirection</code>, the
     *  return value is implementation-specific.
     *
     * @return  the default fetch direction for result sets generated from this
     *      <code>Statement</code> object
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final int getFetchDirection()
        throws SQLException {
        checkIsOpen();
        return delegate_.getFetchDirection();
    }


    /**
     *  Retrieves the number of result set rows that is the default fetch size
     *  for result sets generated from this <code>Statement</code> object. If
     *  this <code>Statement</code> object has not set a fetch size by calling
     *  the method <code>setFetchSize</code>, the return value is
     *  implementation-specific.
     *
     * @return  the default fetch size for result sets generated from this
     *      <code>Statement</code> object
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final int getFetchSize()
        throws SQLException {
        checkIsOpen();
        return delegate_.getFetchSize();
    }


    /**
     *  Retrieves the result set concurrency for <code>ResultSet</code> objects
     *  generated by this <code>Statement</code> object.
     *
     * @return  either <code>ResultSet.CONCUR_READ_ONLY</code> or <code>ResultSet.CONCUR_UPDATABLE</code>
     * @exception  SQLException If an error occurs
     * @since  1.2
     */
    public final int getResultSetConcurrency()
        throws SQLException {
        checkIsOpen();
        return delegate_.getResultSetConcurrency();
    }


    /**
     *  Retrieves the result set type for <code>ResultSet</code> objects
     *  generated by this <code>Statement</code> object.
     *
     * @return  one of <code>ResultSet.TYPE_FORWARD_ONLY</code>, <code>ResultSet.TYPE_SCROLL_INSENSITIVE</code>
     *      , or <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
     * @exception  SQLException If an error occurs
     * @since  1.2
     */
    public final int getResultSetType()
        throws SQLException {
        checkIsOpen();
        return delegate_.getResultSetType();
    }


    /**
     *  Returns the <code>Connection</code> object that produced this <code>Statement</code>
     *  object.
     *
     * @return  the connection that produced this statement
     * @exception  SQLException if a database access error occurs
     * @since  1.2
     */
    public final Connection getConnection()
        throws SQLException {
        checkIsOpen();
        return connection_;
    }


    /**
     *  Executes an SQL statement that returns a single <code>ResultSet</code>
     *  object.
     *
     * @param  sql typically this is a static SQL <code>SELECT</code> statement
     * @return  a <code>ResultSet</code> object that contains the data produced
     *      by the given query; never <code>null</code>
     * @exception  SQLException if a database access error occurs
     */
    public final ResultSet executeQuery( String sql )
        throws SQLException {
        checkIsOpen();
        return wrapResultSet( delegate_.executeQuery( sql ) );
    }


    /**
     *  Executes an SQL <code>INSERT</code>, <code>UPDATE</code> or <code>DELETE</code>
     *  statement. In addition, SQL statements that return nothing, such as SQL
     *  DDL statements, can be executed.
     *
     * @param  sql an SQL <code>INSERT</code>, <code>UPDATE</code> or <code>DELETE</code>
     *      statement or an SQL statement that returns nothing
     * @return  either the row count for <code>INSERT</code>, <code>UPDATE</code>
     *      or <code>DELETE</code> statements, or 0 for SQL statements that
     *      return nothing
     * @exception  SQLException if a database access error occurs
     */
    public final int executeUpdate( String sql )
        throws SQLException {
        checkIsOpen();
        return delegate_.executeUpdate( sql );
    }


    /**
     *  Releases this <code>Statement</code> object's database and JDBC
     *  resources immediately instead of waiting for this to happen when it is
     *  automatically closed. It is generally good practice to release resources
     *  as soon as you are finished with them to avoid tying up database
     *  resources. <P>
     *
     *  <B>Note:</B> A <code>Statement</code> object is automatically closed
     *  when it is garbage collected. When a <code>Statement</code> object is
     *  closed, its current <code>ResultSet</code> object, if one exists, is
     *  also closed.
     *
     * @exception  SQLException if a database access error occurs
     */
    public final void close()
        throws SQLException {
        if( isOpen_ ) {
            ResultSetWrapper resultSetWrapper;
            final Iterator iterator = openResultSets_.iterator();
            while( iterator.hasNext() ) {
                resultSetWrapper = (ResultSetWrapper)iterator.next();
                if( resultSetWrapper.isClosed() == false ) {
                    resultSetWrapper.close();
                }
            }
            openResultSets_.clear();

            delegate_.close();
            delegate_ = null;
        }
        else {
            throw new AlreadyClosedException( "Statement is closed" );
        }
        isOpen_ = false;
    }


    /**
     *  Cancels this <code>Statement</code> object if both the DBMS and driver
     *  support aborting an SQL statement. This method can be used by one thread
     *  to cancel a statement that is being executed by another thread.
     *
     * @exception  SQLException if a database access error occurs
     */
    public final void cancel()
        throws SQLException {
        checkIsOpen();
        delegate_.cancel();
    }


    /**
     *  Clears all the warnings reported on this <code>Statement</code> object.
     *  After a call to this method, the method <code>getWarnings</code> will
     *  return <code>null</code> until a new warning is reported for this <code>Statement</code>
     *  object.
     *
     * @exception  SQLException if a database access error occurs
     */
    public final void clearWarnings()
        throws SQLException {
        checkIsOpen();
        delegate_.clearWarnings();
    }

    //----------------------- Multiple Results --------------------------

    /**
     *  Executes an SQL statement that may return multiple results. Under some
     *  (uncommon) situations a single SQL statement may return multiple result
     *  sets and/or update counts. Normally you can ignore this unless you are
     *  (1) executing a stored procedure that you know may return multiple
     *  results or (2) you are dynamically executing an unknown SQL string. The
     *  methods <code>execute</code>, <code>getMoreResults</code>, <code>getResultSet</code>
     *  , and <code>getUpdateCount</code> let you navigate through multiple
     *  results. The <code>execute</code> method executes an SQL statement and
     *  indicates the form of the first result. You can then use the methods
     *  <code>getResultSet</code> or <code>getUpdateCount</code> to retrieve the
     *  result, and <code>getMoreResults</code> to move to any subsequent
     *  result(s).
     *
     * @param  sql any SQL statement
     * @return  <code>true</code> if the next result is a <code>ResultSet</code>
     *      object; <code>false</code> if it is an update count or there are no
     *      more results
     * @exception  SQLException if a database access error occurs
     */
    public final boolean execute( String sql )
        throws SQLException {
        checkIsOpen();
        return delegate_.execute( sql );
    }


    /**
     *  Adds an SQL command to the current batch of commmands for this <code>Statement</code>
     *  object. This method is optional.
     *
     * @param  sql typically this is a static SQL <code>INSERT</code> or <code>UPDATE</code>
     *      statement
     * @exception  SQLException if a database access error occurs, or the driver
     *      does not support batch statements
     * @since  1.2
     */
    public final void addBatch( String sql )
        throws SQLException {
        checkIsOpen();
        delegate_.addBatch( sql );
    }


    /**
     *  Makes the set of commands in the current batch empty. This method is
     *  optional.
     *
     * @exception  SQLException if a database access error occurs or the driver
     *      does not support batch statements
     * @since  1.2
     */
    public final void clearBatch()
        throws SQLException {
        checkIsOpen();
        delegate_.clearBatch();
    }


    /**
     *  Submits a batch of commands to the database for execution and if all
     *  commands execute successfully, returns an array of update counts. The
     *  <code>int</code> elements of the array that is returned are ordered to
     *  correspond to the commands in the batch, which are ordered according to
     *  the order in which they were added to the batch. The elements in the
     *  array returned by the method <code>executeBatch</code> may be one of the
     *  following:
     *  <OL>
     *    <LI> A number greater than or equal to zero -- indicates that the
     *    command was processed successfully and is an update count giving the
     *    number of rows in the database that were affected by the command's
     *    execution
     *    <LI> A value of <code>-2</code> -- indicates that the command was
     *    processed successfully but that the number of rows affected is unknown
     *    <P>
     *
     *    If one of the commands in a batch update fails to execute properly,
     *    this method throws a <code>BatchUpdateException</code>, and a JDBC
     *    driver may or may not continue to process the remaining commands in
     *    the batch. However, the driver's behavior must be consistent with a
     *    particular DBMS, either always continuing to process commands or never
     *    continuing to process commands. If the driver continues processing
     *    after a failure, the array returned by the method <code>BatchUpdateException.getUpdateCounts</code>
     *    will contain as many elements as there are commands in the batch, and
     *    at least one of the elements will be the following: <P>
     *
     *
     *    <LI> A value of <code>-3</code> -- indicates that the command failed
     *    to execute successfully and occurs only if a driver continues to
     *    process commands after a command fails
     *  </OL>
     *  <P>
     *
     *  A driver is not required to implement this method. The possible
     *  implementations and return values have been modified in the Java 2 SDK,
     *  Standard Edition, version 1.3 to accommodate the option of continuing to
     *  proccess commands in a batch update after a <code>BatchUpdateException</code>
     *  object has been thrown.
     *
     * @return  an array of update counts containing one element for each
     *      command in the batch. The elements of the array are ordered
     *      according to the order in which commands were added to the batch.
     * @exception  SQLException if a database access error occurs or the driver
     *      does not support batch statements. Throws BatchUpdateException (a
     *      subclass of <code>SQLException</code>) if one of the commands sent
     *      to the database fails to execute properly or attempts to return a
     *      result set.
     * @since  1.3
     */
    public final int[] executeBatch()
        throws SQLException {
        checkIsOpen();
        return delegate_.executeBatch();
    }


    /**
     *  Check to see if the connection is still open
     *
     * @exception  SQLException If an error occurs
     */
    protected final void checkIsOpen()
        throws SQLException {
        if( isOpen_ == false ) {
            throw new SQLException( "Statement is closed" );
        }
    }


    /**
     *  Wrap the specified result set in a ResultSetWrapper object and return
     *  the wrapper.
     *
     * @param  resultSet The object to be wrapped
     * @return  The wrapper.
     */
    protected final ResultSet wrapResultSet( final ResultSet resultSet ) {
        final ResultSetWrapper wrapper = new ResultSetWrapper( resultSet );
        openResultSets_.add( wrapper );
        return wrapper;
    }

    // ===============================================================
    // =====  NEW FOR 1.4 =============

    /**
     * Moves to this <code>Statement</code> object's next result, deals with
     * any current <code>ResultSet</code> object(s) according  to the instructions
     * specified by the given flag, and returns
     * <code>true</code> if the next result is a <code>ResultSet</code> object.
     *
     * <P>There are no more results when the following is true:
     * <PRE>
     *      <code>(!getMoreResults() && (getUpdateCount() == -1)</code>
     * </PRE>
     *
     * @param current one of the following <code>Statement</code>
     *        constants indicating what should happen to current
     *        <code>ResultSet</code> objects obtained using the method
     *        <code>getResultSet</code:
     *        <code>CLOSE_CURRENT_RESULT</code>,
     *        <code>KEEP_CURRENT_RESULT</code>, or
     *        <code>CLOSE_ALL_RESULTS</code>
     * @return <code>true</code> if the next result is a <code>ResultSet</code>
     *         object; <code>false</code> if it is an update count or there are no
     *         more results
     * @exception SQLException if a database access error occurs
     * @since 1.4
     */
    public boolean getMoreResults(int current) throws SQLException {
        checkIsOpen();
        return delegate_.getMoreResults(current);
    }


    /**
     * Retrieves any auto-generated keys created as a result of executing this
     * <code>Statement</code> object. If this <code>Statement</code> object did
     * not generate any keys, an empty <code>ResultSet</code>
     * object is returned.
     *
     * @return a <code>ResultSet</code> object containing the auto-generated key(s)
     *         generated by the execution of this <code>Statement</code> object
     * @exception SQLException if a database access error occurs
     * @since java 1.4
     */
    public ResultSet getGeneratedKeys() throws SQLException {
        checkIsOpen();
        return delegate_.getGeneratedKeys();
    }


    /**
     * Executes the given SQL statement and signals the driver with the
     * given flag about whether the
     * auto-generated keys produced by this <code>Statement</code> object
     * should be made available for retrieval.
     *
     * @param sql must be an SQL <code>INSERT</code>, <code>UPDATE</code> or
     *        <code>DELETE</code> statement or an SQL statement that
     *        returns nothing
     * @param autoGeneratedKeys a flag indicating whether auto-generated keys
     *        should be made available for retrieval;
     *         one of the following constants:
     *         <code>Statement.RETURN_GENERATED_KEYS</code>
     *         <code>Statement.NO_GENERATED_KEYS</code>
     * @return either the row count for <code>INSERT</code>, <code>UPDATE</code>
     *         or <code>DELETE</code> statements, or <code>0</code> for SQL
     *         statements that return nothing
     * @exception SQLException if a database access error occurs, the given
     *            SQL statement returns a <code>ResultSet</code> object, or
     *            the given constant is not one of those allowed
     * @since java 1.4
     */
    public int executeUpdate(String sql, int autoGeneratedKeys) throws SQLException {
        checkIsOpen();
        return delegate_.executeUpdate(sql, autoGeneratedKeys);
    }


    /**
     * Executes the given SQL statement and signals the driver that the
     * auto-generated keys indicated in the given array should be made available
     * for retrieval.  The driver will ignore the array if the SQL statement
     * is not an <code>INSERT</code> statement.
     *
     * @param sql an SQL <code>INSERT</code>, <code>UPDATE</code> or
     *        <code>DELETE</code> statement or an SQL statement that returns nothing,
     *        such as an SQL DDL statement
     * @param columnIndexes an array of column indexes indicating the columns
     *        that should be returned from the inserted row
     * @return either the row count for <code>INSERT</code>, <code>UPDATE</code>,
     *         or <code>DELETE</code> statements, or 0 for SQL statements
     *         that return nothing
     * @exception SQLException if a database access error occurs or the SQL
     *            statement returns a <code>ResultSet</code> object
     * @since java 1.4
     */
    public int executeUpdate(String sql, int columnIndexes[]) throws SQLException {
        checkIsOpen();
        return delegate_.executeUpdate(sql, columnIndexes);
    }


    /**
     * Executes the given SQL statement and signals the driver that the
     * auto-generated keys indicated in the given array should be made available
     * for retrieval.  The driver will ignore the array if the SQL statement
     * is not an <code>INSERT</code> statement.
     *
     * @param sql an SQL <code>INSERT</code>, <code>UPDATE</code> or
     *        <code>DELETE</code> statement or an SQL statement that returns nothing
     * @param columnNames an array of the names of the columns that should be
     *        returned from the inserted row
     * @return either the row count for <code>INSERT</code>, <code>UPDATE</code>,
     *         or <code>DELETE</code> statements, or 0 for SQL statements
     *         that return nothing
     * @exception SQLException if a database access error occurs
     *
     * @since java 1.4
     */
    public int executeUpdate(String sql, String columnNames[]) throws SQLException {
        checkIsOpen();
        return delegate_.executeUpdate(sql, columnNames);
    }


    /**
     * Executes the given SQL statement, which may return multiple results,
     * and signals the driver that any
     * auto-generated keys should be made available
     * for retrieval.  The driver will ignore this signal if the SQL statement
     * is not an <code>INSERT</code> statement.
     * <P>
     * In some (uncommon) situations, a single SQL statement may return
     * multiple result sets and/or update counts.  Normally you can ignore
     * this unless you are (1) executing a stored procedure that you know may
     * return multiple results or (2) you are dynamically executing an
     * unknown SQL string.
     * <P>
     * The <code>execute</code> method executes an SQL statement and indicates the
     * form of the first result.  You must then use the methods
     * <code>getResultSet</code> or <code>getUpdateCount</code>
     * to retrieve the result, and <code>getMoreResults</code> to
     * move to any subsequent result(s).
     *
     * @param sql any SQL statement
     * @param autoGeneratedKeys a constant indicating whether auto-generated
     *        keys should be made available for retrieval using the method
     *        <code>getGeneratedKeys</code>; one of the following constants:
     *        <code>Statement.RETURN_GENERATED_KEYS</code> or
     *        <code>Statement.NO_GENERATED_KEYS</code>
     * @return <code>true</code> if the first result is a <code>ResultSet</code>
     *         object; <code>false</code> if it is an update count or there are
     *         no results
     * @exception SQLException if a database access error occurs
     *
     * @since java 1.4
     */
    public boolean execute(String sql, int autoGeneratedKeys) throws SQLException {
        checkIsOpen();
        return delegate_.execute(sql, autoGeneratedKeys);
    }


    /**
     * Executes the given SQL statement, which may return multiple results,
     * and signals the driver that the
     * auto-generated keys indicated in the given array should be made available
     * for retrieval.  This array contains the indexes of the columns in the
     * target table that contain the auto-generated keys that should be made
     * available. The driver will ignore the array if the given SQL statement
     * is not an <code>INSERT</code> statement.
     * <P>
     * Under some (uncommon) situations, a single SQL statement may return
     * multiple result sets and/or update counts.  Normally you can ignore
     * this unless you are (1) executing a stored procedure that you know may
     * return multiple results or (2) you are dynamically executing an
     * unknown SQL string.
     * <P>
     * The <code>execute</code> method executes an SQL statement and indicates the
     * form of the first result.  You must then use the methods
     * <code>getResultSet</code> or <code>getUpdateCount</code>
     * to retrieve the result, and <code>getMoreResults</code> to
     * move to any subsequent result(s).
     *
     * @param sql any SQL statement
     * @param columnIndexes an array of the indexes of the columns in the
     *        inserted row that should be  made available for retrieval by a
     *        call to the method <code>getGeneratedKeys</code>
     * @return <code>true</code> if the first result is a <code>ResultSet</code>
     *         object; <code>false</code> if it is an update count or there
     *         are no results
     * @exception SQLException if a database access error occurs
     *
     * @since java 1.4
     */
    public boolean execute(String sql, int columnIndexes[]) throws SQLException {
        checkIsOpen();
        return delegate_.execute(sql, columnIndexes);
    }


    /**
     * Executes the given SQL statement, which may return multiple results,
     * and signals the driver that the
     * auto-generated keys indicated in the given array should be made available
     * for retrieval. This array contains the names of the columns in the
     * target table that contain the auto-generated keys that should be made
     * available. The driver will ignore the array if the given SQL statement
     * is not an <code>INSERT</code> statement.
     * <P>
     * In some (uncommon) situations, a single SQL statement may return
     * multiple result sets and/or update counts.  Normally you can ignore
     * this unless you are (1) executing a stored procedure that you know may
     * return multiple results or (2) you are dynamically executing an
     * unknown SQL string.
     * <P>
     * The <code>execute</code> method executes an SQL statement and indicates the
     * form of the first result.  You must then use the methods
     * <code>getResultSet</code> or <code>getUpdateCount</code>
     * to retrieve the result, and <code>getMoreResults</code> to
     * move to any subsequent result(s).
     *
     * @param sql any SQL statement
     * @param columnNames an array of the names of the columns in the inserted
     *        row that should be made available for retrieval by a call to the
     *        method <code>getGeneratedKeys</code>
     * @return <code>true</code> if the next result is a <code>ResultSet</code>
     *         object; <code>false</code> if it is an update count or there
     *         are no more results
     * @exception SQLException if a database access error occurs
     *
     * @since java 1.4
     */
    public boolean execute(String sql, String columnNames[]) throws SQLException {
        checkIsOpen();
        return delegate_.execute(sql, columnNames);
    }


   /**
     * Retrieves the result set holdability for <code>ResultSet</code> objects
     * generated by this <code>Statement</code> object.
     *
     * @return either <code>ResultSet.HOLD_CURSORS_OVER_COMMIT</code> or
     *         <code>ResultSet.CLOSE_CURSORS_AT_COMMIT</code>
     * @exception SQLException if a database access error occurs
     *
     * @since java 1.4
     */
    public int getResultSetHoldability() throws SQLException {
        checkIsOpen();
        return delegate_.getResultSetHoldability();
    }
}

