/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.testing;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import junit.framework.TestCase;

/**
 * An extension of junit.framework.TestCase that adds those methods that we really
 * wish were part of JUnit.
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class BaseTestCase extends TestCase {
    /**
     * Create an instance
     * @param name The name of the test
     */
    public BaseTestCase( final String name ) {
        super(name);
    }


    /**
     * Convenience method to signal that this test hasn't been finished yet.  This
     * will print the name of the test to System.out.
     */
    public void notImplemented() {
        System.out.println( "Test not implemented yet [" + getClass().getName() + " : " + getName() + "]" );
    }


    /**
     * Assert that the two collections are the same irrespective of order.
     *
     * @param  a The first collection
     * @param  b The second collection
     */
    public void assertCollectionsEqual( final Collection a, final Collection b ) {
        final Collection copyOfB = new LinkedList( b );
        final Iterator iterator = a.iterator();
        while( iterator.hasNext() ) {
            final Object object = iterator.next();
            if( copyOfB.contains( object ) == false ) {
                fail( "Expected: " + a + " but got: " + b );
            }

            copyOfB.remove( object );
        }
        
        if( copyOfB.isEmpty() == false ) {
        		fail("Second collection has elements that aren't in the first collection: "+copyOfB);
        }
    }


    /**
     * Assert that the two objects are the same.  Junit has a method like this
     * however it does not display what the two objects are.  This method will
     * display the toString() representations of the two objects in the case that
     * the assertion fails.
     *
     * @param description The failure message to use if the two objects are not the same.
     * @param a The first object to compare.
     * @param b The second object to compare.
     */
    public static void assertSame( final String description, final Object a, final Object b ) {
        if( a != b ) {
            fail( description + ": Objects not the same <" + a + "> and <" + b + ">" );
        }
    }


    /**
     * Assert that the two objects are the same.  Junit has a method like this
     * however it does not display what the two objects are.  This method will
     * display the toString() representations of the two objects in the case that
     * the assertion fails.
     *
     * @param a The first object to compare.
     * @param b The second object to compare.
     */
    public static void assertSame( final Object a, final Object b ) {
        if( a != b ) {
            fail( "Objects not the same <" + a + "> and <" + b + ">" );
        }
    }


    /**
     * Assert that the specified condition is false.  Older versions of junit have assertTrue()
     * but not assertFalse so we add it here to be sure that it is present.
     *
     * @param description The failure message to be used if the condition is not false.
     * @param condition The value to check.
     */
    public static void assertFalse( final String description, final boolean condition ) {
        if( condition == true ) {
            fail( description + ": Expected false" );
        }
    }


    /**
     * Assert that the specified condition is false.  Older versions of junit have assertTrue()
     * but not assertFalse so we add it here to be sure that it is present.
     *
     * @param condition The value to check.
     */
    public static void assertFalse( final boolean condition ) {
        if( condition == true ) {
            fail( "Expected false" );
        }
    }


    /**
     * Assert that the specified object is an instance of this class
     *
     * @param label A description of the test
     * @param object The object to test
     * @param clazz The class
     */
    public void assertInstanceOf( final String label, final Object object, final Class clazz ) {
        if( clazz.isAssignableFrom( object.getClass() ) == false ) {
            fail( label + ": object [" + object + "] is not an instance of class ["
                + clazz.getName() + "]" );
        }
    }


    /**
     * Assert that the specified object is an instance of this class
     *
     * @param object The object to test
     * @param clazz The class
     */
    public void assertInstanceOf( final Object object, final Class clazz ) {
        if( clazz.isAssignableFrom( object.getClass() ) == false ) {
            fail("object ["+object+"] is not an instance of class ["+clazz.getName()+"]" );
        }
    }
}
