/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.testing;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.Collections;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import junit.framework.AssertionFailedError;

/**
 * A testing class for catching and logging events.
 * <pre>
 * // Catch all events fired by JFrame
 * final JFrame frame = new JFrame();
 * final EventCatcher eventCatcher = new EventCatcher();
 * eventCatcher.listenTo(frame);
 *
 * frame.show();
 *
 * for( int i=0; i&lt;eventCatcher.size(); i++ ) {
 *     System.out.println(eventCatcher.getEventAt(i));
 * }
 * </pre>
 * @version $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class EventCatcher {

    /**
     * An inner class to handle the various events.
     */
    private InvocationHandler invocationHandler_ = new InvocationHandler() {
        public Object invoke( final Object proxy,
                              final Method method,
                              final Object[] args) {
            final String methodName = method.getName();
            final Object result;
            if( methodName.equals("hashCode") ) {
                result =  new Integer(1);
            }
            else if( methodName.equals("equals") && args.length == 1 ) {
                result = new Boolean(this == args[0]);
            }
            else {
                synchronized(EventCatcher.this) {
                    eventRecords_.add( new EventCatcherRecord(method, (EventObject)args[0]) );
                }
                result = null;
            }
            return result;
        }
    };

    private final List eventRecords_ = new ArrayList();

    /**
     * Create a new EventCatcher.
     */
    public EventCatcher() {
    }


    /**
     * Return information about the event at the specified index.
     *
     * @param index The index.
     * @return The record.
     * @deprecated Use {@link #getEventCatcherRecordAt(int)} instead
     */
    public synchronized EventCatcherRecord get( final int index ) {
        return (EventCatcherRecord)eventRecords_.get(index);
    }


    /**
     * Return the number of events that have been caught.
     * @return the number of events that have been caught.
     * @deprecated Use {@link #getEventCount()} instead
     */
    public synchronized int size() {
        return eventRecords_.size();
    }


    /**
     * Return a listener object that will log all fired events.  This listener
     * should be used when you want to only listen for one kind of events on a bean.
     * If you want to listen to all events then you should just call {@link #listenTo(Object)}
     * <pre>
     * // Catch all window events
     * final ObjectCatcher objectCatcher = new ObjectCatcher();
     * final JFrame frame = new JFrame();
     *
     * frame.addWindowListener( (WindowListener)objectCatcher.getListener(WindowListener.class) );
     * </pre>
     *
     * @param clazz The listener interface that we need to support.
     * @return A listener.
     */
    public Object getListener( final Class clazz ) {
        return Proxy.newProxyInstance( getClass().getClassLoader(),
                                       new Class[]{ clazz },
                                       invocationHandler_ );
    }


    /**
     * Register the event catcher as a listener for all events that this object fires.
     * <pre>
     * // Catch all events fired by JFrame
     * final ObjectCatcher objectCatcher = new ObjectCatcher();
     * final JFrame frame = new JFrame();
     *
     * eventCatcher.listenTo(frame);
     * </pre>
     *
     * @param object The object that we will be listening to.
     * @throws IllegalAccessException If we do not have authorization to call
     * the respective addXXXListener() method
     * @throws InvocationTargetException If an exception is thrown during the
     * call to the addXXXListener() method
     */
    public void listenTo( final Object object )
        throws
            IllegalAccessException,
            InvocationTargetException {

        final Method methods[] = object.getClass().getMethods();

        String methodName;
        Class  parameterClasses[];
        final Map addMethods = new HashMap(89);

        int i;
        for( i=0; i<methods.length; i++ ) {
            methodName = methods[i].getName();
            if( methodName.startsWith("add") && methodName.endsWith("Listener") ) {
                parameterClasses = methods[i].getParameterTypes();
                if( parameterClasses.length == 1 ) {
                    addMethods.put( methods[i], parameterClasses[0] );
                }
            }
        }

        Map.Entry entry;
        Object proxy;
        Method method;

        final Iterator iterator = addMethods.entrySet().iterator();
        while( iterator.hasNext() ) {
            entry = (Map.Entry)iterator.next();
            method = (Method)entry.getKey();
            proxy = getListener( (Class)entry.getValue() );
            method.invoke( object, new Object[]{proxy} );
        }
    }


    /**
     * Return the event at the specified index.
     * @param index The index
     * @return The event at that index.
     */
    public synchronized EventObject getEventAt( final int index ) {
        return getEventCatcherRecordAt(index).getEvent();
    }


    /**
     * Return the record at the specified index.  The record will contain the event and
     * assorted information about the event.
     * @param index The index
     * @return The record at that index.
     */
    public synchronized EventCatcherRecord getEventCatcherRecordAt( final int index ) {
        return (EventCatcherRecord)eventRecords_.get(index);
    }


    /**
     * Return the number of events that have been collected so far.
     *
     * @return The number of events.
     */
    public synchronized int getEventCount() {
        return eventRecords_.size();
    }


    /**
     * Return an immutable list containing all the events collected so far.
     *
     * @return A list of collected events.
     */
    public synchronized List getEvents() {
        final List list = new ArrayList(eventRecords_.size());
        final Iterator iterator = eventRecords_.iterator();
        while( iterator.hasNext() ) {
            final EventCatcherRecord record = (EventCatcherRecord)iterator.next();
            list.add( record.getEvent() );
        }

        return Collections.unmodifiableList(list);
    }


    /**
     * Throw away all the currently collected events.
     */
    public synchronized void clear() {
        eventRecords_.clear();
    }


    /**
     * Compare the specified events against the actual collected event to see if they
     * appear to be the same.  Refer to {@link TestUtil#appearsEqual(Object,Object)}
     * for an explanation of "appearing" to be the same.
     *
     * @param expectedEvents The events that we expect to have been collected.
     */
    public synchronized void assertEventsAppearEquals( final List expectedEvents ) {
        final List actualEvents = getEvents();
        final int eventCount = actualEvents.size();
        if( expectedEvents.size() != eventCount ) {
            throw new AssertionFailedError("Different number of events: expected="
                +expectedEvents+" actual="+actualEvents);
        }

        for( int i=0; i<eventCount; i++ ) {
            if( TestUtil.appearsEqual( expectedEvents.get(i), actualEvents.get(i) ) == false ) {
                throw new AssertionFailedError("Different events: expected="
                    +expectedEvents+" actual="+actualEvents);
            }
        }
    }
}
