/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.testing;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  This class allows you to specify the order that test methods will execute in
 *  and yet still retain the ability to have test methods found via reflection.<p>
 *
 * Generally, when you want to specify the order that tests will be run, you write
 * code that looks like this.
 * <pre>
 * public static Test suite() {
 *     final TestSuite suite = new TestSuite();
 *     suite.addTest( new FooTest("testOne") );
 *     suite.addTest( new FooTest("testTwo") );
 *     return suite;
 * }
 * </pre>
 * The problem with this approach is that when testThree() is written, if it isn't
 * added to the suite method, it won't get executed.<p>
 *
 * Using OrderedTestSuite, you write code like this:
 * <pre>
 * public static Test suite() {
 *     return new OrderedTestSuite(FooTest.class, new String[]{"testOne", "testTwo"});
 * }
 * </pre>
 * testOne() and testTwo() will be executed in order just as in the first example.  The
 * difference is that OrderedTestSuite will use reflection to find other methods starting
 * with test and will execute them.  This means that if you write testThree but forget to
 * add it to suite() then it will still get executed, albeit not in a defined order.
 *
 * @version  $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public final class OrderedTestSuite extends TestSuite {
    private final Class classToTest_;
    private final String[] orderedMethodNames_;


    /**
     *  Create an instance of this test suite
     *
     * @param  clazz the class that we will use to get the individual tests.
     *      This must be a subclass of TestCase
     * @param  methodNames The names of any methods that must be tested in order
     */
    public OrderedTestSuite( final Class clazz, final String[] methodNames ) {
        if( clazz == null ) {
            throw new NullPointerException( "clazz" );
        }
        if( methodNames == null ) {
            throw new NullPointerException( "methodNames" );
        }
        if( TestCase.class.isAssignableFrom( clazz ) == false ) {
            throw new DetailedIllegalArgumentException(
                "clazz", clazz, "Must be a subclass of TestCase" );
        }
        classToTest_ = clazz;
        orderedMethodNames_ = methodNames;

        addMethodsInOrder( getMatchingMethods() );
    }


    private Map getMatchingMethods() {
        final Map matchingMethods = new HashMap( 89 );

        final Method allMethods[] = classToTest_.getMethods();

        int i;
        for( i = 0; i < allMethods.length; i++ ) {
            if( isMatchingMethod( allMethods[i] ) ) {
                matchingMethods.put( allMethods[i].getName(), allMethods[i] );
            }
        }

        return matchingMethods;
    }


    private boolean isMatchingMethod( final Method method ) {
        final String methodName = method.getName();

        if( method.getParameterTypes().length != 0 ) {
            return false;
        }

        if( method.getReturnType() != void.class ) {
            return false;
        }

        if( methodName.startsWith( "test" ) ) {
            return true;
        }

        int i;
        for( i = 0; i < orderedMethodNames_.length; i++ ) {
            if( methodName.equals( orderedMethodNames_[i] ) ) {
                return true;
            }
        }
        return false;
    }


    private void addTest( final Method method ) {
        try {
            final Constructor constructor = classToTest_.getConstructor( new Class[]{String.class} );
            final TestCase test = (TestCase)constructor.newInstance( new Object[]{method.getName()} );

            addTest( test );
        }
        catch( final NoSuchMethodException e ) {
            throw new TestInitializationFailedException(
                "Unable to find a constructor that takes a string", e );
        }
        catch( final InstantiationException e ) {
            throw new TestInitializationFailedException(
                "Unable to instantiate test for method " + method.getName(), e );
        }
        catch( final IllegalAccessException e ) {
            throw new TestInitializationFailedException(
                "Unable to instantiate test for method " + method.getName(), e );
        }
        catch( final InvocationTargetException e ) {
            throw new TestInitializationFailedException(
                "Unable to instantiate test for method" + method.getName(), e.getTargetException() );
        }
    }


    private void addMethodsInOrder( final Map methodMap ) {
        int i;
        String name;
        Method method;

        for( i = 0; i < orderedMethodNames_.length; i++ ) {
            name = orderedMethodNames_[i];

            method = (Method)methodMap.get( name );
            methodMap.remove( name );
            if( method == null ) {
                throw new TestInitializationFailedException( "method not found [" + name + "]" );
            }
            addTest( method );
        }

        final Iterator iterator = methodMap.values().iterator();
        while( iterator.hasNext() ) {
            method = (Method)iterator.next();
            addTest( method );
        }
    }
}

