/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.testing;

import com.gargoylesoftware.base.collections.StringComparator;
import com.gargoylesoftware.base.trace.Trace;
import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

/**
 *  A launching point for executing test cases. This recusively walks through
 *  the directory structure looking for classes that end in Test.class. It then
 *  loads these classes to see if they are instances of TestCase. If so, they
 *  are added to the test suite. Once all directories have been scanned, the
 *  test cases are executed.
 *
 * @version  $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class RecursiveTestSuite extends TestSuite {

    private final TestFilter testFilter_;


    /**
     *  Create a new instance
     *
     * @param  testFilter The object that will filter out tests that we don't
     *      wish to run
     * @param  startingPath The directory that we will start our descent in.
     * @exception  IOException If anything goes wrong during the directory
     *      scanning.
     */
    public RecursiveTestSuite(
            final String startingPath,
            final TestFilter testFilter )
        throws
            IOException {

        this( new File( startingPath ), testFilter );
    }


    /**
     *  Create an instance
     *
     * @param  testFilter The object that will filter out tests that we don't
     *      wish to run
     * @param  startingPath The directory that we will start our descent in.
     * @exception  IOException If anything goes wrong during the directory
     *      scanning.
     */
    public RecursiveTestSuite(
            final File startingPath,
            final TestFilter testFilter )
        throws
            IOException {

        super( "RecursiveTestSuite(" + startingPath + ")" );

        if( testFilter == null ) {
            throw new NullPointerException( "testFilter" );
        }
        if( startingPath == null ) {
            throw new NullPointerException( "startingPath" );
        }
        testFilter_ = testFilter;

        if( startingPath.exists() == false ) {
            throw new DetailedIllegalArgumentException( "startingPath", startingPath, "Doesn't exist");
        }
        if( startingPath.isDirectory() == false ) {
            throw new DetailedIllegalArgumentException( "startingPath", startingPath, "Isn't a directory" );
        }

        final long startTime = System.currentTimeMillis();
        final Set classNames = findAllTestClasses( startingPath );
        final long endTime = System.currentTimeMillis();
        System.out.println( "Scanning for classes: Time=["
                 + ( ( endTime - startTime ) / 1000F )
                 + " seconds] Found=["
                 + classNames.size()
                 + " classes]" );

        final Iterator iterator = classNames.iterator();

        while( iterator.hasNext() ) {
            addTest( getTestForClass( (Class)iterator.next() ) );
        }
    }


    /**
     *  Main entry point.
     *
     * @param  args The arguments
     */
    public static void main( final String args[] ) {

        Trace.getController().setOutRedirected( true );
        Trace.getController().setErrRedirected( true );

        // Force the load of AWT
        new javax.swing.JPanel();

        try {
            final String startingPath;
            if( args.length == 0 ) {
                startingPath = ".";
            }
            else {
                startingPath = args[0];
            }
            final RecursiveTestSuite suite = new RecursiveTestSuite( startingPath, new AcceptAllTestFilter() );

            // TestRunner.run() will invoke System.exit() so we have
            // to disable the buffering now.
            Trace.getController().setBufferingEnabled( false );

            if( suite.testCount() == 0 ) {
                Trace.println( "Nothing to do." );
            }
            else {
                TestRunner.run( suite );
            }
        }
        catch( final Throwable t ) {
            Trace.printStackTrace( Trace.out, t );
        }
        Trace.exit( 0 );
    }


    /**
     *  Dummy test so that JUnit doesn't complain that this suite doesn't have
     *  any tests.
     */
    public void testFoo() {
    }


//    protected Collection findPotentialClasses() {
//        final DirectoryWalker walker = new DirectoryWalker( "." );
//        return Collections.EMPTY_LIST;
//    }


    /**
     *  Return a test suite containing all the tests for the specified class. If
     *  the class has a suite() method then it will be used to get the tests,
     *  otherwise reflection will be used.
     *
     * @param  clazz Description of Parameter
     * @return  The testSuiteForClass value
     */
    private Test getTestForClass( final Class clazz ) {
        try {
            final Method method = clazz.getDeclaredMethod( "suite", new Class[0] );
            return (Test)method.invoke( null, new Object[0] );
        }
        catch( final NoSuchMethodException e ) {
            // Fall through
        }
        catch( final Exception e ) {
            e.printStackTrace();
        }

        return new TestSuite( clazz );
    }


    /**
     * @param  startingDirectory Description of Parameter
     * @return  Description of the Returned Value
     * @exception  IOException Description of Exception
     */
    private Set findAllTestClasses( final File startingDirectory )
        throws IOException {

        final List list = new ArrayList();
        final SortedSet set
                 = new TreeSet( new StringComparator( Locale.getDefault() ) );

        findAllTestClasses( startingDirectory, list );

        final String startingDirectoryFullPath
                 = startingDirectory.getCanonicalPath();
        final int startingDirLength
                 = startingDirectoryFullPath.length() + File.separator.length();

        String className;
        Class clazz;

        final Iterator iterator = list.iterator();
        while( iterator.hasNext() ) {
            className = ( (String)iterator.next() );
            className = className.substring( startingDirLength,
                    className.length() - ".class".length() );
            className = className.replace( File.separatorChar, '.' );

            try {
                clazz = Class.forName( className );
                if( TestCase.class.isAssignableFrom( clazz )
                         && testFilter_.accept( clazz ) ) {
                    set.add( clazz );
                }
            }
            catch( final ClassNotFoundException e ) {
                Trace.printStackTrace( Trace.out, e );
            }
        }
        return set;
    }


    /**
     * @param  directory Description of Parameter
     * @param  list Description of Parameter
     * @exception  IOException Description of Exception
     */
    private void findAllTestClasses( final File directory, final List list )
        throws IOException {

        final String files[] = directory.list();
        if( files == null ) {
            return;
        }

        final String directoryName = directory.getCanonicalPath();
        String fileName;

        int i;
        for( i = 0; i < files.length; i++ ) {
            fileName = directoryName + File.separator + files[i];
            if( files[i].endsWith( "Test.class" ) ) {
                list.add( fileName );
            }
            else {
                final File file = new File( fileName );
                if( file.isDirectory() ) {
                    findAllTestClasses( file, list );
                }
            }
        }
    }
}

