/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.testing;

import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;

/**
 * A simple bean class that can be used for testing purposes.  All
 * properties fire PropertyChangeEvents when changed.
 *
 * @version $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class SampleBean implements Serializable {

    // Required for serialization - do not modify or remove.
    private static final long serialVersionUID = 2000;

    private final PropertyChangeSupport propertyChangeSupport_;

    private String name_;
    private int age_;

    /**
     * Create a new instance.
     */
    public SampleBean() {
        this( "", 0 );
    }

    /**
     * Create a new instance.
     * @param name The name.
     * @param age The age.
     */
    public SampleBean( final String name, final int age ) {
        propertyChangeSupport_ = new PropertyChangeSupport(this);
        name_ = name;
        age_ = age;
    }

    /**
     * Add a property change listener
     * @param listener The listener
     */
    public void addPropertyChangeListener( final PropertyChangeListener listener ) {
        assertNotNull("listener", listener);
        propertyChangeSupport_.addPropertyChangeListener(listener);
    }

    /**
     * Remove a property change listener
     * @param listener The listener
     */
    public void removePropertyChangeListener( final PropertyChangeListener listener ) {
        assertNotNull("listener", listener);
        propertyChangeSupport_.removePropertyChangeListener(listener);
    }

    /**
     * Return the name.
     * @return The name
     */
    public String getName() {
        return name_;
    }

    /**
     * Set the name.
     * @param name The new name.
     */
    public void setName( final String name ) {
        assertNotNull("name", name);

        final String oldValue = name_;
        name_ = name;
        propertyChangeSupport_.firePropertyChange( "name", oldValue, name_ );
    }

    /**
     * Return the age.
     * @return The age.
     */
    public int getAge() {
        return age_;
    }

    /**
     * Set the age.
     * @param age The new age.
     */
    public void setAge( final int age ) {

        final int oldValue = age_;
        age_ = age;
        propertyChangeSupport_.firePropertyChange( "age", oldValue, age_ );
    }


    /**
     * Verify that the specified value is not null.  If it is then throw an exception
     *
     * @param fieldName The name of the field to check
     * @param fieldValue The value of the field to check
     * @exception DetailedNullPointerException If fieldValue is null
     */
    protected final void assertNotNull( final String fieldName, final Object fieldValue )
        throws DetailedNullPointerException {

        if( fieldValue == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}

