/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.trace;

import com.gargoylesoftware.base.util.StringUtil;
import java.io.PrintStream;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * A trace writer specifically designed for the bash shell.  The
 * reason it requires "bash" is that is colour codes the tracing
 * using bash specific escape codes.
 *
 * @version $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class BashTraceWriter implements TraceWriter {
    private static String lastPrefix_ = "";

    // Format used for the timestamp in the trace lines
    private static final Format TIMESTAMP_FORMAT
        = new SimpleDateFormat("HH:mm");


    /**
     * Create a new writer.
     */
    public BashTraceWriter() {
    }


    /**
     * Write the specified item.
     *
     * @param item The TraceItem to be written.
     */
    public void write( final TraceItem item ) {

        final PrintStream outStream
        = Trace.getController().getRealSystemOut();
        final StringBuffer prefixBuffer = new StringBuffer();

        // Bash shell specific - change colour to blue
        prefixBuffer.append("\033[0;34m");

        final Date timestamp = item.getTime();
        if( timestamp != null ) {
            prefixBuffer.append( "Time=[" );
            prefixBuffer.append( TIMESTAMP_FORMAT.format(timestamp) );
            prefixBuffer.append("] ");
        }

        final String threadName = item.getThreadName();
        if( threadName != null ) {
            prefixBuffer.append( "Thread=[" );
            prefixBuffer.append( threadName );
            prefixBuffer.append( "] " );
        }

        prefixBuffer.append("Channel=[");
        prefixBuffer.append(item.getChannel().getName());
        prefixBuffer.append("]");

        // Bash shell specific - revert colour
        prefixBuffer.append("\033[0m");

        final String prefix = prefixBuffer.toString();
        if( prefix.equals(lastPrefix_) == false ) {
            lastPrefix_ = prefix;

           outStream.println("");
            outStream.println(prefix);
        }
        final String message = item.getMessage();

        if( message != null ) {
            outStream.print("\033[0;34m>> \033[0m");
            outStream.print( StringUtil.expandTabs(message,3) );
        }

        final Throwable throwable = item.getThrowable();

        if( throwable != null ) {
            int i;
            final String strings[]
                = Trace.throwableToStringArray(throwable);

            outStream.print("\033[0;34m>> \033[0m");
            outStream.println(strings[0]);

            for( i=1; i<strings.length; i++ ) {
                outStream.println( StringUtil.expandTabs(strings[i],3) );
            }
        }
    }
}

