/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.trace;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.io.Serializable;

/**
 * <p style="color: orange">Internal use only.</p>.
 * <p>A circular queue with blocking semantics.</p>
 *
 * @version  $Revision: 1.7 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class BlockingCircularQueue
         implements
        Serializable {

    // Required for serialization - do not remove or modify
    private static final long serialVersionUID = 2206052328240171051L;

    // Locks for blocking on add/next
    private final Object getLock_ = new Object();
    private final Object addLock_ = new Object();

    // The actual storage
    private final Object queue_[];

    private final int capacity_;
    private int size_;

    // Objects get added to the front and removed from the back
    private int front_;
    private int back_;


    /**
     *  Create the queue with a capacity of 20
     */
    public BlockingCircularQueue() {
        this( 20 );
    }



    /**
     *  Create the queue with the specified capacity
     *
     *@param  capacity The size of the queue
     */
    public BlockingCircularQueue( final int capacity ) {
        queue_ = new Object[capacity];
        capacity_ = capacity;
        if( capacity < 2 ) {
            throw new DetailedIllegalArgumentException( "capacity", capacity, "Must be larger than two" );
        }
    }


    /**
     *  Remove all items from the queue
     */
    public synchronized void clear() {
        front_ = 0;
        back_ = 0;
        size_ = 0;

        // Remove any current object references so that they can be
        // garbage collected.
        int i;
        for( i = 0; i < capacity_; i++ ) {
            queue_[i] = null;
        }
    }


    /**
     *  Add an object to the queue. If the collection is currently full then
     *  block until an object is removed.
     *
     * @param object The object to add.
     * @return true if the object was successfully added.
     */
    public boolean add( final Object object ) {
        assertNotNull( "object", object );
        while( true ) {
            synchronized( this ) {
                if( size_ != capacity_ ) {
                    if( size_ == 0 ) {
                        front_ = 0;
                        back_ = 0;
                    }
                    else {
                        front_ = incrementPosition( front_ );
                    }
                    queue_[front_] = object;
                    synchronized( getLock_ ) {
                        getLock_.notify();
                    }
                    size_++;
                    return true;
                }
            }
            synchronized( addLock_ ) {
                try {
                    addLock_.wait();
                }
                catch( final InterruptedException e ) {
                    //TODO: We should be doing something here
                }
            }
        }
    }


    /**
     *  Return the next item in the queue. The returned item is removed from the
     *  collection. If no objects are present then block until an object has
     *  been added.
     *
     * @return The next item.
     */
    public Object next() {
        while( true ) {
            // If there's something in the queue then return it now.
            synchronized( this ) {
                if( isEmpty() == false ) {
                    final Object object = queue_[back_];
                    queue_[back_] = null;// help the garbage collector
                    back_ = incrementPosition( back_ );
                    synchronized( addLock_ ) {
                        addLock_.notify();
                    }
                    size_--;
                    return object;
                }
            }
            // There wasn't anything in the queue.  Block until something gets added.
            synchronized( getLock_ ) {
                try {
                    getLock_.wait();
                }
                catch( InterruptedException e ) {
                    //TODO: Do something here
                }
            }
        }
    }


    /**
     *  Utility method to increment the position and roll back to zero once we
     *  hit the end. Return the new position after the adjustment.
     * @param position The position to increment
     * @return the new position
     */
    private int incrementPosition( final int position ) {
        if( ( position < 0 ) || ( position >= capacity_ ) ) {
            throw new DetailedIllegalArgumentException(
                    "position", new Integer( position ), "Must be within 1 and " + capacity_ );
        }
        int newPosition = position + 1;
        if( newPosition == capacity_ ) {
            newPosition = 0;
        }
        return newPosition;
    }


    /**
     *  Return the number of objects currently in the collection.
     * @return The object count.
     */
    public int size() {
        return size_;
    }


    /**
     *  Return true if the collection is empty.
     * @return true if the collection is empty.
     */
    public boolean isEmpty() {
        return size_ == 0;
    }


    /**
     *  Return a string representation of this object.
     * @return a string representation of this object.
     */
    public String toString() {
        final StringBuffer buffer = new StringBuffer();
        buffer.append( getClass().getName() );
        buffer.append( ": capacity_=" );
        buffer.append( capacity_ );
        buffer.append( " size_=" );
        buffer.append( size_ );
        buffer.append( " front_=" );
        buffer.append( front_ );
        buffer.append( " back_=" );
        buffer.append( back_ );
        return buffer.toString();
    }


    /**
     *  Verify that the specified value is not null. If it is then throw an
     *  exception
     *
     *@param  fieldName                         The name of the field to check
     *@param  fieldValue                        The value of the field to check
     *@exception  DetailedNullPointerException  If fieldValue is null
     */
    protected final void assertNotNull( final String fieldName, final Object fieldValue )
             throws DetailedNullPointerException {
        if( fieldValue == null ) {
            throw new DetailedNullPointerException( fieldName );
        }
    }
}

