/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.trace;


//TODO: If a trace string contains newlines then prefix each line with the
//      required number of spaces
//TODO: Default channels should be thread specific.
//      - setDefaultChannel(Thread, Channel);
//      - setDefaultChannel(ThreadGroup, Channel);
//      - setDefaultChannel(Channel);

import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * A class that provides a mechanism for logging diagnostic messages.  This
 * mechanism is significantly faster then calling System.out.println directly
 * as the printing is done on a background thread which does not impact the
 * performance of the application threads.
 * <p>
 * The methods for tracing are print(String), println(String) and
 * printStackTrace(Throwable).  The basic idea is that we want the trace
 * methods to return to the working threads as quickly as possible so we
 * put the thing to be traced on a queue.  The queue is then processed by
 * a second thread which performs all the formatting and actual output.<p>
 *
 * Trace can be configured using the {@link TraceController} class - see
 * {@link #getController()}.  It is possible to redirect System.out and
 * System.err so that calling System.out.println("foo") will be the same
 * as calling Trace.println("foo")
 * <pre>
 * Trace.getController().setOutRedirected(true);
 * Trace.getController().setErrRedirected(true);
 * </pre>
 *
 * Calls to any of the print methods will print to a {@link TraceChannel}
 * which in turn will print using whatever {@link TraceWriter}s have been
 * added to it.  Print methods that do not take a {@link TraceChannel}
 * will use the default channel - see
 * {@link TraceController#setDefaultChannel(TraceChannel)}<p>
 *
 * <b>If you are using this code in a JVM prior to 1.3 then you need to read this:</b>
 * <br />Because trace lines are being processed on a second thread, there
 * might still be trace messages in the queue when the VM exits.  To solve
 * this problem we introduce the method {@link #exit(int)} which will shut
 * down the second thread and then call System.exit().  If you are running on
 * a java 1.3 VM or higher then it is not neccessary to call {@link #exit(int)}
 * as Trace will automatically install a shutdown hook which will do the
 * neccessary cleanup.<p>
 *
 * @version $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class Trace {
    /** An exception used to determine where the code is at any point in time. */
    public static class WhereAmIException extends Exception {
        private static final long serialVersionUID = 6876505080685244104L;

		/** Instantiate one */
        public WhereAmIException() {
            super("?? Where am I ??");
        }
    }

    /**
     * The equivilent of "standard out"
     */
    public static final TraceChannel out = new TraceChannel("STDOUT");
    /**
     * The equivilent of "standard error"
     */
    public static final TraceChannel err = new TraceChannel("STDERR");

    // line separator used for println
    private static final String LINE_SEPARATOR = System.getProperty("line.separator");

    private static final TraceItemDispatcher TRACE_ITEM_DISPATCHER = new TraceItemDispatcher();
    private static final TraceController TRACE_CONTROLLER = new TraceController();


    /**
     * Private constructor to prevent instantiation of this class.
     */
    private Trace() {
    }

    /**
     * Print a line to the specified channel.
     * If the channel is null then nothing will be printed.
     * @param channel The trace channel to use
     * @param string The text string to write.
     */
    public static void print( final TraceChannel channel, String string ) {
        assertNotNull("channel", channel);
        if( string == null ) {
            string = "<null>";
        }

        final TraceItemDispatcher dispatcher = getDispatcher();
        final TraceItem item = dispatcher.getNewTraceItem();

        item.setMessage(string);
        item.setChannel(channel);
        dispatcher.dispatch(item);
    }

    /**
     * Print a line to the default channel
     * If the channel is null then nothing will be printed.
     * @param string The text string to write.
     */
    public static void print( final String string ) {
        print( getController().getDefaultChannel(), string );
    }

    /**
     * Print the line to the specified channel with a new line at the end.
     * If the channel is null then nothing will be printed.
     * @param channel The trace channel to use
     * @param string the string to write.
     */
    public static void println( final TraceChannel channel, final String string ) {
        if( channel == null ) {
            return;
        }
        print( channel, string + LINE_SEPARATOR );
    }

    /**
     * Print the line to the default channel with a new line at the end.
     *
     * @param string the string to write.
     */
    public static void println( final String string ) {
        print( getController().getDefaultChannel(), string + LINE_SEPARATOR );
    }

    /**
     * Print the stack trace to the specified channel.
     * If the channel is null then nothing will be printed.
     *
     * @param channel The trace channel to use
     * @param throwable The exception to print
     */
    public static void printStackTrace( final TraceChannel channel,
                                        final Throwable throwable ) {

        if( channel == null ) {
            return;
        }
        if( throwable == null ) {
            println( channel, "<null exception>" );
            return;
        }

        final TraceItemDispatcher dispatcher = getDispatcher();
        final TraceItem item = dispatcher.getNewTraceItem();

        item.setThrowable(throwable);
        item.setChannel(channel);
        dispatcher.dispatch(item);
    }

    /**
     * Print the stack trace to the default channel.
     * @param throwable The exception to print.
     */
    public static void printStackTrace( final Throwable throwable ) {
        printStackTrace( getController().getDefaultChannel(), throwable );
    }

    /**
     * Print the specified lines to the default trace channel.
     * @param lines The lines to print.
     */
    public static void printLines( final String lines[] ) {
        printLines( getController().getDefaultChannel(), lines );
    }

    /**
     * Print the specified lines to the trace channel.
     * If the channel is null then nothing will be printed.
     *
     * @param channel The trace channel to use
     * @param lines the lines to print.
     */
    public static void printLines( final TraceChannel channel, final String lines[] ) {
        if( channel == null ) {
            return;
        }
        if( lines == null ) {
            println( channel, "<null lines>" );
            return;
        }

        //TODO: Send the array as one request to the dispatcher so that the prefix
        //      string is only printed once for the array
        int i;
        for( i=0; i<lines.length; i++ ) {
            println( channel, lines[i] );
        }
    }

    /**
     * Print a stack trace to show where we came from.  It will be printed to the default
     * channel.
     */
    public static void whereAmI() {
        whereAmI( getController().getDefaultChannel() );
    }

    /**
     * Print a stack trace to show where we came from. If the channel is null then
     * nothing will be printed.
     * @param channel The trace channel to use.
     */
    public static void whereAmI( final TraceChannel channel ) {
        if( channel == null ) {
            return;
        }

        printStackTrace( channel, new WhereAmIException() );
    }

    /**
     * Dump the stack trace from the specified throwable into an array
     * of strings where each line in the trace is a separate string.
     * Additionally, expand all tabs to spaces.
     *
     * @param t The exception.
     * @return The resulting string.
     */
    public static String[] throwableToStringArray( final Throwable t ) {
        assertNotNull("t", t);

        String token;
        char c;
        StringBuffer buffer;
        int length;
        int i;

        final List vector = new ArrayList();
        final StringTokenizer tokenizer
        = new StringTokenizer( throwableToString(t), "\n");

        while( tokenizer.hasMoreTokens() ) {
            token = tokenizer.nextToken();

            // Now expand the tabs
            length = token.length();
            buffer = new StringBuffer(length*2);
            for( i=0; i<length; i++ ) {
                c = token.charAt(i);
                if( c == '\t' ) {
                    buffer.append("   ");
                }
                else {
                    buffer.append(c);
                }
            }
            vector.add(buffer.toString());
        }

        final String array[] = new String[vector.size()];
        vector.toArray(array);
        return array;
    }

    /**
     * Dump the stack trace from the specified throwable into a String.
     *
     * @param t The exception.
     * @return The resulting string.
     */
    public static String throwableToString( final Throwable t ) {
        assertNotNull("t", t);

        final StringWriter writer = new StringWriter();
        final PrintWriter printWriter = new PrintWriter(writer);
        t.printStackTrace(printWriter);
        printWriter.close();
        return writer.toString();
    }

    /**
     * Return the controller object for the debugging stuff
     *
     * @return The controller in use.
     */
    public static TraceController getController() {
        return TRACE_CONTROLLER;
    }

    /** @return The dispatcher */
    static TraceItemDispatcher getDispatcher() {
        return TRACE_ITEM_DISPATCHER;
    }

    /**
     * Flush the trace queue.
     */
    public static void flush() {
        getDispatcher().flush();
    }

    /**
     * Shutdown all buffering and exit the VM with the specified
     * status code.
     * <p>
     * <b>Note</b> This is no longer needed if you are running JDK1.3 or higher as
     * we now register a shutdown hook to disable buffering before the VM exits.
     * @param statusCode The status code returned when the application exits.
     */
    public static void exit( final int statusCode ) {
        getController().setBufferingEnabled(false);
        System.exit(statusCode);
    }


    /**
     * Verify that the specified value is not null.  If it is then throw an exception
     *
     * @param fieldName The name of the field to check
     * @param fieldValue The value of the field to check
     * @exception DetailedNullPointerException If fieldValue is null
     */
    protected static final void assertNotNull( final String fieldName, final Object fieldValue )
        throws DetailedNullPointerException {

        if( fieldValue == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}

