/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.trace;

import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashSet;
import java.util.Set;

/**
 * This class represents a "channel" that diagnostic messages can be written to.  The
 * channel will contain zero or more TraceWriters which will be responsible for handling
 * the messages.
 *
 * @version $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class TraceChannel {
    private final String name_;
    private boolean isEnabled_ = true;
    private PropertyChangeSupport propertyChangeSupport_ = null;
    private final Set traceWriters_ = new HashSet();

    /**
     * Create a trace channel with the specified name.
     *
     * @param name The name of the channel.
     */
    public TraceChannel( final String name ) {
        assertNotNull("name", name);
        name_ = name;
    }

    /**
     * Return the name of this trace channel.
     *
     * @return the name.
     */
    public final String getName() {
        return name_;
    }

    /**
     * Return a string representation of this object.
     * @return a string representation of this object.
     */
    public String toString() {
        final StringBuffer buffer = new StringBuffer();
        buffer.append( getClass().getName() );
        buffer.append( "[name=[" );
        buffer.append( getName() );
        buffer.append( "] isEnabled=[");
        buffer.append( isEnabled_ );
        buffer.append( "]]" );

        return buffer.toString();
    }

    /**
     * Set whether or not this channel is enabled.  If it is not enabled
     * then any requests sent to this channel will be discarded.
     *
     * @param isEnabled true if this channel should be enabled.
     */
    public void setEnabled( final boolean isEnabled ) {
        final boolean oldValue = isEnabled_;
        isEnabled_ = isEnabled;

        if( propertyChangeSupport_ != null && oldValue != isEnabled_ ) {
            propertyChangeSupport_.firePropertyChange("enabled",
                                                      isEnabled_,
                                                      oldValue );
        }
    }

    /**
     * Return true if this channel is enabled.
     * @return true if this channel is enabled.
     */
    public boolean isEnabled() {
        return isEnabled_;
    }

    /**
     * Add a property change listener.
     *
     * @param listener The new listener
     */
    public void addPropertyChangeListener( final PropertyChangeListener listener ) {
        if( propertyChangeSupport_ == null ) {
            synchronized(this) {
                if( propertyChangeSupport_ == null ) {
                    propertyChangeSupport_ = new PropertyChangeSupport(this);
                }
            }
        }
        propertyChangeSupport_.addPropertyChangeListener(listener);
    }

    /**
     * Remove a property change listener.
     *
     * @param listener The listener to remove
     */
    public void removePropertyChangeListener( final PropertyChangeListener listener ) {
        if( propertyChangeSupport_ != null ) {
            propertyChangeSupport_.removePropertyChangeListener(listener);
        }
    }

    /**
     * Return a collection of trace writers
     * @return The trace writers
     */
    /*package*/ Set getTraceWriters() {
        return traceWriters_;
    }

    /**
     * Add a trace writer.
     * @param writer A trace writer
     */
    public void addTraceWriter( final TraceWriter writer ) {
        assertNotNull("writer", writer);
        synchronized( traceWriters_ ) {
            traceWriters_.add(writer);
        }
    }

    /**
     * Remove a trace writer.
     * @param writer A trace writer
     */
    public void removeTraceWriter( final TraceWriter writer ) {
        assertNotNull("writer", writer);
        synchronized( traceWriters_ ) {
            traceWriters_.remove(writer);
        }
    }


    /**
     * Verify that the specified value is not null.  If it is then throw an exception
     *
     * @param fieldName The name of the field to check
     * @param fieldValue The value of the field to check
     * @exception DetailedNullPointerException If fieldValue is null
     */
    protected final void assertNotNull( final String fieldName, final Object fieldValue )
        throws DetailedNullPointerException {

        if( fieldValue == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}
