/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.trace;

import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.io.PrintStream;

/**
 *  A controller object for the tracing mechanism.
 *
 *@version    $Revision: 1.4 $
 *@author     <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class TraceController {

    private PrintStream systemOut_;
    private PrintStream systemErr_;

    private TraceChannel defaultChannel_ = Trace.out;


    /**
     *  Instantiate one
     */
    TraceController() {
    }


    /**
     *  Shutdown the tracing thread and flush the buffers. Any print calls after
     *  this method has been called will be written immediately on the thread
     *  that made the call.
     *
     *@param  enabled  true if buffering is to be enabled.
     */
    public void setBufferingEnabled( final boolean enabled ) {
        Trace.getDispatcher().setBufferingEnabled( enabled );
    }


    /**
     *  Return true if buffering is enabled.
     *
     *@return    true if buffering is enabled.
     */
    public boolean isBufferingEnabled() {
        return Trace.getDispatcher().isBufferingEnabled();
    }


    /**
     *  Specify whether or not System.out should be redirected to print through
     *  Trace.println
     *
     *@param  redirected  true if System.out should be redirected.
     */
    public synchronized void setOutRedirected( final boolean redirected ) {
        if( redirected == isOutRedirected() ) {
            // nothing has changed.  return immediately
            return;
        }

        if( redirected ) {
            systemOut_ = System.out;
            System.setOut( new PrintStream( new TraceOutputStream() ) );
        }
        else {
            System.setOut( systemOut_ );
            systemOut_ = null;
        }

    }


    /**
     *  Return true if System.out has been redirected to print through
     *  Trace.println
     *
     *@return    true if System.out has been redirected.
     */
    public boolean isOutRedirected() {
        return systemOut_ != null;
    }


    /**
     *  Return the real stream that corresponds to the console for System.out.
     *  If System.out has been redirected then this method will return the
     *  original value.
     *
     *@return    the real System.out
     */
    public PrintStream getRealSystemOut() {
    		final PrintStream result;
        if( systemOut_ == null ) {
            result = System.out;
        }
        else {
            result = systemOut_;
        }
        return result;
    }


    /**
     *  Specify whether or not System.err should be redirected to print through
     *  Trace.println
     *
     *@param  redirected  true if System.err should be redirected.
     */
    public synchronized void setErrRedirected( final boolean redirected ) {
        if( redirected == isErrRedirected() ) {
            // nothing has changed.  return immediately
            return;
        }

        if( redirected ) {
            systemErr_ = System.err;
            System.setErr( new PrintStream( new TraceOutputStream() ) );
        }
        else {
            System.setErr( systemErr_ );
            systemErr_ = null;
        }

    }


    /**
     *  Return true if System.err has been redirected to print through
     *  Trace.println
     *
     *@return    true if System.err has been redirected.
     */
    public boolean isErrRedirected() {
        return systemErr_ != null;
    }


    /**
     *  Return the real stream that corresponds to the console for System.err.
     *  If System.err has been redirected then this method will return the
     *  original value.
     *
     *@return    the real System.err
     */
    public PrintStream getRealSystemErr() {
    		final PrintStream result;
        if( systemErr_ == null ) {
            result = System.err;
        }
        else {
            result = systemErr_;
        }
        return result;
    }


    /**
     *  Close down the debugging facilities in preparation for application
     *  shutdown. This will disable the buffering and turn off redirections for
     *  System.out and System.err.
     */
    public void close() {
        setBufferingEnabled( false );
        setErrRedirected( false );
        setOutRedirected( false );
    }


    /**
     *  Set the default channel. The default is used when a channel is not
     *  specified in a call to Trace.print(), Trace.println() or
     *  Trace.printStackTrace()
     *
     *@param  channel  the new channel.
     */
    public void setDefaultChannel( final TraceChannel channel ) {
        assertNotNull( "channel", channel );
        defaultChannel_ = channel;
    }


    /**
     *  Return the default channel
     *
     *@return    the default channel.
     */
    public TraceChannel getDefaultChannel() {
        return defaultChannel_;
    }


    /**
     *  Verify that the specified value is not null. If it is then throw an
     *  exception
     *
     *@param  fieldName                         The name of the field to check
     *@param  fieldValue                        The value of the field to check
     *@exception  DetailedNullPointerException  If fieldValue is null
     */
    protected final void assertNotNull( final String fieldName, final Object fieldValue )
             throws DetailedNullPointerException {

        if( fieldValue == null ) {
            throw new DetailedNullPointerException( fieldName );
        }
    }
}

