/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.trace;

import com.gargoylesoftware.base.util.DetailedNullPointerException;

/**
 * <p style="color: orange">Internal use only.</p>.
 *
 * @version $Revision: 1.7 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class TraceItemQueue {
    private TraceItem firstItem_;
    private TraceItem lastItem_;
    private int itemCount_;

    /**
     *
     */
    public TraceItemQueue() {
        firstItem_ = null;
        lastItem_ = null;
        itemCount_ = 0;
    }

    /**
     * @param item The item to push.
     */
    public synchronized void push( final TraceItem item ) {
        assertNotNull("item", item);
        if( item.getPrevItem() != null || item.getNextItem() != null ) {
            throw new IllegalArgumentException("Pointers are not null");
        }

        if( itemCount_ == 0 ) {
            lastItem_ = item;
        }

        if( firstItem_ != null ) {
            firstItem_.setPrevItem( item );
        }
        item.setNextItem( firstItem_ );
        firstItem_ = item;
        itemCount_++;
    }

    /**
     * Return the next item on the queue or null if the queue is empty
     * @return The next item or null.
     */
    public synchronized TraceItem pop() {
        //      System.out.println("***TraceItemQueue.pop() "+this);
        checkValidity();
        final TraceItem popItem = lastItem_;

        if( popItem != null ) {
            lastItem_ = lastItem_.getPrevItem();
            if( lastItem_ == null ) {
                firstItem_ = null;
            }
            else {
                lastItem_.setNextItem( null );
            }
            popItem.setPrevItem( null );
            popItem.setNextItem( null );
            itemCount_--;
        }

        return popItem;
    }

    private void checkValidity() {
        if( itemCount_ == 0 ) {
            if( firstItem_ != null ) {
                throw new IllegalStateException("firstItem should be null: "+this);
            }
            if( lastItem_ != null ) {
                throw new IllegalStateException("lastItem should be null: "+this);
            }
        }
    }
    /**
     * @return the number of items in the queue.
     */
    public synchronized int size() {
        return itemCount_;
    }

    /**
     * @return The string representation of this object.
     */
    public String toString() {
        return getClass().getName()+" itemCount=["+itemCount_
               +"] firstItem=["+firstItem_+"] lastItem=["+lastItem_+"]";
    }


    /**
     * Verify that the specified value is not null.  If it is then throw an exception
     *
     * @param fieldName The name of the field to check
     * @param fieldValue The value of the field to check
     * @exception DetailedNullPointerException If fieldValue is null
     */
    protected final void assertNotNull( final String fieldName, final Object fieldValue )
        throws DetailedNullPointerException {

        if( fieldValue == null ) {
            throw new DetailedNullPointerException(fieldName);
        }
    }
}

