/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.util;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
 *  A class to walk through the directory structure from a given starting point
 *  and return either files or directories or both.<p>
 *
 * The following sample gets all java files.
 * <pre>
 * final FileFilter filter = new FileFilter() {
 *     public boolean accept( final File file ) {
 *         return file.getName().endsWith(".java");
 *     }
 * };
 *
 * final DirectoryWalker directoryWalker = new DirectoryWalker(".");
 * final Collection files = directoryWalker.getFiles(filter);
 * </pre>
 *
 * @version  $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DirectoryWalker {
    private final File startingDirectory_;


    /**
     *  Create an instance
     *
     * @param  startingDirectory the directory to start in
     */
    public DirectoryWalker( final String startingDirectory ) {
        if( startingDirectory == null ) {
            throw new NullPointerException();
        }
        if( startingDirectory.length() == 0 ) {
            throw new DetailedIllegalArgumentException( "startingDirectory", startingDirectory, "May not be empty" );
        }
        startingDirectory_ = new File( startingDirectory );
        if( startingDirectory_.exists() == false ) {
            throw new DetailedIllegalArgumentException( "startingDirectory", startingDirectory, "Doesn't exist" );
        }
        if( startingDirectory_.isDirectory() == false ) {
            throw new DetailedIllegalArgumentException( "startingDirectory", startingDirectory, "Not a directory" );
        }
    }


    /**
     *  Walk through the directory structure and return a collection containing
     *  all those files for which the filter returns true
     *
     * @param  filter An object to determine whether or not to include this file
     *      in the returned collection
     * @return  A collection of File objects
     */
    public Collection getFiles( final FileFilter filter ) {

        return walk( filter, true, false );
    }


    /**
     *  Walk through the directory structure and return a collection containing
     *  all those directories for which the filter returns true
     *
     * @param  filter An object to determine whether or not to include this
     *      directory in the returned collection
     * @return  A collection of File objects
     */
    public Collection getDirectories( final FileFilter filter ) {
        return walk( filter, false, true );
    }


    /**
     *  Walk through the directory structure and return a collection containing
     *  all those files and directories for which the filter returns true
     *
     * @param  filter An object to determine whether or not to include this
     *      file/directory in the returned collection
     * @return  A collection of File objects
     */
    public Collection getFilesAndDirectories( final FileFilter filter ) {
        return walk( filter, true, true );
    }


    private Collection walk(
            final FileFilter filter,
            final boolean includeFiles,
            final boolean includeDirectories ) {

        int i;
        final Collection results = new ArrayList();
        final List pendingDirectories = new LinkedList();
        pendingDirectories.add( startingDirectory_ );

        File dir;

        if( includeDirectories && filter.accept( startingDirectory_ ) ) {
            results.add( startingDirectory_ );
        }

        while( pendingDirectories.isEmpty() == false ) {
            dir = (File)pendingDirectories.get( 0 );
            pendingDirectories.remove( 0 );

            final File[] files = dir.listFiles();
            for( i = 0; i < files.length; i++ ) {
                final File currentFile = files[i];
                if( includeFiles && currentFile.isFile() && filter.accept( currentFile ) ) {
                    results.add( currentFile );
                }
                else if( currentFile.isDirectory() ) {
                    if( includeDirectories && filter.accept( currentFile ) ) {
                        results.add( currentFile );
                    }

                    pendingDirectories.add( currentFile );
                }
            }
        }

        return results;
    }
}

