/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.util;

import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.PlainDocument;
import javax.swing.text.Position;
import javax.swing.text.Segment;

/**
 * Various utility methods relating to documents.
 *
 * @version $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DocumentUtil {

    /**
     * Private constructor to prevent instantiation of this class.
     */
    private DocumentUtil() {
    }

    /**
     * Return a wrapper document that converts all input to uppercase.
     * @param document The document to wrapper.
     * @return The new wrapper document.
     */
    public static Document upperCaseDocument( final Document document ) {
        return new PassthroughDocument(document) {
            public void insertString(
                    final int offset,
                    final String str,
                    final AttributeSet a)
                throws
                    BadLocationException {

                super.insertString(offset, str.toUpperCase(), a);
            }
        };
    }

    /**
     * Return a PlainDocument wrappered so that it accepts uppercase
     * input only.
     * @return The new wrapper document.
     */
    public static Document upperCaseDocument() {
        return upperCaseDocument( new PlainDocument() );
    }

    /**
     * Return a wrapper document that prevents the user from typing more
     * than a specified number of characters.
     * @param document The document to wrapper.
     * @param maxLength The maximum number of characters that can be placed in
     * this document.
     * @return The new wrapper document.
     */
    public static Document lengthLimitedDocument( final Document document,
                                                  final int maxLength ) {
        return new LengthLimitedDocument(document,maxLength);
    }

    /**
     * Return a PlainDocument wrappered so that it is length limited.
     * @param maxLength The maximum number of characters that can be placed in
     * this document.
     * @return The new wrapper document.
     */
    public static Document lengthLimitedDocument( final int maxLength ) {
        return lengthLimitedDocument( new PlainDocument(), maxLength );
    }

    /**
     * A wrapper document that prevents the user from typing more than
     * a specified number of characters.
     */
    private static class LengthLimitedDocument extends PassthroughDocument {
        private int maxLength_;

        /**
         * Create an instance
         * @param document The document to wrap
         * @param maxLength The maximum number of characters that can be entered into this document
         */
        public LengthLimitedDocument( final Document document,
                                      final int maxLength ) {
            super(document);
            if( maxLength < 0 ) {
                throw new DetailedIllegalArgumentException(
                    "maxLength", new Integer(maxLength), "May not be less than zero");
            }
            maxLength_ = maxLength;
        }
        /** @inheritDoc Document.insertString(int,String,AttributeSet) */
        public void insertString(
                final int offset,
                final String string,
                final AttributeSet attributeSet)
            throws
                BadLocationException {

            final int documentLength = getLength();
            final int stringLength = string.length();

            if( documentLength == maxLength_ ) {
                return;
            }
            else if( documentLength + stringLength <= maxLength_ ) {
                super.insertString( offset, string, attributeSet );
            }
            else {
                super.insertString( offset,
                                    string.substring(0, maxLength_ - documentLength),
                                    attributeSet );
            }
        }
    }

    /**
     * A document that merely passes all requests through to another document.
     * This is used as the base class for the custom documents provided here.
     */
    private abstract static class PassthroughDocument implements Document {
        private final Document delegate_;

        /**
         * Create an instance
         * @param delegate The document to wrap.
         */
        public PassthroughDocument( final Document delegate ) {
            if( delegate == null ) {
                throw new DetailedNullPointerException("delegate");
            }
            delegate_ = delegate;
        }
        /** @inheritDoc Document.getLength() */
        public int getLength() {
            return delegate_.getLength();
        }

        /** @inheritDoc Document.addDocumentListener(DocumentListener) */
        public void addDocumentListener(DocumentListener listener) {
            delegate_.addDocumentListener(listener);
        }

        /** @inheritDoc Document.removeDocumentListener(DocumentListener) */
        public void removeDocumentListener(DocumentListener listener) {
            delegate_.removeDocumentListener(listener);
        }

        /** @inheritDoc Document.addUndoableEditListener(UndoableEditListener) */
        public void addUndoableEditListener(UndoableEditListener listener) {
            delegate_.addUndoableEditListener(listener);
        }

        /** @inheritDoc Document.removeUndoableEditListener(UndoableEditListener) */
        public void removeUndoableEditListener(UndoableEditListener listener) {
            delegate_.removeUndoableEditListener(listener);
        }

        /** @inheritDoc Document.getProperty(Object) */
        public Object getProperty(Object key) {
            return delegate_.getProperty(key);
        }

        /** @inheritDoc Document.putProperty(Object,Object) */
        public void putProperty(Object key, Object value) {
            delegate_.putProperty(key,value);
        }

        /** @inheritDoc Document.remove(int,int) */
        public void remove(int offs, int len) throws BadLocationException {
            delegate_.remove(offs, len);
        }

        /** @inheritDoc Document.insertString(int,String,AttributeSet) */
        public void insertString(int offset, String str, AttributeSet a) throws BadLocationException {
            delegate_.insertString(offset, str, a);
        }

        /** @inheritDoc Document.getText(int,int) */
        public String getText(int offset, int length) throws BadLocationException {
            return delegate_.getText(offset, length);
        }

        /** @inheritDoc Document.getText(int,int,Segment) */
        public void getText(int offset, int length, Segment txt) throws BadLocationException {
            delegate_.getText(offset, length, txt);
        }

        /** @inheritDoc Document.getStartPosition() */
        public Position getStartPosition() {
            return delegate_.getStartPosition();
        }

        /** @inheritDoc Document.getEndPosition() */
        public Position getEndPosition() {
            return delegate_.getEndPosition();
        }

        /** @inheritDoc Document.createPosition(int) */
        public Position createPosition(int offs) throws BadLocationException {
            return delegate_.createPosition(offs);
        }

        /** @inheritDoc getRootElements() */
        public Element[] getRootElements() {
            return delegate_.getRootElements();
        }

        /** @inheritDoc Document.getDefaultRootElement() */
        public Element getDefaultRootElement() {
            return delegate_.getDefaultRootElement();
        }

        /** @inheritDoc Document.render(Runnable) */
        public void render(Runnable r) {
            delegate_.render(r);
        }
    }
}

