/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.util;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

/**
 * A collection of utility methods dealing with proxy objects
 *
 * @version $Revision: 1.3 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public final class ProxyUtil {
    // Private constructor to prevent instantiation
    private ProxyUtil() {
    }

    /**
     * Create a proxy object that implements the interface specified by 'clazz'.
     * All methods invoked on this object will return null.
     *
     * @param clazz The interface that the proxy will support
     * @return The new proxy object.
     */
    public static Object createProxy( final Class clazz ) {
        return createProxy( clazz, new Object[0][] );
    }


    /**
     * Create a proxy object that implements the interface specified by 'clazz'.
     * Any methods specified in args will return the given result, any methods
     * not specified here will return null.
     * <pre>
     * final Object args[][] = {
     *     {"getAutoCommit", Boolean.TRUE},
     *     {"getCatalog", "foobar"}
     * };
     * final Connection connection = ProxyUtil.createProxy(Connection.class, args);
     * </pre>
     * Calling getAutoCommit() on this object will return true.  Calling getCatalog()
     * on this object will return "foobar".  Calling any other method will return null.
     *
     * @param clazz The interface that will be supported by the generated proxy
     * @param args methodname/object pairs
     * @return The new proxy object
     */
    public static Object createProxy( final Class clazz, final Object args[][] ) {
        if( clazz == null ) {
            throw new NullPointerException("clazz");
        }
        if( args == null ) {
            throw new NullPointerException("args");
        }

        final InvocationHandler handler = new InvocationHandler() {
            public Object invoke(Object proxy, Method method, Object[] methodArgs) {
                final String methodName = method.getName();

                int i;
                for( i=0; i<args.length; i++ ) {
                    if( methodName.equals(args[i][0]) ) {
                        return args[i][1];
                    }
                }

                if( methodName.equals("toString") ) {
                    return "Proxy["+clazz.getName()+"]";
                }

                return null;
            }
        };
        return Proxy.newProxyInstance(
            clazz.getClassLoader(),
            new Class[] { clazz },
            handler);
    }
}
