/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.collections;

import com.gargoylesoftware.base.testing.EventCatcher;
import com.gargoylesoftware.base.testing.EventCatcherRecord;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import junit.framework.TestCase;
import java.util.ArrayList;
import java.util.List;
import java.util.EventObject;

/**
 * Tests for NotificationList.
 *
 * @version $Revision: 1.7 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class NotificationListTest extends TestCase {

    /**
     * Create a new test.
     * @param name The name of the test.
     */
    public NotificationListTest( final String name ) {
        super(name);
    }

    /**
     * Test the constructor with a null delegate.
     */
    public void testConstructor_Null() {
        try {
            new NotificationList(null);
            fail("Expected exception for null parameter");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("delegate", e.getArgumentName());
        }
    }

    /**
     * Test adding a null listener.
     */
    public void testAddListener_Null() {
        final NotificationList list = new NotificationList(new ArrayList());
        try {
            list.addNotificationListListener(null);
            fail("Expected exception for null parameter");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("listener", e.getArgumentName());
        }
    }

    /**
     * Test removing a null listener.
     */
    public void testRemoveListener_Null() {
        final NotificationList list = new NotificationList(new ArrayList());
        try {
            list.removeNotificationListListener(null);
            fail("Expected exception for null parameter");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("listener", e.getArgumentName());
        }
    }

    /**
     * Test adding a new object.
     * @throws Exception if the test fails
     */
    public void testAdd() throws Exception {
        final NotificationList list = new NotificationList(new ArrayList());
        final EventCatcher eventCatcher = new EventCatcher();

        eventCatcher.listenTo(list);

        list.add( "zero" );
        assertEquals( "eventCatcher.size()", 1, eventCatcher.getEventCount() );
        checkRecord(eventCatcher,0,0,0,1, NotificationListEvent.INSERT);

        list.add("one");
        assertEquals( "eventCatcher.size()", 2, eventCatcher.getEventCount() );
        checkRecord(eventCatcher,1,1,1,1, NotificationListEvent.INSERT);

        list.add( 1, "one-half" );
        assertEquals( "eventCatcher.size()", 3, eventCatcher.getEventCount() );
        checkRecord(eventCatcher,2,1,1,1, NotificationListEvent.INSERT);
    }

    /**
     * Test adding a null collection.
     */
    public void testAddAll_NullCollection() {
        final NotificationList list = new NotificationList(new ArrayList());
        try {
            list.addAll(null);
            fail("Expected exception");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("collection", e.getArgumentName());
        }
    }

    /**
     * Test adding a new collection at a specified index.
     */
    public void testAddAll_Index_NullCollection() {
        final NotificationList list = new NotificationList(new ArrayList());
        try {
            list.addAll(0, null);
            fail("Expected exception");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("collection", e.getArgumentName());
        }
    }

    /**
     * Test addAll().
     * @throws Exception if the test fails
     */
    public void testAddAll() throws Exception {
        final List listToAdd = new ArrayList();
        listToAdd.add("Popeye");
        listToAdd.add("Olive Oyl");
        listToAdd.add("Wimpy");

        final NotificationList list = new NotificationList(new ArrayList());
        final EventCatcher eventCatcher = new EventCatcher();

        eventCatcher.listenTo(list);

        list.addAll( listToAdd );
        assertEquals( "eventCatcher.size()", 1, eventCatcher.getEventCount() );
        checkRecord(eventCatcher,0,0,0,3, NotificationListEvent.INSERT);

        list.addAll( listToAdd );
        assertEquals( "eventCatcher.size()", 2, eventCatcher.getEventCount() );
        checkRecord(eventCatcher,1,3,3,3, NotificationListEvent.INSERT);

        // Now test adding at a specific offset
        list.addAll( 4, listToAdd );
        assertEquals( "eventCatcher.size()", 3, eventCatcher.getEventCount() );
        checkRecord(eventCatcher,2,4,4,3, NotificationListEvent.INSERT);
    }

    /**
     * Check one record
     * @param eventCatcher The event catcher
     * @param recordIndex The record index
     * @param startIndex The start index
     * @param endIndex The end index
     * @param elementCount The element count
     * @param action The action
     */
    private void checkRecord( final EventCatcher eventCatcher,
                              final int recordIndex,
                              final int startIndex,
                              final int endIndex,
                              final int elementCount,
                              final int action ) {
        final EventCatcherRecord record = eventCatcher.getEventCatcherRecordAt(recordIndex);

        final EventObject eventObject = record.getEvent();
        if( eventObject instanceof NotificationListEvent  == false ) {
            fail("Expected instance of NotificationListEvent but got: "
                 + eventObject.getClass().getName() );
        }
        final NotificationListEvent event = (NotificationListEvent)eventObject;

        final String prefix = "record["+recordIndex+"] ";
        assertEquals( prefix+"startIndex", event.getStartIndex(), startIndex );
        assertEquals( prefix+"endIndex", event.getEndIndex(), endIndex );
        assertEquals( prefix+"action", event.getAction(), action );

        switch( action ) {
            case NotificationListEvent.INSERT:
                assertEquals( prefix+"oldValues().size()",
                              event.getOldValues().size(),
                              0 );
                assertEquals( prefix+"newValues().size()",
                              event.getNewValues().size(),
                              elementCount );
                break;

            case NotificationListEvent.CHANGE:
                assertEquals( prefix+"oldValues().size()",
                              event.getOldValues().size(),
                              elementCount );
                assertEquals( prefix+"newValues().size()",
                              event.getNewValues().size(),
                              elementCount );
                break;

            case NotificationListEvent.REMOVE:
                assertEquals( prefix+"oldValues().size()",
                              event.getOldValues().size(),
                              elementCount );
                assertEquals( prefix+"newValues().size()",
                              event.getNewValues().size(),
                              0 );
                break;

            default:
                fail("Unexpected action: "+action);

        }
    }

    /**
     * Test set()
     * @throws Exception if the test fails
     */
    public void testSet() throws Exception {
        final List listToAdd = new ArrayList();
        listToAdd.add("Popeye");
        listToAdd.add("Olive Oyl");
        listToAdd.add("Wimpy");

        final NotificationList list = new NotificationList(new ArrayList());
        final EventCatcher eventCatcher = new EventCatcher();

        list.addAll( listToAdd );

        eventCatcher.listenTo(list);

        list.set( 1, "Scooby Doo" );

        assertEquals( "eventCatcher.size()", 1, eventCatcher.getEventCount() );
        checkRecord(eventCatcher,0,1,1,1, NotificationListEvent.CHANGE);

        final NotificationListEvent event =
        (NotificationListEvent)eventCatcher.getEventCatcherRecordAt(0).getEvent();
        assertEquals("oldValue", event.getOldValues().get(0), "Olive Oyl");
        assertEquals("newValue", event.getNewValues().get(0), "Scooby Doo");
    }

    /**
     * Test remove(int)
     * @throws Exception if the test fails
     */
    public void testRemove_int() throws Exception {
        final List listToAdd = new ArrayList();
        listToAdd.add("Popeye");
        listToAdd.add("Olive Oyl");
        listToAdd.add("Wimpy");

        final NotificationList list = new NotificationList(new ArrayList());
        final EventCatcher eventCatcher = new EventCatcher();

        list.addAll( listToAdd );
        eventCatcher.listenTo(list);

        list.remove(1);

        assertEquals( "eventCatcher.size()", 1, eventCatcher.getEventCount() );
        checkRecord(eventCatcher,0,1,1,1, NotificationListEvent.REMOVE);
        assertEquals( "list.size()", list.size(), 2 );
    }
}


