/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import javax.swing.SwingUtilities;
import junit.framework.TestCase;

/**
 *  Tests for AbstractUIController
 *
 * @version  $Revision: 1.8 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public final class AbstractUIControllerTest extends TestCase {

    private static final String TASK_COMPLETE = "taskComplete";
    private static final String TASK_ERROR_THROWN = "taskErrorThrown";
    private static final String TASK_EXCEPTION_THROWN = "taskExceptionThrown";
    private static final String TASK_SUCCESSFUL = "taskSuccessful";

    private Locale originalLocale_;


    /**
     *  Create an instance of the test
     *
     * @param  name The name of the test.
     */
    public AbstractUIControllerTest( final String name ) {
        super( name );
    }



    /**
     *  The JUnit setup method
     */
    public void setUp() {
        originalLocale_ = Locale.getDefault();

        // We need the swing event queue initialized for all the tests to run.  Creating a swing
        // component (doesn't matter which one) will do the initialization for us.
        new javax.swing.JLabel();
    }


    /**
     *  The teardown method for JUnit
     */
    public void tearDown() {
        Locale.setDefault( originalLocale_ );
    }


    /**
     *  Test the startTask() method
     *
     * @throws  Exception When the test fails
     */
    public void testStartTask()
        throws Exception {
        final String workerString = "worker";
        final String uiString = "ui";

        final List list = Collections.synchronizedList( new ArrayList() );

        /** A task for testing */
        class TestStartTask extends WorkerTask {
            public String failMessage = null;

            /** Code that will run on the worker thread */
            public void runOnWorkerThread() {
                if( SwingUtilities.isEventDispatchThread() == true ) {
                    failMessage = "runOnWorkerThread() was called from the UI thread";
                }
                list.add( workerString );
            }

            /** Code that will run on the ui thread */
            public void runOnUIThread() {
                list.add( uiString );
                if( SwingUtilities.isEventDispatchThread() == false ) {
                    failMessage = "runOnUIThread() was not called from the UI thread";
                }
            }
        }

        final TestStartTask task = new TestStartTask();
        final LoggingUIController controller = new LoggingUIController();

        controller.testTask( task );

        if( task.failMessage != null ) {
            fail( task.failMessage );
        }

        final List expectedStatusMessages = Arrays.asList( new String[]{
            TASK_SUCCESSFUL, TASK_COMPLETE } );
        assertEquals( expectedStatusMessages, controller.getList() );

        assertEquals( "list size", 2, list.size() );
        assertEquals( "worker element", workerString, list.get( 0 ) );
        assertEquals( "UI element", uiString, list.get( 1 ) );
    }


    /**
     *  Test successful completion of a task
     *
     * @throws  Exception If an error occurs
     */
    public void testTaskSuccessful()
        throws Exception {
        final LoggingUIController controller = new LoggingUIController();
        controller.testTask( new ThrowableTask( null, null ) );

        final List expectedResult = new ArrayList();
        expectedResult.add( TASK_SUCCESSFUL );
        expectedResult.add( TASK_COMPLETE );
        assertListsSame( expectedResult, controller.getList() );
    }


    /**
     *  Test a task that throws an exception from the worker thread
     *
     * @throws  Exception If an error occurs
     */
    public void testTaskException_WorkerThread()
        throws Exception {
        final LoggingUIController controller = new LoggingUIController();
        final IllegalStateException e = new IllegalStateException();
        controller.testTask( new ThrowableTask( e, null ) );

        final List expectedResult = new ArrayList();
        expectedResult.add( TASK_EXCEPTION_THROWN );
        expectedResult.add( e );
        expectedResult.add( TASK_COMPLETE );
        assertListsSame( expectedResult, controller.getList() );
    }


    /**
     *  Test a task that throws an exception from the ui thread
     *
     * @throws  Exception If an error occurs
     */
    public void testTaskException_UiThread()
        throws Exception {
        final LoggingUIController controller = new LoggingUIController();
        final IllegalStateException e = new IllegalStateException();
        controller.testTask( new ThrowableTask( null, e ) );

        final List expectedResult = new ArrayList();
        expectedResult.add( TASK_EXCEPTION_THROWN );
        expectedResult.add( e );
        expectedResult.add( TASK_COMPLETE );
        assertListsSame( expectedResult, controller.getList() );
    }


    /**
     *  Test a task that throws an error from the worker thread
     *
     * @throws  Exception If an error occurs
     */
    public void testTaskError_WorkerThread()
        throws Exception {
        final LoggingUIController controller = new LoggingUIController();
        final Error e = new OutOfMemoryError();
        controller.testTask( new ThrowableTask( e, null ) );

        final List expectedResult = new ArrayList();
        expectedResult.add( TASK_ERROR_THROWN );
        expectedResult.add( e );
        expectedResult.add( TASK_COMPLETE );
        assertListsSame( expectedResult, controller.getList() );
    }


    /**
     *  Test a task that throws an error from the ui thread
     *
     * @throws  Exception If an error occurs
     */
    public void testTaskError_UiThread()
        throws Exception {
        final LoggingUIController controller = new LoggingUIController();
        final Error e = new OutOfMemoryError();
        controller.testTask( new ThrowableTask( null, e ) );

        final List expectedResult = new ArrayList();
        expectedResult.add( TASK_ERROR_THROWN );
        expectedResult.add( e );
        expectedResult.add( TASK_COMPLETE );
        assertListsSame( expectedResult, controller.getList() );
    }


    /**
     *  Test changing the locale
     */
    public void testLocaleChanged() {
        final List list = new ArrayList();

        /** Dummy controller for testing */
        class MyController extends AbstractUIController {
            /** @param locale The new locale */
            protected final void localeChanged( final Locale locale ) {
                assertEquals( "locale == getLocale()", locale, getLocale() );
                list.add( locale );
            }

            /** The code to run */
            protected final void runImpl() {
            }
        }
        Locale.setDefault( Locale.GERMAN );
        final MyController controller = new MyController();

        assertEquals( "before run()", 0, list.size() );
        controller.run();

        assertEquals( "localeChanged on construction", 1, list.size() );
        assertEquals( "locale at construction", Locale.GERMAN, list.get( 0 ) );

        controller.setLocale( Locale.FRENCH );
        assertEquals( "localeChanged on setLocale", 2, list.size() );
        assertEquals( "locale at construction", Locale.FRENCH, list.get( 1 ) );
    }


    /**
     *  Test setLocale() with a null locale
     */
    public void testSetLocale_Null() {
        final AbstractUIController controller =
            new AbstractUIController() {
                protected void localeChanged( final Locale locale ) {
                }


                protected void runImpl() {
                }
            };

        try {
            controller.setLocale( null );
            fail( "Expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    private void assertListsSame( final List expected, final List actual ) {
        final int max = Math.min( expected.size(), actual.size() );

        int i;
        for( i = 0; i < max; i++ ) {
            assertEquals( "list[" + i + "]", expected.get( i ), actual.get( i ) );
        }
        assertEquals( "list.size", expected.size(), actual.size() );
    }


    private class LoggingUIController extends AbstractUIController {
        private List list_ = Collections.synchronizedList( new ArrayList() );

        /** @return An immutable copy of the logged information */
        public List getList() {
            return Collections.unmodifiableList( list_ );
        }

        /** @inheritDoc AbstractUIController#taskComplete(WorkerTask) */
        public void taskComplete( final WorkerTask task ) {
            list_.add( TASK_COMPLETE );
            checkIsOnUIThread();
            synchronized( this ) {
                this.notifyAll();
            }
        }

        /** @inheritDoc AbstractUIController#taskSuccessful(WorkerTask) */
        public void taskSuccessful( final WorkerTask task ) {
            list_.add( TASK_SUCCESSFUL );
            checkIsOnUIThread();
        }

        /** @inheritDoc AbstractUIController#taskErrorThrown(WorkerTask,Throwable) */
        public void taskErrorThrown( final WorkerTask task, final Throwable t ) {
            list_.add( TASK_ERROR_THROWN );
            checkIsOnUIThread();
            list_.add( t );
        }

        /** @inheritDoc AbstractUIController#taskExceptionThrown(WorkerTask,Exception) */
        public void taskExceptionThrown( final WorkerTask task, final Exception e ) {
            list_.add( TASK_EXCEPTION_THROWN );
            checkIsOnUIThread();
            list_.add( e );
        }

        /**
         * Perform testing of this task
         * @param task The task to test
         * @throws InterruptedException If the thread is interrupted
         */
        public synchronized void testTask( final WorkerTask task )
            throws InterruptedException {
            startTask( task );
            this.wait( 5000 );
        }


        protected void localeChanged( final Locale locale ) {
        }


        protected void runImpl() {
        }


        private void checkIsOnUIThread() {
            if( SwingUtilities.isEventDispatchThread() == false ) {
                list_.add( "Not on UI thread!" );
            }
        }
    }


    private class ThrowableTask extends WorkerTask {
        private Throwable workerThrowable_;
        private Throwable uiThrowable_;


        /**
         *  Create an instance of {1}
         *
         * @param  workerThrowable
         * @param  uiThrowable
         */
        public ThrowableTask( final Throwable workerThrowable, final Throwable uiThrowable ) {
            workerThrowable_ = workerThrowable;
            uiThrowable_ = uiThrowable;
        }

        /** @inheritDoc AbstractUIController#runOnWorkerThread() */
        public void runOnWorkerThread()
            throws Exception {
            throwIfNeeded( workerThrowable_ );
        }

        /** @inheritDoc AbstractUIController#runOnUIThread() */
        public void runOnUIThread()
            throws Exception {
            throwIfNeeded( uiThrowable_ );
        }


        private void throwIfNeeded( final Throwable t )
            throws Exception {
            if( t != null ) {
                if( t instanceof Exception ) {
                    throw ( Exception )t;
                }

                throw ( Error )t;
            }
        }
    }
}

