/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.util.DetailedNullPointerException;
import java.awt.Component;
import junit.framework.TestCase;

/**
 * Tests for DefaultComponentLoader.
 * 
 * @version $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DefaultComponentLoaderTest extends TestCase {

    /**
     * Create a new test.
     * @param name The name of the test.
     */
    public DefaultComponentLoaderTest( final String name ) {
        super(name);
    }

    /**
     * Test constructor with valid parameters
     * @throws ClassNotFoundException If the specified class was not found.
     */
    public void testStringConstructor_ValidParms() throws ClassNotFoundException {
        new DefaultComponentLoader("java.awt.Frame");
    }

    /**
     * Test the string constructor with a class that isn't a component.
     * @throws Exception if something bad happens.
     */
    public void testStringConstuctor_NotComponent() throws Exception {
        try {
            new DefaultComponentLoader("java.util.Date");
            fail("Expected exception for class that isn't a component");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test the string constructor with the name of a class that doesn't exist.
     * @throws Exception if something bad happens.
     */
    public void testStringConstuctor_BadClassName() throws Exception {
        try {
            new DefaultComponentLoader("java.util.Foobar");
            fail("Expected exception for missing class");
        }
        catch( final ClassNotFoundException e ) {
            // Expected path
        }
    }

    /**
     * Test the string constructor with a null class name.
     * @throws Exception if something bad happens.
     */
    public void testStringConstuctor_Null() throws Exception {
        try {
            new DefaultComponentLoader((String)null);
            fail("Expected exception for null string");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("clazz", e.getArgumentName());
        }
    }

    /**
     * Test the class constructor with a null class.
     * @throws Exception if something bad happens.
     */
    public void testClassConstuctor_Null() throws Exception {
        try {
            new DefaultComponentLoader((Class)null);
            fail("Expected exception for null class");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("clazz", e.getArgumentName());
        }
    }

    /**
     * Test the class constructor with a class that doesn't have a default constructor.
     * @throws Exception if something bad happens.
     */
    public void testClassConstuctor_NoDefaultConstructor() throws Exception {
        /** A fake component for testing */
        class FakeComponent extends Component {
            private static final long serialVersionUID = 1L;

			/** @param a Some parameter that is ignored for the test */
            public FakeComponent(int a) {
            }
        }
        try {
            new DefaultComponentLoader( FakeComponent.class );
            fail("Expected exception for class with no default constructor");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test the class constructor with a class that has a private constructor.
     * @throws Exception if something bad happens.
     */
    public void testClassConstuctor_PrivateDefaultConstructor() throws Exception {
        /** A fake component for testing */
        class FakeComponent extends Component {
            private static final long serialVersionUID = 1L;

			private FakeComponent() {
            }
        }
        try {
            new DefaultComponentLoader( FakeComponent.class );
            fail("Expected exception for class with private default constructor");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }
}

