/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import junit.framework.TestCase;

/**
 * Tests for ReflectedAction
 *
 * @version  $Revision: 1.9 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class ReflectedActionTest extends TestCase {
    private List resultList_ = new ArrayList();

    /**
     * Create a test
     * @param name The name of the test
     */
    public ReflectedActionTest( final String name ) {
        super(name);
    }


    /**
     * Create an instance with a null object
     */
    public void testConstructor_NullObject() {

        try {
            new ReflectedAction(null, "foo");
            fail("Expected exception for null object");
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     * Create an instance with a null method name
     */
    public void testConstructor_NullMethodName() {

        try {
            new ReflectedAction(this, null);
            fail("Expected exception for null method name");
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     * Create an instance with a bad method name
     */
    public void testConstructor_BadMethodName() {

        try {
            new ReflectedAction(this, "foo");
            fail("Expected exception for method name that doesn't exist [foo]");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }

        try {
            new ReflectedAction(this, "");
            fail("Expected exception for empty method name");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }

        try {
            new ReflectedAction(this, "methodWithTwoParameters");
            fail("Expected exception for empty method name");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }


    /**
     * Create an instance with legal values
     */
    public void testConstructor() {
        new ReflectedAction(this, "methodWithNoParameters");
        new ReflectedAction(this, "methodWithActionEventParameter");
    }


    /**
     * Create the actionPerformed() method
     */
    public void testActionPerformed() {
        resultList_.clear();

        ReflectedAction action;
        final ActionEvent actionEvent = new ActionEvent(this, 0, "");

        action = new ReflectedAction(this, "methodWithNoParameters");
        action.actionPerformed( actionEvent );
        assertEquals( "resultList.size()", 1, resultList_.size());
        assertEquals( "resultList[0]", "methodWithNoParameters", resultList_.get(0));

        action = new ReflectedAction(this, "methodWithActionEventParameter");
        action.actionPerformed( actionEvent );
        assertEquals( "resultList.size()", 2, resultList_.size());
        assertEquals( "resultList[1]", actionEvent, resultList_.get(1));
    }


    /**
     * Test the actionPerformed() method when it throws an exception
     */
    public void testActionPerformed_ExceptionThrown() {
        /** Fake action for testing */
        class ReflectedActionWithErrorHandling extends ReflectedAction {
            private static final long serialVersionUID = 1L;
			private Exception exception = null;
            /**
             * @param object Object to invoke
             * @param methodName Method to invoke
             */
            public ReflectedActionWithErrorHandling(final Object object, final String methodName ) {
                super(object, methodName);
            }
            /** @param e Exception that was thrown */
            protected void exceptionThrown( final Exception e ) {
                exception = e;
            }
            /** @return The exception that was thrown */
            public Exception getException() {
                return exception;
            }
        }

        final ReflectedActionWithErrorHandling action
            = new ReflectedActionWithErrorHandling(this, "methodThatThrowsException");
        final ActionEvent actionEvent = new ActionEvent(this, 0, "");

        action.actionPerformed(actionEvent);
        assertNotNull(action.getException());
        assertTrue("Outer exception is InvocationTargetException",
            action.getException() instanceof InvocationTargetException );
        assertTrue("Inner exception is IllegalStateException",
            ((InvocationTargetException)action.getException()).getTargetException() instanceof IllegalStateException );
    }


    /**
     * Checkstyle warns us about unused private methods.  There are a number
     * of private methods in this class that are required for the tests but
     * are never called directly so checkstyle complains about them.  This
     * method (which should never be executed) calls all these private methods
     * just to fool checkstyle.
     * @throws Exception If something goes wrong.
     */
    void fakeOutCheckstyle() throws Exception {
        methodWithNoParameters();
        methodWithActionEventParameter(null);
        methodWithTwoParameters(null, null);
        methodThatThrowsException();
    }
    
    private void methodWithNoParameters() {
        resultList_.add("methodWithNoParameters");
    }


    private void methodWithActionEventParameter( final ActionEvent event ) {
        resultList_.add(event);
    }


    private void methodWithTwoParameters( final ActionEvent event, final String s ) {
    }


    private void methodThatThrowsException() {
        throw new IllegalStateException();
    }
}
