/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import com.gargoylesoftware.base.util.DetailedNullPointerException;
import junit.framework.TestCase;

/**
 * Tests for TableLayoutConstraints
 *
 * @version  $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class TableLayoutConstraintsTest extends TestCase {

    /**
     * Create a new test case.
     * @param name The name of the test.
     */
    public TableLayoutConstraintsTest( final String name ) {
        super(name);
    }

    /**
     * Test the constuctor with legal values.
     */
    public void testConstructor2Parms() {
        final TableLayoutConstraints constraints
        = new TableLayoutConstraints(1,2);

        assertEquals("row", constraints.getRow(), 1 );
        assertEquals("column", constraints.getColumn(),2);
        assertEquals("rowSpan", constraints.getRowSpan(), 1);
        assertEquals("columnSpan", constraints.getColumnSpan(), 1 );

        assertEquals("verticalAlignment",
                     constraints.getVerticalAlignment(),
                     TableLayout.CENTER );
        assertTrue("verticalStretch",
               constraints.getVerticalStretch() == false );

        assertEquals("horizontalAlignment",
                     constraints.getHorizontalAlignment(),
                     TableLayout.LEFT );
        assertTrue("horizontalStretch",
               constraints.getHorizontalStretch() == false );
    }

    /**
     * Test the constructor will negative parameters.
     */
    public void testConstructor2Parms_Negative() {
        try {
            new TableLayoutConstraints(-1,2);
            fail("Expected exception for negative row");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }

        try {
            new TableLayoutConstraints(1,-2);
            fail("Expected exception for negative column");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test a negative row.
     */
    public void testNegativeRow() {
        try {
            new TableLayoutConstraints(-1,2);
            fail("Expected IllegalArgumentException");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test a negative row span
     */
    public void testNegativeRowSpan() {
        final TableLayoutConstraints constraints = new TableLayoutConstraints(1,2);
        try {
            constraints.setRowSpan(-1);
            fail("Expected IllegalArgumentException");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test a zero row span.
     */
    public void testZeroRowSpan() {
        final TableLayoutConstraints constraints = new TableLayoutConstraints(1,2);
        try {
            constraints.setRowSpan(0);
            fail("Expected IllegalArgumentException");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test a negative column.
     */
    public void testNegativeColumn() {
        try {
            new TableLayoutConstraints(1,-2);
            fail("Expected IllegalArgumentException");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test a negative column span.
     */
    public void testNegativeColumnSpan() {
        final TableLayoutConstraints constraints = new TableLayoutConstraints(1,2);
        try {
            constraints.setColumnSpan(-4);
            fail("Expected IllegalArgumentException");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test a zero column span
     */
    public void testZeroColumnSpan() {
        final TableLayoutConstraints constraints = new TableLayoutConstraints(1,2);
        try {
            constraints.setColumnSpan(0);
            fail("Expected IllegalArgumentException");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test an illegal vertical alignment.
     */
    public void testIllegalVerticalAlignment() {
        final TableLayoutConstraints constraints = new TableLayoutConstraints(1,2);
        try {
            constraints.setVerticalAlignment(TableLayout.RIGHT);
            fail("Expected IllegalArgumentException");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test an illegal horizontal alignment.
     */
    public void testIllegalHorizontalAlignment() {
        final TableLayoutConstraints constraints = new TableLayoutConstraints(1,2);
        try {
            constraints.setHorizontalAlignment(TableLayout.TOP);
            fail("Expected IllegalArgumentException");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }


    /**
     * Test makeConstraints() with bad input.
     */
    public void testMakeConstraints_BadData() {
        final String data[] = {
            "", ","
        };

        int i;
        for( i=0; i<data.length; i++ ) {
            try {
                TableLayoutConstraints.makeConstraints(data[i]);
                fail("Expected exception for ["+data[i]+"]");
            }
            catch( final DetailedIllegalArgumentException e ) {
                assertEquals(e.getMessage(), "constraintString", e.getArgumentName());
            }
        }
    }


    /**
     * Test makeConstraints() with bad input.
     */
    public void testMakeConstraints_Null() {

        try {
            TableLayoutConstraints.makeConstraints(null);
            fail("Expected exception for null");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("constraintString", e.getArgumentName());
        }
    }


    /**
     * Test makeConstraints with legal input.
     */
    public void testMakeConstraints() {
        final TableLayoutConstraints constraints
            = TableLayoutConstraints.makeConstraints("5,17");
        assertEquals("row", 5, constraints.getRow() );
        assertEquals("column", 17, constraints.getColumn() );
    }

    /**
     * Test make constraints with spanning.
     */
    public void testMakeConstraints_spanning() {
        final TableLayoutConstraints constraints
            = TableLayoutConstraints.makeConstraints("5+2,7+3");
        assertEquals("row", 5, constraints.getRow() );
        assertEquals("column", 7, constraints.getColumn() );
        assertEquals("rowSpan", 2, constraints.getRowSpan() );
        assertEquals("columnSpan", 3, constraints.getColumnSpan() );

        if( constraints.getVerticalStretch() == true ) {
            fail("Not expecting vertical stretch");
        }
        if( constraints.getHorizontalStretch() == true ) {
            fail("Not expecting horizontal stretch");
        }
    }

    /**
     * Test makeConstraints() with stretching.
     */
    public void testMakeConstraints_stretching() {
        final TableLayoutConstraints constraints
            = TableLayoutConstraints.makeConstraints("5s,7+3s");
        assertEquals("row", 5, constraints.getRow() );
        assertEquals("column", 7, constraints.getColumn() );
        assertEquals("columnSpan", 3, constraints.getColumnSpan() );

        if( constraints.getVerticalStretch() == false ) {
            fail("Expected vertical stretch");
        }
        if( constraints.getHorizontalStretch() == false ) {
            fail("Expected horizontal stretch");
        }
    }

}
