/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.gui;

import java.awt.Dimension;
import java.awt.Point;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import junit.framework.TestCase;

import com.gargoylesoftware.base.util.DetailedNullPointerException;

/**
 * Tests for TableLayout
 *
 * @version  $Revision: 1.7 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class TableLayoutTest extends TestCase {

    /**
     * Create a new test case.
     * @param name The name of the test.
     */
    public TableLayoutTest( final String name ) {
        super(name);
    }

    /**
     * Make sure that everything works ok if there aren't any children
     */
    public void testNoComponents() {

        final TableLayout layout = new TableLayout();
        final Dimension zeroSize = new Dimension(0,0);
        final Dimension maxSize = new Dimension(Integer.MAX_VALUE,Integer.MAX_VALUE);

        final JPanel panel = new JPanel();
        panel.setLayout(layout);

        assertEquals( "minimumSize", zeroSize,
                      panel.getMinimumSize());
        assertEquals( "preferredSize", zeroSize,
                      panel.getPreferredSize());
        assertEquals( "maximumSize", maxSize,
                      panel.getMaximumSize());

        panel.setSize(800,600);
        panel.validate();
    }

    /**
     * Test with no children but with insets specified
     */
    public void testNoComponents_WithInsets() {

        final TableLayout layout = new TableLayout();
        final JPanel panel = new JPanel();
        panel.setLayout(layout);
        panel.setBorder( BorderFactory.createEmptyBorder(2,2,2,2) );

        assertEquals( "minimumSize", new Dimension(4,4),
                      panel.getMinimumSize());
        assertEquals( "preferredSize", new Dimension(4,4),
                      panel.getPreferredSize());

        panel.setSize(800,600);
        panel.validate();
    }

    /**
     * The layout should always be called by the same container.
     * Make sure it throws an exception if it is called by more
     * than one container.
     */
    public void testDifferentContainers() {

        final TableLayout layout = new TableLayout();
        final JPanel panel1 = new JPanel();
        final JPanel panel2 = new JPanel();

        layout.minimumLayoutSize(panel1);
        try {
            layout.minimumLayoutSize(panel2);
            fail("Expected IllegalArgumentException when using different containers");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Ensure that an exception is thrown if null is passed in as
     * the container.
     */
    public void testNullContainers() {

        final TableLayout layout = new TableLayout();

        try {
            layout.minimumLayoutSize(null);
            fail("Expected exception when using null container");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("newParent", e.getArgumentName());
        }
    }

    /**
     * Make sure you can't add the same component twice.
     */
    public void testAddComponentTwice() {

        final TableLayout layout = new TableLayout();
        final JPanel panel = new JPanel();
        layout.addLayoutComponent( panel, new TableLayoutConstraints(3,4) );

        try {
            layout.addLayoutComponent(panel, new TableLayoutConstraints(1,2));
            fail("Should not be able to add the same component twice");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Make sure you can't pass in a null component.
     */
    public void testAddNullComponent() {

        final TableLayout layout = new TableLayout();
        try {
            layout.addLayoutComponent(null, new TableLayoutConstraints(1,2));
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("comp", e.getArgumentName());
        }
    }

    /**
     * Make sure that the layout doesn't accept constraints that
     * aren't TableLayoutConstraints or String objects.
     */
    public void testAddStringConstraint() {

        final TableLayout layout = new TableLayout();
        try {
            layout.addLayoutComponent(new JPanel(), new Integer(5));
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Make sure you can't remove the same object twice.
     */
    public void testRemoveComponentTwice() {

        final TableLayout layout = new TableLayout();
        final JPanel panel = new JPanel();
        layout.addLayoutComponent( panel, new TableLayoutConstraints(1,2) );
        layout.removeLayoutComponent( panel );
        try {
            layout.removeLayoutComponent( panel );
            fail("Should be not able to remove the same component twice");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Make sure you can't remove a null component.
     */
    public void testRemoveNullComponent() {

        final TableLayout layout = new TableLayout();
        try {
            layout.removeLayoutComponent(null);
            fail("Expected exception");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("comp", e.getArgumentName());
        }
    }

    /**
     * Test with only one child to ensure that all dimensions
     * are calculated correctly.
     */
    public void testOneChild() {

        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();
        JPanel child;

        parent.setLayout( layout );
        parent.setMaximumSize( new Dimension(600,700) );

        child = new JPanel();
        child.setMinimumSize  ( new Dimension(20,30) );
        child.setPreferredSize( new Dimension(40,50) );
        child.setMaximumSize  ( new Dimension(60,70) );
        parent.add( child, new TableLayoutConstraints(1,1) );

        parent.setSize(800,600);
        layout.layoutContainer(parent);

        assertEquals( "parent.minimumSize",
                      new Dimension(20,30),
                      parent.getMinimumSize() );
        assertEquals( "parent.preferredSize",
                      new Dimension(40,50),
                      parent.getPreferredSize() );
        assertEquals( "parent.maximumSize",
                      new Dimension(600,700),
                      parent.getMaximumSize() );

        assertEquals( "child.size",
                      new Dimension(40,50),
                      child.getSize() );
    }

    /**
     * Test with one child and some insets to make sure all
     * dimensions are calculated correctly.
     */
    public void testOneChildWithInsetsOnParent() {

        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();
        JPanel child;

        parent.setLayout( layout );
        parent.setMaximumSize( new Dimension(600,700) );
        parent.setBorder( BorderFactory.createEmptyBorder(2,2,2,2) );

        child = new JPanel();
        child.setMinimumSize  ( new Dimension(20,30) );
        child.setPreferredSize( new Dimension(40,50) );
        child.setMaximumSize  ( new Dimension(60,70) );

        parent.add(child, new TableLayoutConstraints(1,1) );

        parent.setSize(800,600);
        layout.layoutContainer(parent);

        assertEquals( "parent.minimumSize",
                      new Dimension(24,34),
                      parent.getMinimumSize() );
        assertEquals( "parent.preferredSize",
                      new Dimension(44,54),
                      parent.getPreferredSize() );
        assertEquals( "parent.maximumSize",
                      new Dimension(600,700),
                      parent.getMaximumSize() );

        assertEquals( "child.size",
                      new Dimension(40,50),
                      child.getSize() );

        final Dimension childMinimumSize = child.getMinimumSize();
        parent.setSize( new Dimension( childMinimumSize.width+20, childMinimumSize.height+20) );
        layout.layoutContainer(parent);
        assertEquals( "child.location",
                      new Point(2,2),
                      child.getLocation() );
    }

    /**
     * Test the alignment with one child and insets.
     */
    public void testParentAlignmentWithOneChildAndInsets() {

        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();
        JPanel child;

        parent.setLayout( layout );
        parent.setMaximumSize( new Dimension(600,700) );
        final int top = 2;
        final int bottom = 2;
        final int left = 2;
        final int right = 2;
        parent.setBorder( BorderFactory.createEmptyBorder(top,left,bottom,right) );

        final int childWidth  = 20;
        final int childHeight = 30;

        child = new JPanel();
        child.setMinimumSize  ( new Dimension(20,30) );
        child.setPreferredSize( new Dimension(childWidth, childHeight) );
        child.setMaximumSize  ( new Dimension(60,70) );
        parent.add(child, new TableLayoutConstraints(1,1) );

        final int parentWidth  = 800;
        final int parentHeight = 600;
        parent.setSize( parentWidth, parentHeight );

        layout.setHorizontalAlignment( TableLayout.LEFT );
        layout.setVerticalAlignment( TableLayout.TOP );
        layout.layoutContainer(parent);
        assertEquals( "child.location for LEFT,TOP",
                      new Point(2,2),
                      child.getLocation() );

        layout.setHorizontalAlignment( TableLayout.CENTER );
        layout.setVerticalAlignment( TableLayout.CENTER );
        layout.layoutContainer(parent);
        assertEquals( "child.location for CENTER, CENTER",
                      new Point( (parentWidth-childWidth-left-right)/2 + left,
                                 (parentHeight-childHeight-top-bottom)/2 + top),
                      child.getLocation() );

        layout.setHorizontalAlignment( TableLayout.RIGHT );
        layout.setVerticalAlignment( TableLayout.BOTTOM );
        layout.layoutContainer(parent);
        assertEquals( "child.location for BOTTOM, RIGHT",
                      new Point( parentWidth-childWidth-left,
                                 parentHeight-childHeight-top ),
                      child.getLocation() );
    }


    /**
     * Test removing a component.
     */
    public void testRemove() {
        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();

        parent.setLayout( layout );

        final JPanel child = new JPanel();
        child.setPreferredSize( new Dimension(100,100) );
        parent.add(child, new TableLayoutConstraints(1,1) );

        assertEquals( "parent.preferredSize",
                      new Dimension(100,100),
                      parent.getPreferredSize() );

        parent.remove(child);
        assertEquals( "parent.preferredSize",
                      new Dimension(0,0),
                      parent.getPreferredSize() );
    }


    /**
     * Test the case where removing a component changes the
     * sizes but doesn't change the number of rows or columns.
     *
     *  ***** *****
     *  * 1 * * 2 *
     *  ***** *****
     *
     *  ***** *******
     *  * 3 * *     *
     *  ***** *  4  *
     *        *     *
     *        *******
     */
    public void testRemoveWhereColumnAndRowCountsDontChange() {
        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();

        parent.setLayout( layout );

        final int small = 10;
        final int large = 50;
        final Dimension smallSize = new Dimension(small, small);
        final Dimension largeSize = new Dimension(large, large);

        final JPanel child1 = new JPanel();
        final JPanel child2 = new JPanel();
        final JPanel child3 = new JPanel();
        final JPanel child4 = new JPanel();

        child1.setMinimumSize(smallSize);
        child2.setMinimumSize(smallSize);
        child3.setMinimumSize(smallSize);
        child4.setMinimumSize(largeSize);

        parent.add( child1, new TableLayoutConstraints(0,0) );
        parent.add( child2, new TableLayoutConstraints(0,1) );
        parent.add( child3, new TableLayoutConstraints(1,0) );
        parent.add( child4, new TableLayoutConstraints(1,1) );

        assertEquals( "parent.minimumSize",
                      new Dimension(small+large, small+large),
                      parent.getMinimumSize() );

        parent.remove(child4);

        assertEquals( "parent.minimumSize",
                      new Dimension(small+small, small+small),
                      parent.getMinimumSize() );
    }


    /**
     * Regression test: Exception was being thrown when a component was added
     * after it had been removed once.
     */
    public void testRemoveAndAddAgain() {
        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();

        parent.setLayout( layout );

        final JPanel child = new JPanel();
        parent.add(child, new TableLayoutConstraints(1,1) );

        parent.setSize( 800,600 );

        // The original bug would throw an exception on execution of the
        // following line.
        layout.setMinimumRowHeight(9,10);

        parent.remove(child);
        parent.add(child, new TableLayoutConstraints(1,1) );
        layout.layoutContainer(parent);
    }


    /**
     * Test the ability to expand rows.
     */
    public void testOneChildExpandedRow() {

        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();
        JPanel child;

        parent.setLayout( layout );

        child = new JPanel();
        child.setMinimumSize  ( new Dimension(20,30) );
        child.setPreferredSize( new Dimension(40,50) );

        final TableLayoutConstraints constraints = new TableLayoutConstraints(0,0);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        parent.add( child, constraints );

        layout.setRowExpandable( 0, true );

        assertEquals( "parent.minimumSize",
                      new Dimension(20,30),
                      parent.getMinimumSize() );
        assertEquals( "parent.preferredSize",
                      new Dimension(40,50),
                      parent.getPreferredSize() );

        parent.setSize(800,600);
        layout.layoutContainer(parent);

        assertEquals( "child.size",
                      new Dimension(40,600),
                      child.getSize() );
        assertEquals( "child.location",
                      new Point((800-40)/2, 0),
                      child.getLocation() );
    }


    /**
     * Test the ability to expand columns.
     */
    public void testOneChildExpandedColumn() {

        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();
        JPanel child;

        parent.setLayout( layout );

        child = new JPanel();
        child.setMinimumSize  ( new Dimension(20,30) );
        child.setPreferredSize( new Dimension(40,50) );

        final TableLayoutConstraints constraints = new TableLayoutConstraints(0,0);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        parent.add( child, constraints );

        layout.setColumnExpandable( 0, true );

        assertEquals( "parent.minimumSize",
                      new Dimension(20,30),
                      parent.getMinimumSize() );
        assertEquals( "parent.preferredSize",
                      new Dimension(40,50),
                      parent.getPreferredSize() );

        parent.setSize(800,600);
        layout.layoutContainer(parent);

        assertEquals( "child.size",
                      new Dimension(800,50),
                      child.getSize() );
        assertEquals( "child.location",
                      new Point(0,(600-50)/2),
                      child.getLocation() );
    }


    /**
     * Make sure that the child doesn't get stretched beyond its
     * maximum size.
     */
    public void testOneChildExpandedWithMaxSize() {

        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();

        parent.setLayout( layout );

        final JPanel child = new JPanel();
        child.setMinimumSize  ( new Dimension(20,30) );
        child.setPreferredSize( new Dimension(40,50) );
        child.setMaximumSize( new Dimension(100,100));

        final TableLayoutConstraints constraints = new TableLayoutConstraints(0,0);
        constraints.setHorizontalAlignment( TableLayout.LEFT );
        constraints.setVerticalAlignment( TableLayout.TOP );
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        constraints.setObeyMaximumSize(true);
        parent.add( child, constraints );

        layout.setRowExpandable   ( 0, true );
        layout.setColumnExpandable( 0, true );

        assertEquals( "parent.minimumSize",
                      new Dimension(20,30),
                      parent.getMinimumSize() );
        assertEquals( "parent.preferredSize",
                      new Dimension(40,50),
                      parent.getPreferredSize() );

        parent.setSize(800,600);
        layout.layoutContainer(parent);

        assertEquals( "child.size",
                      new Dimension(100,100),
                      child.getSize() );
        assertEquals( "child.location",
                      new Point(0,0),
                      child.getLocation() );


    }


    /**
     * <pre>
     * ################################
     * #     parent                   #
     * #                              #
     * #    #######################   #
     * #    # child1 # child2     #   #
     * #    # (0,0)  # (0,1)      #   #
     * #    #        #            #   #
     * #    #        ##############   #
     * #    #        # child3     #   #
     * #    #        # (1,1)      #   #
     * #    #        #            #   #
     * #    #######################   #
     * #    # child4              #   #
     * #    # (2,0)               #   #
     * #    #                     #   #
     * #    #######################   #
     * #                              #
     * #                              #
     * ################################
     * </pre>
     */
    public void testFourChildrenWithComponentStretching() {
        final JPanel parent = new JPanel();
        final JLabel child1 = new JLabel("child1");
        final JLabel child2 = new JLabel("child2");
        final JLabel child3 = new JLabel("child3");
        final JLabel child4 = new JLabel("child4");

        // By default, labels set their maximum size to be the same
        // as their preferred size which means that they won't stretch.
        // Since we're testing stretching, we force the maximum size
        // to be huge.
        final Dimension maximumSize = new Dimension(1000,1000);
        child1.setMaximumSize( maximumSize );
        child2.setMaximumSize( maximumSize );
        child3.setMaximumSize( maximumSize );
        child4.setMaximumSize( maximumSize );

        final TableLayout layout = new TableLayout();
        TableLayoutConstraints constraints;
        constraints = new TableLayoutConstraints(0,0);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);

        parent.setLayout( layout );

        constraints = new TableLayoutConstraints(0,0);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        constraints.setRowSpan(2);
        parent.add( child1, constraints );

        constraints = new TableLayoutConstraints(0,1);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        parent.add( child2, constraints );

        constraints = new TableLayoutConstraints(1,1);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        parent.add( child3, constraints );

        constraints = new TableLayoutConstraints(2,0);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        constraints.setColumnSpan(2);
        parent.add( child4, constraints );

        parent.setSize(2000,2000);
        layout.layoutContainer(parent);

        final Dimension child1Size = child1.getSize();
        final Dimension child2Size = child2.getSize();
        final Dimension child3Size = child3.getSize();
        final Dimension child4Size = child4.getSize();

        assertEquals( "Children 2 and 3 should have the same width",
                      child2Size.width, child3Size.width );
        assertEquals( "child1.height == child2.height+child3.height",
                      child1Size.height,
                      child2Size.height + child3Size.height );
        assertEquals( "child4.width == child1.width+child2.width",
                      child4Size.width,
                      child1Size.width + child2Size.width );

        assertEquals( "child4.width should be no wider than the sum of the preferred sizes",
                      child4Size.width,
                      child1.getPreferredSize().width
                        + Math.max( child2.getPreferredSize().width,
                      child3.getPreferredSize().width ) );
    }


    /**
     * <pre>
     * ################################
     * #     parent                   #
     * #                              #
     * #    #######################   #
     * #    # child1 # child2     #   #
     * #    # (0,0)  # (0,1)      #   #
     * #    #        #            #   #
     * #    #        ##############   #
     * #    #        # child3     #   #
     * #    #        # (1,1)      #   #
     * #    #        #            #   #
     * #    #######################   #
     * #    # child4              #   #
     * #    # (2,0)               #   #
     * #    #                     #   #
     * #    #######################   #
     * #                              #
     * #                              #
     * ################################
     * </pre>
     */
    public void testNoExpansionBeyondMaximumSize() {
        final JPanel parent = new JPanel();
        final JLabel child1 = new JLabel("child1");
        final JLabel child2 = new JLabel("child2");
        final JLabel child3 = new JLabel("child3");
        final JLabel child4 = new JLabel("child4");

        final TableLayout layout = new TableLayout();
        TableLayoutConstraints constraints;

        parent.setLayout( layout );

        constraints = new TableLayoutConstraints(0,0);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        constraints.setRowSpan(2);
        constraints.setObeyMaximumSize(true);
        parent.add( child1, constraints );

        constraints = new TableLayoutConstraints(0,1 );
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        constraints.setObeyMaximumSize(true);
        parent.add( child2, constraints );

        constraints = new TableLayoutConstraints(1,1);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        constraints.setObeyMaximumSize(true);
        parent.add( child3, constraints );

        constraints = new TableLayoutConstraints(2,0);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        constraints.setColumnSpan(2);
        constraints.setObeyMaximumSize(true);
        parent.add( child4, constraints );

        parent.setSize(2000,2000);
        layout.layoutContainer(parent);

        assertEquals( "child1", child1.getSize(), child1.getMaximumSize() );
        assertEquals( "child2", child1.getSize(), child2.getMaximumSize() );
        assertEquals( "child3", child1.getSize(), child3.getMaximumSize() );
        assertEquals( "child4", child1.getSize(), child4.getMaximumSize() );
    }


    /**
     * Test the minimum sizes.
     */
    public void testMinimumSizesOnColumnsAndRows() {

        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();
        JPanel child;

        parent.setLayout( layout );

        child = new JPanel();
        child.setMinimumSize  ( new Dimension(20,30) );
        child.setPreferredSize( new Dimension(40,50) );
        child.setMaximumSize  ( new Dimension(10000,10000) );
        final TableLayoutConstraints constraints = new TableLayoutConstraints(1,1);
        constraints.setVerticalStretch(true);
        constraints.setHorizontalStretch(true);
        parent.add( child, constraints );

        layout.setRowExpandable(1, true );
        layout.setColumnExpandable(1, true );

        layout.setMinimumRowHeight  (0,11);
        layout.setMinimumColumnWidth(0,12);
        layout.setMinimumRowHeight  (2,13);
        layout.setMinimumColumnWidth(2,14);

        assertEquals( "parent.preferredSize",
                      new Dimension(40+12+14,50+11+13),
                      parent.getPreferredSize() );
        assertEquals( "parent.minimumSize",
                      new Dimension(20+12+14,30+11+13),
                      parent.getMinimumSize() );

        parent.setSize(800,600);
        layout.layoutContainer(parent);

        assertEquals( "child.size",
                      new Dimension(800-12-14, 600-11-13),
                      child.getSize() );
        assertEquals( "child.location",
                      new Point(12,11),
                      child.getLocation() );

    }


    /**
     * Test expandable rows with insets
     */
    public void testExpandableRowWithInsets() {
        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();
        TableLayoutConstraints constraints;

        parent.setLayout( layout );

        final JPanel child1 = new JPanel();
        child1.setPreferredSize( new Dimension(10,20) );
        parent.add( child1, new TableLayoutConstraints(1,1) );

        final JPanel child2 = new JPanel();
        child2.setPreferredSize( new Dimension(10,20) );
        child2.setMaximumSize( new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE) );
        constraints = new TableLayoutConstraints(3,1);
        constraints.setVerticalStretch(true);
        parent.add( child2, constraints );

        final JPanel child3 = new JPanel();
        child3.setPreferredSize( new Dimension(10,20) );
        parent.add( child3, new TableLayoutConstraints(5,1) );

        final int borderLeft   = 2;
        final int borderRight  = 2;
        final int borderTop    = 2;
        final int borderBottom = 2;
        layout.setRowExpandable(3, true );
        parent.setBorder( BorderFactory.createEmptyBorder(borderTop, borderLeft, borderBottom, borderRight ) );

        final int parentWidth = 800;
        final int parentHeight = 600;
        parent.setSize( parentWidth, parentHeight );
        //layout.setHorizontalAlignment( TableLayout.LEFT );
        layout.layoutContainer(parent);

        // Child 1
        assertEquals( "child1.size",
                      new Dimension(10, 20),
                      child1.getSize() );
        assertEquals( "child1.location",
                      borderTop,
                      child1.getLocation().y );

        // Child 2
        assertEquals( "child2.location",
                      borderTop + child1.getSize().height,
                      child2.getLocation().y );

        // Child 3
        assertEquals( "child3.size",
                      new Dimension(10, 20),
                      child3.getSize() );
        assertEquals( "child3.location",
                      parentHeight - borderBottom - child3.getSize().height,
                      child3.getLocation().y );

    }

    /**
     * If the actual size is less than the preferred size of the layout then
     * some components were being sized to larger than their preferred size.
     * Make sure this is fixed.
     */
    public void testShrinkToFit() {
        final JPanel parent = new JPanel();
        final TableLayout layout = new TableLayout();
        TableLayoutConstraints constraints;

        parent.setLayout( layout );

        JPanel child1 = new JPanel();
        child1.setMinimumSize( new Dimension(10,10) );
        child1.setPreferredSize( new Dimension(200,200) );
        constraints = new TableLayoutConstraints(1,1);
        parent.add( child1, constraints );

        JPanel child2 = new JPanel();
        child2.setMinimumSize( new Dimension(10,10) );
        child2.setPreferredSize( new Dimension(20,20) );
        constraints = new TableLayoutConstraints(1,3);
        parent.add( child2, constraints );

        JPanel child3 = new JPanel();
        child3.setMinimumSize( new Dimension(10,10) );
        child3.setPreferredSize( new Dimension(200,200) );
        constraints = new TableLayoutConstraints(1,5);
        parent.add( child3, constraints );

        assertEquals("parent.minimumSize",
                     new Dimension(30,10),
                     parent.getMinimumSize() );
        assertEquals("parent.preferredSize",
                     new Dimension(420,200),
                     parent.getPreferredSize() );

        // We set the parent size to be smaller than the children's preferred
        parent.setSize( new Dimension(390,420) );
        layout.layoutContainer(parent);

        assertEquals("child1.size",
                     new Dimension(185,200),
                     child1.getSize() );
        assertEquals("child2.size",
                     new Dimension(20,20),
                     child2.getSize() );
        assertEquals("child3.size",
                     new Dimension(185,200),
                     child3.getSize() );
    }

    /**
     * Regression test for a bug.
     */
    public void testEmptyColumnsExpanding() {
        final TableLayout layout = new TableLayout();
        final JPanel parent = new JPanel(layout);

        TableLayoutConstraints constraints;

        final JPanel child1 = new JPanel();
        final JPanel child2 = new JPanel();
        final JPanel child3 = new JPanel();
        final JPanel child4 = new JPanel();

        final Dimension smallSize = new Dimension(10,10);
        final Dimension largeSize = new Dimension(100,10);

        child1.setMinimumSize  ( smallSize );
        child1.setPreferredSize( smallSize );
        child2.setMinimumSize  ( smallSize );
        child2.setPreferredSize( smallSize );
        child3.setMinimumSize  ( smallSize );
        child3.setPreferredSize( smallSize );
        child4.setMinimumSize  ( smallSize );
        child4.setPreferredSize( largeSize );

        layout.setColumnExpandable(1,true);

        constraints = new TableLayoutConstraints(1,1);
        constraints.setHorizontalStretch(true);
        parent.add( child1, constraints );

        constraints = new TableLayoutConstraints(1,3);
        constraints.setHorizontalStretch(true);
        parent.add( child2, constraints );

        constraints = new TableLayoutConstraints(3,1);
        constraints.setHorizontalStretch(true);
        parent.add( child3, constraints );

        constraints = new TableLayoutConstraints(5,1);
        constraints.setColumnSpan(3);
        constraints.setHorizontalStretch(true);
        parent.add( child4, constraints );

        assertEquals("parent.preferredSize",
                     new Dimension(100,30),
                     parent.getPreferredSize() );

        parent.setSize( parent.getPreferredSize() );
        layout.layoutContainer(parent);

        assertEquals("child1.size",
                     new Dimension(90,10),
                     child1.getSize() );
        assertEquals("child1.location",
                     new Point(0,0),
                     child1.getLocation() );

        assertEquals("child2.size",
                     new Dimension(10,10),
                     child2.getSize() );
        assertEquals("child2.location",
                     new Point(90,0),
                     child2.getLocation() );

        assertEquals("child3.size",
                     new Dimension(90,10),
                     child3.getSize() );
        assertEquals("child3.location",
                     new Point(0,10),
                     child3.getLocation() );

        assertEquals("child4.size",
                     new Dimension(100,10),
                     child4.getSize() );
        assertEquals("child4.location",
                     new Point(0,20),
                     child4.getLocation() );

    }

    /**
     *  *******
     *  *  A  *
     *  *     *
     *  *******
     *
     *  ******************
     *  *      B         *
     *  *                *
     *  ******************
     *
     * Regression testing a bug where having an odd number for the width of A (or B for that
     * matter) caused column 0 to expand.  Column 0 should end up with a width of zero as there
     * are no components in it and it is not marked as expandable.
     */
    public void testOddSizeWidth() {
        final TableLayout layout = new TableLayout();
        final JPanel panel = new TableLayoutDebuggingPanel(layout);

        TableLayoutConstraints constraints;

        layout.setColumnExpandable(1,true);
        layout.setColumnExpandable(2,true);

        final JPanel compA = new JPanel();
        compA.setName("A");
        compA.setPreferredSize( new Dimension(102, 30) );
        constraints = new TableLayoutConstraints(1,1);

        panel.add( compA, constraints );

        final JPanel compB = new JPanel();
        compB.setName("B");
        compB.setPreferredSize( new Dimension(300,30) );
        constraints = new TableLayoutConstraints(2,1);
        constraints.setColumnSpan(2);
        constraints.setHorizontalStretch(true);
        panel.add( compB, constraints );

        panel.setSize( panel.getPreferredSize() );
        layout.layoutContainer(panel);

        assertEquals( "compA.position even width", new Point(0,0), compA.getLocation() );

        // So far everything should have worked since we're using even widths.  Now we
        // set the width to an odd number and watch things blow up.

        layout.invalidateLayout(panel);
        compA.setPreferredSize( new Dimension(103, 30) );
        panel.setSize( panel.getPreferredSize() );
        layout.layoutContainer(panel);

        assertEquals( "compA.position odd width", new Point(0,0), compA.getLocation() );
    }

    /**
     * Regression test for bug where an exception would be thrown if the size of the panel
     * was so small that there wasn't room to display the insets.
     */
    public void testSmallerSizeThanAllowedByInsets() {
        final TableLayout layout = new TableLayout();
        final JPanel panel = new TableLayoutDebuggingPanel(layout);

        panel.add( new JLabel("foo"), "1,1" );
        panel.setBorder( BorderFactory.createEmptyBorder(5,5,5,5) );

        panel.setSize( new Dimension(0,0) );
        layout.layoutContainer(panel);
    }

    /**
     * Regression test for a bug where adding and then removing a component followed
     * by adding a new one will cause an ArrayOutOfBoundsException to be thrown.
     */
    public void testAddAddRemoveAdd() {
        final TableLayout layout = new TableLayout();
        final JPanel panel = new JPanel(layout);

        final JPanel child1 = new JPanel();
        final JPanel child2 = new JPanel();
        final JPanel child3 = new JPanel();

        panel.add( "1,1", child1 );

        layout.addLayoutComponent( child2, "1,3" );
        layout.removeLayoutComponent(child2);
        layout.addLayoutComponent( child3, "1,3" );
        layout.layoutContainer(panel);
    }
}


