/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.objectstore;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import junit.framework.TestCase;

/**
 * Tests for DispatchingObjectStore
 *
 * @version  $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DispatchingObjectStoreTest extends TestCase {

    private static class CommandA extends ObjectStoreCommand {

		private static final long serialVersionUID = 1L;
    }


    /**
     * A fake object store for testing purposes
     */
    public static class ObjectStoreA extends ObjectStore {
        /** Create one */
        public ObjectStoreA() {}
        private final List list_ = new ArrayList();
        /**
         * Execute the command
         * @param command The command to execute
         * @return The list
         */
        protected Object executeImpl( final ObjectStoreCommand command ) {
            list_.add(command);
            return list_;
        }
    }


    /**
     * Create an instance
     * @param name The name of the test
     */
    public DispatchingObjectStoreTest( final String name ) {
        super(name);
    }


    /**
     * Test the dispatching logic
     */
    public void testDispatch() {

        final Map map = new HashMap(3);
        map.put( CommandA.class, ObjectStoreA.class );

        final DispatchingObjectStore store
            = new DispatchingObjectStore(map, Collections.EMPTY_MAP);
        final ObjectStoreCommand command = new CommandA();

        final List list = (List)store.execute(command);

        assertEquals( "list.size", 1, list.size() );
        assertSame( command, list.get(0) );
    }


    /**
     * Try passing null into the constructor
     */
    public void testConstructor_Null() {
        try {
            new DispatchingObjectStore(null, Collections.EMPTY_MAP);
            fail("Expected exception");
        }
        catch( final NullPointerException e ) {
            // expected path
        }
    }


    /**
     * Try to dispatch a command for which we haven't defined a store.
     */
    public void testExecute_NotFound() {
        final ObjectStore store
            = new DispatchingObjectStore(new HashMap(3), Collections.EMPTY_MAP);
        try {
            store.execute( new CommandA() );
            fail("Expected exception");
        }
        catch( ObjectStoreCommandNotSupportedException e ) {
            // Expected path
        }
    }


    /**
     * Make sure that findStore() is being called
     */
    public void testFindStore() {

        final List list = new ArrayList();

        final DispatchingObjectStore store = new DispatchingObjectStore(Collections.EMPTY_MAP) {
            protected ObjectStore findStore( final ObjectStoreCommand command ) {
                list.add(command);
                return new ObjectStoreA();
            }
        };
        final ObjectStoreCommand command = new CommandA();

        store.execute(command);

        assertEquals( "list.size", 1, list.size() );
        assertSame( command, list.get(0) );
    }
}
