/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.objectstore;

import com.gargoylesoftware.base.resource.ResourceFactory;
import com.gargoylesoftware.base.resource.ResourceManager;
import com.gargoylesoftware.base.resource.ProxyResourceFactory;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import junit.framework.TestCase;

/**
 *  Tests for object store.
 *
 * @version  $Revision: 1.7 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class ObjectStoreTest extends TestCase {

    /**
     *  Create an instance
     *
     * @param  name The name of the test
     */
    public ObjectStoreTest( final String name ) {
        super( name );
    }


    /**
     *  Test that exceptions are being wrapped in an ObjectStoreException
     */
    public void testService_ExceptionThrown() {
        final Exception exception = new NullPointerException();
        final ExceptionObjectStore store = new ExceptionObjectStore( exception );
        final List collectedExceptions = new ArrayList();
        final List collectedErrors = new ArrayList();
        final List expectedExceptions = new ArrayList();

        expectedExceptions.add( exception );

        store.collectExceptionsInto( collectedExceptions );
        store.collectErrorsInto( collectedErrors );

        try {
            store.execute( new FakeObjectStoreCommand() );
            fail( "Expected exception" );
        }
        catch( final ObjectStoreException e ) {
            assertEquals( exception, e.getException() );
        }

        assertEquals( expectedExceptions, collectedExceptions );
        assertEquals( new ArrayList(), collectedErrors );
    }


    /**
     *  Test that exceptions are being wrapped in an ObjectStoreException
     */
    public void testService_ErrorThrown() {
        final Error error = new OutOfMemoryError();
        final ExceptionObjectStore store = new ExceptionObjectStore( error );
        final List collectedExceptions = new ArrayList();
        final List collectedErrors = new ArrayList();
        final List expectedErrors = new ArrayList();

        expectedErrors.add( error );

        store.collectExceptionsInto( collectedExceptions );
        store.collectErrorsInto( collectedErrors );

        try {
            store.execute( new FakeObjectStoreCommand() );
            fail( "Expected exception" );
        }
        catch( final OutOfMemoryError e ) {
            // Expected path
        }

        assertEquals( new ArrayList(), collectedExceptions );
        assertEquals( expectedErrors, collectedErrors );
    }


    /**
     *  Ensure that ObjectStoreExceptions are not being wrapped
     */
    public void testService_ObjectStoreExceptionThrown() {
        final Exception exception = new ObjectStoreException( "foo" );

        try {
            new ExceptionObjectStore( exception ).execute( new FakeObjectStoreCommand() );
            fail( "Expected exception" );
        }
        catch( final ObjectStoreException e ) {
            assertEquals( exception, e );
        }
    }


    /**
     *  Ensure that ObjectStoreCommandNotSupportedExceptions are not being
     *  wrapped
     * @deprecated
     */
    public void testService_ObjectStoreCommandNotSupportedExceptionThrown() {
        final Exception exception = new ObjectStoreCommandNotSupportedException( "foo" );

        try {
            new ExceptionObjectStore( exception ).execute( new FakeObjectStoreCommand() );
            fail( "Expected exception" );
        }
        catch( final ObjectStoreCommandNotSupportedException e ) {
            assertEquals( exception, e );
        }
    }



    /**
     *  Ensure that an exception is being thrown when null is passed into
     *  setResourceMap()
     */
    public void testSetResourceMap_Null() {
        try {
            new EmptyObjectStore().setResourceFactoryMap( null );
            fail( "Expected exception for null map" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Test setResourceMap()
     */
    public void testSetResourceMap() {
        final ResourceFactory factory = new ProxyResourceFactory( new ArrayList() );
        final ObjectStore store =
            new ObjectStore() {
                protected Object executeImpl( final ObjectStoreCommand command ) {
                    return getResource( "foo" );
                }
            };
        store.setResourceFactoryMap( Collections.singletonMap( "foo", factory ) );
        store.execute( new FakeObjectStoreCommand() );
    }


    /**
     *  The keys in the map must be strings. The values must be
     *  ResourceFactories.
     */
    public void testSetResourceMap_BadKey() {
        final ObjectStore store = new EmptyObjectStore();
        try {
            store.setResourceFactoryMap( Collections.singletonMap( "jdbc", ProxyResourceFactory.class ) );
            fail( "Expected exception for resource factory of wrong type" );
        }
        catch( final ClassCastException e ) {
            // Expected path
        }
    }


    /**
     *  The keys in the map must be strings. The values must be
     *  ResourceFactories.
     */
    public void testSetResourceMap_BadValue() {
        final ObjectStore store = new EmptyObjectStore();
        try {
            store.setResourceFactoryMap( Collections.singletonMap( new Integer( 4 ), new ProxyResourceFactory() ) );
            fail( "Expected exception for key of wrong type" );
        }
        catch( final ClassCastException e ) {
            // Expected path
        }
    }


    /**
     *  Test the accessors for ResourceManager
     */
    public void testSetResourceManager() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        final ObjectStore store = new EmptyObjectStore();
        store.setResourceManager( resourceManager );
        assertEquals( resourceManager, store.getResourceManager() );
    }


    private static class ExceptionObjectStore extends ObjectStore {
        private final Throwable exception_;
        private List collectedExceptions_;
        private List collectedErrors_;


        /**
         *  Create an instance of {1}
         *
         * @param  e
         */
        public ExceptionObjectStore( final Throwable e ) {
            exception_ = e;
        }


        protected Object executeImpl( final ObjectStoreCommand command )
            throws Exception {
            if( exception_ instanceof Exception ) {
                throw ( Exception )exception_;
            }

            throw ( Error )exception_;
        }


        protected Object handleException( final Exception e ) {
            if( collectedExceptions_ != null ) {
                collectedExceptions_.add( e );
            }
            return super.handleException( e );
        }


        protected Object handleError( final Error e ) {
            if( collectedErrors_ != null ) {
                collectedErrors_.add( e );
            }
            return super.handleError( e );
        }

        /** @param list The list that the exceptions will be placed within */
        public void collectExceptionsInto( final List list ) {
            collectedExceptions_ = list;
        }

        /** @param list The list that the exceptions will be placed within */
        public void collectErrorsInto( final List list ) {
            collectedErrors_ = list;
        }
    }


    private static class EmptyObjectStore extends ObjectStore {
        protected Object executeImpl( final ObjectStoreCommand command ) {
            return null;
        }
    }


    private static class FakeObjectStoreCommand extends ObjectStoreCommand {

		private static final long serialVersionUID = 1L;
    }
}

