/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.objectstore;

import junit.framework.TestCase;

/**
 * Tests for ReflectedObjectStore
 *
 * @version  $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class ReflectedObjectStoreTest extends TestCase {
    /**
     *  Create an instance
     *
     * @param  name The name of the test
     */
    public ReflectedObjectStoreTest( final String name ) {
        super( name );
    }


    /**
     * Test calling execute when no commands have been registered.  This should
     * throw an ObjectStoreCommandNotSupportedException
     */
    public void testExecute_NoCommandsRegistered() {
        final TestStore store = new TestStore();

        final ObjectStoreCommand commandB = new CommandB();

        try {
            store.execute( commandB );
            fail( "Expected exception" );
        }
        catch( final ObjectStoreCommandNotSupportedException e ) {
            assertEquals( "command", commandB, e.getCommand() );
        }
    }


    /**
     * Test calling execute when some commands are registered but an unregistered
     * is passed into execute
     */
    public void testExecute_SomeCommandsRegistered_DifferentCommandExecuted() {
        final TestStore store = new TestStore();
        store.registerCommand( CommandA.class, "methodA" );

        final ObjectStoreCommand commandB = new CommandB();

        try {
            store.execute( commandB );
            fail( "Expected exception" );
        }
        catch( final ObjectStoreCommandNotSupportedException e ) {
            assertEquals( "command", commandB, e.getCommand() );
        }
    }


    /**
     * Simple test for execute
     */
    public void testExecute() {
        final TestStore store = new TestStore();
        store.registerCommand( CommandA.class, "methodA" );

        final String result = (String)store.execute( new CommandA() );

        assertEquals( "A", result );
    }


    /**
     * Try to register something that isn't a command class
     */
    public void testRegisterCommand_NotACommand() {
        final TestStore store = new TestStore();
        try {
            store.registerCommand( int.class, "methodA" );
            fail( "Expected IllegalArgumentException" );
        }
        catch( final IllegalArgumentException e ) {
            // expected path
        }
    }


    /**
     * Try registering a method that doesn't exist
     */
    public void testRegisterCommand_BadMethodName() {
        final TestStore store = new TestStore();
        try {
            store.registerCommand( CommandA.class, "foobar" );
            fail( "Excepted exception" );
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }


    private class TestStore extends ReflectedObjectStore {
        /**
         *  Create an instance
         */
        public TestStore() {
        	if( false ) {
        		// Trick so that the "find used methods" function won't identify methodA as unused
        		methodA(null);
        	}
        }


        private Object methodA( final CommandA command ) {
            return "A";
        }
    }


    private class CommandA extends ObjectStoreCommand {

		private static final long serialVersionUID = 1L;
    }


    private class CommandB extends ObjectStoreCommand {

		private static final long serialVersionUID = 1L;
    }
}

