/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource;

import java.util.ArrayList;
import java.util.List;
import junit.framework.TestCase;

/**
 *  tests for PooledResourceFactory
 *
 * @version  $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class PooledResourceFactoryTest extends TestCase {

    /**
     *  Create an instance
     *
     * @param  name the name of the test
     */
    public PooledResourceFactoryTest( final String name ) {
        super( name );
    }


    /**
     *  Test the constructor with a null resource factory
     */
    public void testConstructor_Null() {
        try {
            new PooledResourceFactory( null );
            fail( "Expected NullPointerException" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Test preferredCacheSize
     */
    public void testSetPreferredCacheSize() {
        final List list = new ArrayList();
        final PooledResourceFactory factory
                 = new PooledResourceFactory( new ProxyResourceFactory( list ) );

        factory.setPreferredCacheSize( 0 );
        assertEquals( 0, factory.getPreferredCacheSize() );

        factory.setPreferredCacheSize( 3 );
        assertEquals( 3, factory.getPreferredCacheSize() );

        try {
            factory.setPreferredCacheSize( -1 );
            fail( "preferredCacheSize may not be negative" );
        }
        catch( final IllegalArgumentException e ) {
            assertEquals( 3, factory.getPreferredCacheSize() );
        }
    }


    /**
     *  Test allocating and releasing resources to ensure that the cache is
     *  being used correctly.
     */
    public void testGetAndRelease() {
        final List loggedMethods = new ArrayList();
        final PooledResourceFactory factory
                 = new PooledResourceFactory( new ProxyResourceFactory( loggedMethods ) );
        final ResourceManager resourceManager = new ResourceManager( getName() );

        factory.setPreferredCacheSize( 2 );

        ManagedResource a;
        ManagedResource b;
        ManagedResource c;

        a = factory.getResource( resourceManager );
        assertListContainsAndClear( loggedMethods, "getResourceImpl" );
        factory.releaseResource( resourceManager, a );
        assertListContainsAndClear( loggedMethods, "reinitializeResourceIfPossible" );
        a = factory.getResource( resourceManager );
        assertListEmpty( loggedMethods );

        b = factory.getResource( resourceManager );
        assertListContainsAndClear( loggedMethods, "getResourceImpl" );

        c = factory.getResource( resourceManager );
        assertListContainsAndClear( loggedMethods, "getResourceImpl" );

        factory.releaseResource( resourceManager, a );
        assertListContainsAndClear( loggedMethods, "reinitializeResourceIfPossible" );
        factory.releaseResource( resourceManager, b );
        assertListContainsAndClear( loggedMethods, "reinitializeResourceIfPossible" );
        factory.releaseResource( resourceManager, c );
        assertListContainsAndClear( loggedMethods, "releaseResourceImpl" );
        loggedMethods.clear();
    }


    private void assertListEmpty( final List list ) {
        assertNotNull( "List cannot be null", list );
        if( list.isEmpty() == false ) {
            fail( "Expected empty list but got " + list );
        }
    }


    private void assertListContainsAndClear( final List list, final Object object ) {
        if( list.size() != 1 ) {
            fail( "Expected only one element: got " + list );
        }
        assertEquals( "object", object, list.get( 0 ) );
        list.clear();
    }
}

