/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource;

import junit.framework.TestCase;

/**
 *  Tests for com.gargoylesoftware.base.objectstore.ResourceFactory
 *
 * @version  $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class ResourceFactoryTest extends TestCase {

    /**
     *  Create a test
     *
     * @param  name The name of the test
     */
    public ResourceFactoryTest( final String name ) {
        super( name );
    }



    /**
     *  Try passing null into getResource()
     */
    public void testGetResource_null() {
        try {
            new ProxyResourceFactory().getResource( null );
            fail( "Expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Try passing a null store into releaseResource()
     */
    public void testReleaseResource_nullStore() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        final ProxyResourceFactory factory = new ProxyResourceFactory();
        final ManagedResource resource = factory.getResource( resourceManager );
        try {
            factory.releaseResource( null, resource );
            fail( "Expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Try passing a null resource into releaseResource()
     */
    public void testReleaseResource_nullResource() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        final ProxyResourceFactory factory = new ProxyResourceFactory();
        try {
            factory.releaseResource( resourceManager, null );
            fail( "Expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Try calling releaseResource() with a resource that wasn't allocated from
     *  this factory.
     */
    public void testReleaseResource_NotAllocatedHere() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        final ProxyResourceFactory factory = new ProxyResourceFactory();
        factory.getResource( resourceManager );
        try {
            factory.releaseResource( resourceManager, new ProxyResourceFactory.FakeResource( "foobar" ) );
            fail( "Expected exception" );
        }
        catch( final ResourceException e ) {
            // Expected path
        }
    }


    /**
     *  Try releasing the same resource twice in a row
     */
    public void testReleaseResource_ReleaseTwice() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        final ProxyResourceFactory factory = new ProxyResourceFactory();
        final ManagedResource resource = factory.getResource( resourceManager );
        factory.releaseResource( resourceManager, resource );

        try {
            factory.releaseResource( resourceManager, resource );
            fail( "Expected exception" );
        }
        catch( final ResourceException e ) {
            // Expected path
        }
    }


    /**
     *  Test the normal conditions using releaseResource()
     */
    public void testGetAndRelease() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        final ProxyResourceFactory factory = new ProxyResourceFactory();
        final ManagedResource r1 = factory.getResource( resourceManager );
        final ManagedResource r2 = factory.getResource( resourceManager );

        assertEquals( "r1", "1", r1.toString() );
        assertEquals( "r2", "2", r2.toString() );

        assertEquals( "list.size", 2, factory.getAllocatedResourceCount() );

        factory.releaseResource( resourceManager, r1 );
        factory.releaseResource( resourceManager, r2 );

        assertEquals( "list.size", 0, factory.getAllocatedResourceCount() );
    }


    /**
     *  Test the normal conditions using releaseAllResources()
     */
    public void testGetAndReleaseAll() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        final ProxyResourceFactory factory = new ProxyResourceFactory();
        final ManagedResource r1 = factory.getResource( resourceManager );
        final ManagedResource r2 = factory.getResource( resourceManager );

        assertEquals( "r1", "1", r1.toString() );
        assertEquals( "r2", "2", r2.toString() );

        assertEquals( "list.size", 2, factory.getAllocatedResourceCount() );

        factory.releaseAllResources( resourceManager );

        assertEquals( "list.size", 0, factory.getAllocatedResourceCount() );
    }


    /**
     *  Test allocating and releasing from multiple stores
     */
    public void testGetFromMultipleManagersAndReleaseAll() {
        final ResourceManager resourceManager1 = new ResourceManager( getName() + "-1" );
        final ResourceManager resourceManager2 = new ResourceManager( getName() + "-2" );

        final ProxyResourceFactory factory = new ProxyResourceFactory();
        factory.setAllowReinitialization( false );
        final ManagedResource r1 = factory.getResource( resourceManager1 );
        final ManagedResource r2 = factory.getResource( resourceManager2 );
        final ManagedResource r3 = factory.getResource( resourceManager2 );

        assertEquals( "r1", "1", r1.toString() );
        assertEquals( "r2", "2", r2.toString() );
        assertEquals( "r3", "3", r3.toString() );

        assertEquals( "list.size", 3, factory.getAllocatedResourceCount() );

        factory.releaseAllResources( resourceManager1 );
        assertEquals( "list.size", 2, factory.getAllocatedResourceCount() );

        factory.releaseAllResources( resourceManager2 );
        assertEquals( "list.size", 0, factory.getAllocatedResourceCount() );
    }


    /**
     *  Attempt to reinitialize the specified resource
     *
     * @param  resource the resource to reinitialize
     * @return  true if the resource could be reinitialized. This one always
     *      returns false.
     */
    public boolean reinitializeResourceIfPossible( final ManagedResource resource ) {
        return false;
    }
}

