/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import junit.framework.TestCase;

/**
 *  Tests for ResourceManager
 *
 * @version  $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class ResourceManagerTest extends TestCase {
    /**
     *  Instantiate the test
     *
     * @param  name Name of the test
     */
    public ResourceManagerTest( final String name ) {
        super( name );
    }


    /**
     *  Ensure that an exception is being thrown when we attempt to release a
     *  resource that hadn't been allocated from this factory.
     */
    public void testReleaseResource_ObjectNotFromFactory() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        try {
            resourceManager.releaseResource(
                new ManagedResource() {
                    public void setResourceFactoryName( final String name ) {
                    }


                    public String getResourceFactoryName() {
                        return "Flintstone";
                    }
                } );
            fail( "Expected exception for unregistered name" );
        }
        catch( final ResourceFactoryNotFoundException e ) {
            // Expected path
        }
    }


    /**
     *  Ensure that an exception is being thrown when null is passed into
     *  releaseResource()
     */
    public void testReleaseResource_null() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        try {
            resourceManager.releaseResource( null );
            fail( "Expected exception for null name" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Test the resource allocation
     */
    public void testResourceAllocation() {
        final List list = new ArrayList();
        final ProxyResourceFactory factory = new ProxyResourceFactory( list );

        final ResourceManager resourceManager = new ResourceManager( getName() );
        resourceManager.addFactory( "foo", factory );

        final ManagedResource resource = ( ManagedResource )resourceManager.getResource( "foo" );
        resourceManager.getResource( "foo" );
        resourceManager.releaseResource( resource );
        resourceManager.releaseAllResources();

        final List expected = Arrays.asList( new String[]{
            "getResourceImpl", "getResourceImpl", "releaseResourceImpl",
            "releaseAllResources", "releaseResourceImpl"} );
        assertEquals( expected, list );
    }


    /**
     *  Ensure that an exception is being thrown when null is passed into
     *  getResource
     */
    public void testGetResource_Null() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        resourceManager.addFactory( "jdbc", new ProxyResourceFactory() );
        try {
            resourceManager.getResource( null );
            fail( "Expected NullPointerException" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Ensure that an exception is being thrown when an unregistered name is
     *  passed into getResource
     */
    public void testGetResource_BadName() {
        final ResourceManager resourceManager = new ResourceManager( getName() );
        resourceManager.addFactory( "jdbc", new ProxyResourceFactory() );
        try {
            resourceManager.getResource( "aResourceNameThatShouldNotMatchAnything" );
            fail( "Expected ResourceFactoryNotFoundException" );
        }
        catch( final ResourceFactoryNotFoundException e ) {
            // Expected path
        }
    }

    /** */
    public void testResourceAllocation_Many() {
        final List list = new ArrayList();
        final ProxyResourceFactory proxyResourceFactory = new ProxyResourceFactory( list );
        proxyResourceFactory.setAllowReinitialization(true);

        final PooledResourceFactory factory = new PooledResourceFactory(proxyResourceFactory);
        factory.setPreferredCacheSize(1);

        final ResourceManager resourceManager1 = new ResourceManager( getName() + "1");
        resourceManager1.addFactory( "foo", factory );

        final ResourceManager resourceManager2 = new ResourceManager( getName() + "2");
        resourceManager2.addFactory( "foo", factory );

        final ManagedResource resource = (ManagedResource)resourceManager1.getResource("foo");
        for( int i=0; i<10; i++ ) {
            resourceManager1.getResource( "foo" );
            resourceManager2.getResource( "foo" );
        }
        resourceManager1.releaseResource(resource);
        for( int i=0; i<10; i++ ) {
            resourceManager1.getResource( "foo" );
            resourceManager2.getResource( "foo" );
        }
        resourceManager1.releaseAllResources();
        resourceManager2.releaseAllResources();
    }

}

