/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import com.gargoylesoftware.base.util.ProxyUtil;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Tests for ConnectionWrapper
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class ConnectionWrapperTest extends TestCase {

    /**
     *  Create a test
     *
     * @param  name The name of the test
     */
    public ConnectionWrapperTest( final String name ) {
        super( name );
    }


    /**
     *  Return a test suite containing all the tests for this class
     *
     * @return  The test suite
     */
    public static Test suite() {
        final String excludedMethods[] = {
                "close", "isClosed", "getMetaData"
                };
        final String wrappedMethods[] = {
                "createStatement", "prepareStatement", "prepareCall"
                };
        final TestSuite suite = WrapperTestCase.getWrapperTestSuite(
                Connection.class, ConnectionWrapper.class, excludedMethods, wrappedMethods );

        suite.addTest( new TestSuite( ConnectionWrapperTest.class ) );

        return suite;
    }


    /**
     *  Try passing null into the constructor
     */
    public void testConstructor_Null() {
        try {
            new ConnectionWrapper( null );
            fail( "Expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Test createStatement(). Make sure that it returns a wrapped statement.
     *  Then make sure that the statement is closed when the connection is
     *  closed.
     *
     * @exception  SQLException If an error occurs
     */
    public void testCreateStatement()
        throws SQLException {
        final List list = new ArrayList();
        final Statement statement = (Statement)createTracingProxy( Statement.class, list );
        final Connection connection = (Connection)ProxyUtil.createProxy(
                Connection.class, new Object[][]{{"createStatement", statement}} );

        final ConnectionWrapper connectionWrapper = new ConnectionWrapper( connection );
        final Statement wrappedStatement = connectionWrapper.createStatement();

        assertTrue( "statement is wrapped", wrappedStatement instanceof StatementWrapper );
        assertSame( "wrapped statement", statement, ( (StatementWrapper)wrappedStatement ).getDelegate() );

        assertEquals( "list.size before close", 0, list.size() );
        connectionWrapper.close();
        assertEquals( "list.size after close", 1, list.size() );
        assertEquals( "closeMethod", "close", ( (Method)list.get( 0 ) ).getName() );
    }


    /**
     *  Test createStatement(). Make sure that it returns a wrapped statement.
     *  Then make sure that the statement is closed when the connection is
     *  closed.
     *
     * @exception  SQLException If an error occurs
     */
    public void testCreateStatement_intint()
        throws SQLException {
        final List list = new ArrayList();
        final Statement statement = (Statement)createTracingProxy( Statement.class, list );
        final Connection connection = (Connection)ProxyUtil.createProxy(
                Connection.class, new Object[][]{{"createStatement", statement}} );

        final ConnectionWrapper connectionWrapper = new ConnectionWrapper( connection );
        final Statement wrappedStatement = connectionWrapper.createStatement( 1, 1 );

        assertTrue( "statement is wrapped", wrappedStatement instanceof StatementWrapper );
        assertSame( "wrapped statement", statement, ( (StatementWrapper)wrappedStatement ).getDelegate() );

        assertEquals( "list.size before close", 0, list.size() );
        connectionWrapper.close();
        assertEquals( "list.size after close", 1, list.size() );
        assertEquals( "closeMethod", "close", ( (Method)list.get( 0 ) ).getName() );
    }


    /**
     *  Test prepareStatement(). Make sure that it returns a wrapped statement.
     *  Then make sure that the statement is closed when the connection is
     *  closed.
     *
     * @exception  SQLException If an error occurs
     */
    public void testPrepareStatement()
        throws SQLException {
        final List list = new ArrayList();
        final PreparedStatement statement
                 = (PreparedStatement)createTracingProxy( PreparedStatement.class, list );
        final Connection connection = (Connection)ProxyUtil.createProxy(
                Connection.class, new Object[][]{{"prepareStatement", statement}} );

        final ConnectionWrapper connectionWrapper = new ConnectionWrapper( connection );
        final Statement wrappedStatement = connectionWrapper.prepareStatement( "foo" );

        assertTrue( "statement is wrapped", wrappedStatement instanceof PreparedStatementWrapper );
        assertSame( "wrapped statement", statement, ( (PreparedStatementWrapper)wrappedStatement ).getDelegate() );

        assertEquals( "list.size before close", 0, list.size() );
        connectionWrapper.close();
        assertEquals( "list.size after close", 1, list.size() );
        assertEquals( "closeMethod", "close", ( (Method)list.get( 0 ) ).getName() );
    }


    /**
     *  Test prepareStatement(). Make sure that it returns a wrapped statement.
     *  Then make sure that the statement is closed when the connection is
     *  closed.
     *
     * @exception  SQLException If an error occurs
     */
    public void testPrepareStatement_StringIntInt()
        throws SQLException {
        final List list = new ArrayList();
        final PreparedStatement statement
                 = (PreparedStatement)createTracingProxy( PreparedStatement.class, list );
        final Connection connection = (Connection)ProxyUtil.createProxy(
                Connection.class, new Object[][]{{"prepareStatement", statement}} );

        final ConnectionWrapper connectionWrapper = new ConnectionWrapper( connection );
        final Statement wrappedStatement = connectionWrapper.prepareStatement( "foo", 1, 1 );

        assertTrue( "statement is wrapped", wrappedStatement instanceof PreparedStatementWrapper );
        assertSame( "wrapped statement", statement, ( (PreparedStatementWrapper)wrappedStatement ).getDelegate() );

        assertEquals( "list.size before close", 0, list.size() );
        connectionWrapper.close();
        assertEquals( "list.size after close", 1, list.size() );
        assertEquals( "closeMethod", "close", ( (Method)list.get( 0 ) ).getName() );
    }


    /**
     *  Test prepareCall(). Make sure that it returns a wrapped statement. Then
     *  make sure that the statement is closed when the connection is closed.
     *
     * @exception  SQLException If an error occurs
     */
    public void testPrepareCall_String()
        throws SQLException {
        final List list = new ArrayList();
        final CallableStatement statement
                 = (CallableStatement)createTracingProxy( CallableStatement.class, list );
        final Connection connection = (Connection)ProxyUtil.createProxy(
                Connection.class, new Object[][]{{"prepareCall", statement}} );

        final ConnectionWrapper connectionWrapper = new ConnectionWrapper( connection );
        final Statement wrappedStatement = connectionWrapper.prepareCall( "foo" );

        assertTrue( "statement is wrapped", wrappedStatement instanceof CallableStatementWrapper );
        assertSame( "wrapped statement", statement, ( (CallableStatementWrapper)wrappedStatement ).getDelegate() );

        assertEquals( "list.size before close", 0, list.size() );
        connectionWrapper.close();
        assertEquals( "list.size after close", 1, list.size() );
        assertEquals( "closeMethod", "close", ( (Method)list.get( 0 ) ).getName() );
    }


    /**
     *  Test prepareCall(). Make sure that it returns a wrapped statement. Then
     *  make sure that the statement is closed when the connection is closed.
     *
     * @exception  SQLException If an error occurs
     */
    public void testPrepareCall_StringIntInt()
        throws SQLException {
        final List list = new ArrayList();
        final CallableStatement statement
                 = (CallableStatement)createTracingProxy( CallableStatement.class, list );
        final Connection connection = (Connection)ProxyUtil.createProxy(
                Connection.class, new Object[][]{{"prepareCall", statement}} );

        final ConnectionWrapper connectionWrapper = new ConnectionWrapper( connection );
        final Statement wrappedStatement = connectionWrapper.prepareCall( "foo", 1, 1 );

        assertTrue( "statement is wrapped", wrappedStatement instanceof CallableStatementWrapper );
        assertSame( "wrapped statement", statement, ( (CallableStatementWrapper)wrappedStatement ).getDelegate() );

        assertEquals( "list.size before close", 0, list.size() );
        connectionWrapper.close();
        assertEquals( "list.size after close", 1, list.size() );
        assertEquals( "closeMethod", "close", ( (Method)list.get( 0 ) ).getName() );
    }


    /**
     *  Test the getMetaData() method to ensure that:
     *  <ol>
     *    <li> The DatabaseMetaData object is wrapped
     *    <li> The DatabaseMetaData object returns the wrapped connection when
     *    getConnection is called on it.
     *  </ol>
     *
     *
     * @exception  Exception If an error occurs
     */
    public void testGetDatabaseMetaData()
        throws Exception {
        final DatabaseMetaData metaData
                 = (DatabaseMetaData)ProxyUtil.createProxy( DatabaseMetaData.class );
        final Connection connection = (Connection)ProxyUtil.createProxy(
                Connection.class, new Object[][]{{"getMetaData", metaData}} );

        final ConnectionWrapper connectionWrapper = new ConnectionWrapper( connection );
        final DatabaseMetaData wrappedMetaData = connectionWrapper.getMetaData();

        assertTrue( "metadata is wrapped", wrappedMetaData instanceof DatabaseMetaDataWrapper );
        assertSame( "wrapped metaData", metaData, ( (DatabaseMetaDataWrapper)wrappedMetaData ).getDelegate() );

        assertEquals( "metaData.getConnection", connectionWrapper, wrappedMetaData.getConnection() );

        connectionWrapper.close();

        try {
            wrappedMetaData.getConnection();
            fail( "Expected exception" );
        }
        catch( final SQLException e ) {
            // Expected path
        }
    }


    /**
     *  Try closing the connection twice
     *
     * @exception  Exception If an error occurs
     */
    public void testCloseTwice()
        throws Exception {
        final Connection connection = (Connection)ProxyUtil.createProxy( Connection.class );
        final ConnectionWrapper wrapper = new ConnectionWrapper( connection );

        wrapper.close();
        try {
            wrapper.close();
            fail( "Expected exception when calling close() on a closed object" );
        }
        catch( final AlreadyClosedException e ) {
            // Expected path
        }
    }


    /**
     *  Regression test for bug where isClosed() would throw a
     *  NullPointerException if the connection was already closed
     *
     * @exception  Exception If an error occurs
     */
    public void testIsClosedOnClosedConnection()
        throws Exception {
        final Connection connection = (Connection)ProxyUtil.createProxy( Connection.class );
        final ConnectionWrapper wrapper = new ConnectionWrapper( connection );

        wrapper.close();
        assertTrue( wrapper.isClosed() );
    }


    /**
     *  Test releaseAll where one of the statements has already been closed
     *
     * @exception  Exception If an error occurs
     */
    public void testCloseWhenStatementAlreadyClosed()
        throws Exception {
        final Statement statement
                 = (Statement)ProxyUtil.createProxy( Statement.class );

        final Connection connection
                 = (Connection)ProxyUtil.createProxy( Connection.class,
                new Object[][]{{"createStatement", new StatementWrapper( statement )}} );

        final ConnectionWrapper connectionWrapper
                 = new ConnectionWrapper( connection );

        final StatementWrapper statementWrapper
                 = (StatementWrapper)connectionWrapper.createStatement();
        statementWrapper.close();

        connectionWrapper.close();
    }


    /**
     *  Create a proxy object that tracks what methods were called
     *
     * @param  clazz The interface used to create the proxy
     * @param  list The list that will contain all the method objects
     * @return  The new proxy object
     */
    private Object createTracingProxy( final Class clazz, final List list ) {
        final InvocationHandler handler =
            new InvocationHandler() {
                public Object invoke( Object proxy, Method method, Object[] methodArgs ) {
                    list.add( method );
                    return null;
                }
            };
        return Proxy.newProxyInstance(
                clazz.getClassLoader(),
                new Class[]{clazz},
                handler );
    }
}

