/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import com.gargoylesoftware.base.util.ProxyUtil;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Tests for DatabaseMetaDataWrapper
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DatabaseMetaDataWrapperTest extends TestCase {

    /**
     *  Create a test
     *
     * @param  name The name of the test
     */
    public DatabaseMetaDataWrapperTest( final String name ) {
        super( name );
    }


    /**
     *  Return a test suite containing all the tests for this class
     *
     * @return  The test suite
     */
    public static Test suite() {
        final String excludedMethods[] = {
                "close", "getDriverMajorVersion", "getDriverMinorVersion", "getConnection"
                };
        final String wrappedMethods[] = {
                "getProcedures", "getTables", "getSchemas", "getCatalogs", "getTableTypes",
                "getColumns", "getColumnPrivileges", "getTablePrivileges", "getBestRowIdentifier",
                "getVersionColumns", "getPrimaryKeys", "getImportedKeys", "getExportedKeys",
                "getCrossReference", "getTypeInfo", "getIndexInfo", "getUDTs", "getProcedureColumns"
                };
        final TestSuite suite = WrapperTestCase.getWrapperTestSuite(
                DatabaseMetaData.class, DatabaseMetaDataWrapper.class, excludedMethods, wrappedMethods );

        suite.addTest( new TestSuite( DatabaseMetaDataWrapperTest.class ) );

        return suite;
    }


    /**
     *  Try passing a null into the constructor
     */
    public void testConstructor_null() {
        try {
            new DatabaseMetaDataWrapper( null );
            fail( "Expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Try closing the metadata twice
     *
     * @exception  Exception If an error occurs
     */
    public void testCloseTwice()
        throws Exception {
        final DatabaseMetaData metaData
                 = (DatabaseMetaData)ProxyUtil.createProxy( DatabaseMetaData.class );
        final DatabaseMetaDataWrapper wrapper = new DatabaseMetaDataWrapper( metaData );

        wrapper.close();
        try {
            wrapper.close();
            fail( "Expected exception when calling close() on a closed object" );
        }
        catch( final AlreadyClosedException e ) {
            // Expected path
        }
    }


    /**
     *  Test close where one of the result sets has already been closed
     *
     * @exception  Exception If an error occurs
     */
    public void testCloseWhenResultSetAlreadyClosed()
        throws Exception {
        final ResultSet resultSet
                 = (ResultSet)ProxyUtil.createProxy( ResultSet.class );

        final DatabaseMetaData metaData
                 = (DatabaseMetaData)ProxyUtil.createProxy( DatabaseMetaData.class,
                new Object[][]{{"getCatalogs", new ResultSetWrapper( resultSet )}} );

        final DatabaseMetaDataWrapper metaDataWrapper
                 = new DatabaseMetaDataWrapper( metaData );

        final ResultSetWrapper resultSetWrapper
                 = (ResultSetWrapper)metaDataWrapper.getCatalogs();
        resultSetWrapper.close();

        metaDataWrapper.close();
    }
}

