/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import com.gargoylesoftware.base.resource.ManagedResource;
import com.gargoylesoftware.base.resource.ResourceManager;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.sql.Connection;
import java.sql.SQLException;
import junit.framework.TestCase;

/**
 *  Tests for com.gargoylesoftware.base.objectstore.jdbc.JDBCResourceFactory
 *
 * @version  $Revision: 1.6 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class JDBCResourceFactoryTest extends TestCase {

    /**
     *  Create a test
     *
     * @param  name The name of the test
     */
    public JDBCResourceFactoryTest( final String name ) {
        super( name );
    }


    /**
     *  Create a proxy connection object
     *
     * @return  The proxy
     */
    public static Connection makeConnection() {
        final InvocationHandler handler =
            new InvocationHandler() {
                public Object invoke( Object proxy, Method method, Object[] args ) {
                    return null;
                }
            };

        return (Connection)Proxy.newProxyInstance(
                ManagedResource.class.getClassLoader(),
                new Class[]{Connection.class, ManagedResource.class},
                handler );
    }


    /**
     *  Test creating an instance with a null database name
     *
     * @exception  Exception If an error occurs
     */
    public void testConstructor_NullDatabaseName()
        throws Exception {
        try {
            new JDBCResourceFactory( null, "foo", "foo" );
            fail( "Exception exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Test creating an instance with a null user id
     *
     * @exception  Exception If an error occurs
     */
    public void testConstructor_NullUser()
        throws Exception {
        try {
            new JDBCResourceFactory( "foo", null, "foo" );
            fail( "Exception exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Test creating an instance with a null password
     *
     * @exception  Exception If an error occurs
     */
    public void testConstructor_NullPassword()
        throws Exception {
        try {
            new JDBCResourceFactory( "foo", "foo", null );
            fail( "Exception exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Make sure that the constructor will throw an exception if we cannot open
     *  a connection to the database.
     */
    public void testConstructor_NoConnection() {
        try {
            /** Fake class for testing */
            class MyResourceFactory extends JDBCResourceFactory {
                /**
                 * @param database The database
                 * @param user The user
                 * @param password The password
                 * @throws SQLException If something fails
                 */
                public MyResourceFactory(
                        final String database, final String user, final String password )
                    throws SQLException {
                    super(database, user, password);
                }
                /**
                 * @param manager The resource manager
                 * @return The resource
                 * @throws SQLException If something fails
                 */
                protected ManagedResource getResourceImpl( final ResourceManager manager )
                    throws SQLException {

                    throw new SQLException( "NoConnectionTest" );
                }
            }
            new MyResourceFactory( "database", "user", "password" );
            fail( "Expected exception" );
        }
        catch( final SQLException e ) {
            assertEquals( "NoConnectionTest", e.getMessage() );
        }
    }
}

