/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import com.gargoylesoftware.base.util.ProxyUtil;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Tests for StatementWrapper
 *
 * @version  $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class StatementWrapperTest extends TestCase {

    /**
     *  Create a test
     *
     * @param  name The name of the test
     */
    public StatementWrapperTest( final String name ) {
        super( name );
    }


    /**
     *  Return a test suite containing all the tests for this class
     *
     * @return  The test suite
     */
    public static Test suite() {
        final String excludedMethods[] = {
                "close", "getConnection"
                };
        final String wrappedMethods[] = {
                "executeQuery", "getResultSet", "getConnection"
                };
        final TestSuite suite = WrapperTestCase.getWrapperTestSuite(
                Statement.class, StatementWrapper.class, excludedMethods, wrappedMethods );

        suite.addTest( new TestSuite( StatementWrapperTest.class ) );

        return suite;
    }


    /**
     *  Test the constructor with a null statement
     */
    public void testConstructor_NullStatement() {
        try {
            new StatementWrapper( null );
            fail( "Expected exception for null statement" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Test the constructor
     *
     * @exception  Exception If an error occurs
     */
    public void testConstructor()
        throws Exception {
        final Statement statement = (Statement)ProxyUtil.createProxy( Statement.class );
        final StatementWrapper wrapper = new StatementWrapper( statement );

        assertSame( "delegate", statement, wrapper.getDelegate() );
    }


    /**
     *  Test setConnection()
     *
     * @exception  Exception If an error occurs
     */
    public void testSetConnection()
        throws Exception {
        final Connection connection = (Connection)ProxyUtil.createProxy( Connection.class );
        final Statement statement = (Statement)ProxyUtil.createProxy( Statement.class );
        final StatementWrapper wrapper = new StatementWrapper( statement );
        wrapper.setConnection( connection );

        assertSame( "connection", connection, wrapper.getConnection() );
    }


    /**
     *  Test setConnection() with a null connection
     */
    public void testSetConnection_NullConnection() {
        final StatementWrapper wrapper
                 = new StatementWrapper( (Statement)ProxyUtil.createProxy( Statement.class ) );
        try {
            wrapper.setConnection( null );
            fail( "Expected exception for null connection" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Try closing the statement twice
     *
     * @exception  Exception If an error occurs
     */
    public void testCloseTwice()
        throws Exception {
        final Statement statement = (Statement)ProxyUtil.createProxy( Statement.class );
        final StatementWrapper wrapper = new StatementWrapper( statement );

        wrapper.close();
        try {
            wrapper.close();
            fail( "Expected exception when calling close() on a closed object" );
        }
        catch( final AlreadyClosedException e ) {
            // Expected path
        }
    }


    /**
     *  Test close where one of the result sets has already been closed
     *
     * @exception  Exception If an error occurs
     */
    public void testCloseWhenResultSetAlreadyClosed()
        throws Exception {
        final ResultSet resultSet
                 = (ResultSet)ProxyUtil.createProxy( ResultSet.class );

        final Statement statement
                 = (Statement)ProxyUtil.createProxy( Statement.class,
                new Object[][]{{"executeQuery", new ResultSetWrapper( resultSet )}} );

        final StatementWrapper statementWrapper
                 = new StatementWrapper( statement );

        final ResultSetWrapper resultSetWrapper
                 = (ResultSetWrapper)statementWrapper.executeQuery( "foo" );
        resultSetWrapper.close();

        statementWrapper.close();
    }
}

