/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.resource.jdbc;

import com.gargoylesoftware.base.util.DetailedIllegalArgumentException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Savepoint;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  A TestCase that dynamically generates a seperate test for each method in a
 *  specified interface. This is used to test the various jdbc wrappers.
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class WrapperTestCase extends TestCase {

    private final Method method_;
    private final Class interfaceToTest_;
    private final Constructor wrapperConstructor_;
    private final boolean isResultWrapped_;


    /**
     *  Create an instance
     *
     * @param  testName
     * @param  interfaceToTest
     * @param  method
     * @param  wrapperConstructor
     * @param  isResultWrapped
     */
    private WrapperTestCase(
            final String testName,
            final Class interfaceToTest,
            final Method method,
            final Constructor wrapperConstructor,
            final boolean isResultWrapped ) {

        super( testName );
        method_ = method;
        interfaceToTest_ = interfaceToTest;
        wrapperConstructor_ = wrapperConstructor;
        isResultWrapped_ = isResultWrapped;
    }


    /**
     *  Create a test suite containing a seperate test for each method in
     *  'interfaceToTest'
     *
     * @param  interfaceToTest The interface being tested
     * @param  wrapperClass The concrete class that we are testing. This
     *      implements the interface 'interfaceToTest'
     * @param  excludedMethods An array of method names. These methods will not
     *      be tested.
     * @param  wrappedMethods An array of method names. These methods are
     *      expected to return wrapped objects which contain the original result
     * @return  A test suite
     */
    public static TestSuite getWrapperTestSuite(
            final Class interfaceToTest,
            final Class wrapperClass,
            final String excludedMethods[],
            final String wrappedMethods[] ) {

        if( interfaceToTest == null ) {
            throw new NullPointerException( "interfaceToTest" );
        }
        if( excludedMethods == null ) {
            throw new NullPointerException( "excludedMethods" );
        }
        if( wrappedMethods == null ) {// Implement support for this
            throw new NullPointerException( "wrappedMethods" );
        }
        if( wrapperClass == null ) {
            throw new NullPointerException( "wrapperClass" );
        }

        final Constructor wrapperConstructor;
        try {
            wrapperConstructor = wrapperClass.getConstructor( new Class[]{interfaceToTest} );
        }
        catch( final NoSuchMethodException e ) {
            throw new DetailedIllegalArgumentException(
                "wrapperClass", wrapperClass,
                "Does not have a constructor that takes an instance of "+interfaceToTest.getName() );
        }

        String testName;

        final TestSuite suite = new TestSuite();
        final Method methods[] = interfaceToTest.getMethods();
        int i;
        for( i = 0; i < methods.length; i++ ) {
            if( contains( methods[i], excludedMethods ) == false ) {
                testName = createTestName( wrapperClass, methods[i] );
                suite.addTest( new WrapperTestCase( testName,
                        interfaceToTest,
                        methods[i],
                        wrapperConstructor,
                        contains( methods[i], wrappedMethods ) ) );
            }
        }

        return suite;
    }


    private static String createTestName( final Class wrapperClass, final Method method ) {
        final StringBuffer buffer = new StringBuffer( 100 );
        buffer.append( "Method: " );
        buffer.append( wrapperClass.getName() );
        buffer.append( '.' );
        buffer.append( method.getName() );
        buffer.append( '(' );

        final Class parameterClasses[] = method.getParameterTypes();

        int i;
        for( i = 0; i < parameterClasses.length; i++ ) {
            if( i != 0 ) {
                buffer.append( "," );
            }

            buffer.append( parameterClasses[i].getName() );
        }
        buffer.append( ")" );
        return buffer.toString();
    }


    private static boolean contains( final Method method, final String array[] ) {
        final String methodName = method.getName();

        int i;
        for( i = 0; i < array.length; i++ ) {
            if( methodName.equals( array[i] ) ) {
                return true;
            }
        }

        return false;
    }


    /**
     *  During normal operation, the wrapper should allow all method calls to
     *  pass through to the wrapped object. Verify that this is working
     *  correctly.
     *
     * @exception  Exception If an error occurs
     */
    protected void runTest()
        throws Exception {

        final Method methods[] = interfaceToTest_.getMethods();
        assertTrue( "the interface has methods", methods.length != 0 );

        final List list = new ArrayList();
        final InvocationHandler handler =
            new InvocationHandler() {
                public Object invoke( Object proxy, Method method, Object[] args ) {
                    if( method.getName().equals( "toString" ) ) {
                        return "Proxy(" + interfaceToTest_.getClass().getName() + ")";
                    }

                    list.add( method );
                    if( args != null ) {
                        int i;
                        for( i = 0; i < args.length; i++ ) {
                            list.add( args[i] );
                        }
                    }

                    if( method.getReturnType().equals( void.class ) ) {
                        return null;
                    }

                    final Object rc = makeObject( method.getReturnType() );
                    list.add( rc );
                    return rc;
                }
            };

        final Object proxy = Proxy.newProxyInstance(
                getClass().getClassLoader(),
                new Class[]{interfaceToTest_},
                handler );
        final Object testObject = wrapperConstructor_.newInstance( new Object[]{proxy} );

        testPassthrough( testObject, method_, list );

        closeObject( testObject );

        try {
            testPassthrough( testObject, method_, list );
            fail( "Expected exception when invoking method on a closed object" );
        }
        catch( final InvocationTargetException e ) {
            assertTrue( e.getTargetException() instanceof SQLException );
        }
    }


    private void testPassthrough(
            final Object testObject,
            final Method method,
            final List list )
        throws
            Exception {

        list.clear();

        if( false ) {
            System.out.println( "ConnectionWrapperTest.testPassthrough() method=" + method );
        }

        final Class parameterTypes[] = method.getParameterTypes();
        final Object parameters[] = new Object[parameterTypes.length];

        int i;
        for( i = 0; i < parameterTypes.length; i++ ) {
            parameters[i] = makeObject( parameterTypes[i] );
        }

        final Object rc = method.invoke( testObject, parameters );

        final boolean methodHasReturn = ( method.getReturnType().equals( void.class ) == false );
        int expectedListSize = parameters.length + 1;
        if( methodHasReturn ) {
            expectedListSize++;
        }
        assertEquals( "list.size", expectedListSize, list.size() );

        assertEquals( "method", method, list.get( 0 ) );
        for( i = 0; i < parameters.length; i++ ) {
            assertEquals( "Parameter[" + i + "]", parameters[i], list.get( i + 1 ) );
        }

        if( methodHasReturn ) {
            final Object expectedResult = list.get( i + 1 );
            assertNotNull( "expectedResult", expectedResult );
            if( isResultWrapped_ ) {
                Method getDelegateMethod = null;
                try {
                    getDelegateMethod = rc.getClass().getMethod( "getDelegate", new Class[0] );
                }
                catch( final NoSuchMethodException e ) {
                    fail( "Returned object doesn't have a getDelegate() method.  Likely it isn't a wrapper." );
                }
                final Object delegate = getDelegateMethod.invoke( rc, new Object[0] );

                assertEquals( "returned delegate", expectedResult, delegate );
            }
            else {
                assertEquals( "returned object", expectedResult, rc );
            }
        }

    }


    private Object makeObject( final Class clazz ) {
        if( clazz == int.class ) {
            return new Integer( 5 );
        }
        if( clazz == double.class ) {
            return new Double( 5.0 );
        }
        if( clazz == float.class ) {
            return new Float( 5 );
        }
        if( clazz == byte.class ) {
            return new Byte( (byte)5 );
        }
        if( clazz == short.class ) {
            return new Short( (short)5 );
        }
        if( clazz == long.class ) {
            return new Long( 5 );
        }
        if( clazz == boolean.class ) {
            return new Boolean( true );
        }
        if( clazz == void.class ) {
            return null;
        }
        if( clazz == String.class ) {
            return new String( "" );
        }
        if( clazz == Map.class ) {
            return new HashMap( 3 );
        }
        if( clazz == SQLWarning.class ) {
            return new SQLWarning();
        }
        if( clazz == int[].class ) {
            return new int[]{1, 2, 3};
        }
        if( clazz == byte[].class ) {
            return new byte[]{};
        }
        if( clazz == String[].class ) {
            return new String[]{};
        }
        if( clazz == URL.class ) {
            try {
                return new URL("http://foo.com");
            }
            catch( final MalformedURLException cantHappenButCompilerRequiresUsToCatchSomething ) {
                cantHappenButCompilerRequiresUsToCatchSomething.printStackTrace();
            }
        }

        if( clazz == Statement.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == PreparedStatement.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == CallableStatement.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == DatabaseMetaData.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == Connection.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == ResultSet.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == Savepoint.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == ParameterMetaData.class ) {
            return makeEmptyProxy( clazz );
        }

        if( clazz == java.util.Calendar.class ) {
            return java.util.Calendar.getInstance();
        }
        if( clazz == java.math.BigDecimal.class ) {
            return new java.math.BigDecimal( 1 );
        }
        if( clazz == java.sql.Date.class ) {
            return new java.sql.Date( 1 );
        }
        if( clazz == java.sql.Time.class ) {
            return new java.sql.Time( 1 );
        }
        if( clazz == java.sql.Timestamp.class ) {
            return new java.sql.Timestamp( 1 );
        }
        if( clazz == java.sql.Array.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == java.sql.Ref.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == java.sql.Clob.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == java.sql.Blob.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == java.sql.ResultSetMetaData.class ) {
            return makeEmptyProxy( clazz );
        }
        if( clazz == java.io.InputStream.class ) {
            return new java.io.ByteArrayInputStream( new byte[0] );
        }
        if( clazz == java.io.Reader.class ) {
            return new java.io.StringReader( "foo" );
        }
        if( clazz == Object.class ) {
            return new Object();
        }

        System.out.println( "WrapperTestCase.makeObject(" + clazz.getName() + ")" );
        return null;
    }


    private Object makeEmptyProxy( final Class clazz ) {
        final InvocationHandler handler =
            new InvocationHandler() {
                public Object invoke( Object proxy, Method method, Object[] args ) {
                    final String methodName = method.getName();

                    if( methodName.equals( "equals" ) && args.length == 1 ) {
                        return new Boolean( proxy == args[0] );
                    }
                    else if( methodName.equals( "toString" ) ) {
                        return "Proxy(" + clazz.getName() + ")";
                    }
                    return null;
                }
            };

        return Proxy.newProxyInstance(
                getClass().getClassLoader(),
                new Class[]{clazz},
                handler );
    }


    private void closeObject( final Object object )
        throws Exception {
        final Method closeMethod = object.getClass().getMethod( "close", new Class[0] );
        closeMethod.invoke( object, new Object[0] );
    }
}

