/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.testing;

import junit.framework.TestCase;

/**
 *  Tests for OrderedTestSuite
 *
 * @version  $Revision: 1.4 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class OrderedTestSuiteTest extends TestCase {
    /**
     *  Create an instance
     *
     * @param  name Name of the test
     */
    public OrderedTestSuiteTest( final String name ) {
        super( name );
    }


    /**
     *  Try passing a null class into the constructor
     */
    public void testConstructor_NullClass() {
        try {
            new OrderedTestSuite( null, new String[0] );
            fail( "Expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Try passing a null method list into the constructor
     */
    public void testConstructor_NullMethodList() {
        try {
            new OrderedTestSuite( MyTestCase.class, null );
            fail( "Expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Pass in a class that is not a subclass of TestCase
     */
    public void testConstructor_NonTestCase() {
        try {
            new OrderedTestSuite( String.class, new String[0] );
            fail( "Expected exception" );
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }


    /**
     *  Test constructor - good path
     */
    public void testConstructor() {
        final OrderedTestSuite suite = new OrderedTestSuite( MyTestCase.class, new String[0] );
        assertEquals( 3, suite.testCount() );
    }


    /**
     *  Test that the ordering is correct
     */
    public void testOrdering() {

        doTestOrdering( new String[]{"testThree", "testTwo", "testOne"} );
        doTestOrdering( new String[]{"testOne", "testTwo", "testThree"} );
    }


    /**
     *  Test order with one method that doesn't start with "test"
     */
    public void testOrdering_NonTest() {
        doTestOrdering( new String[]{"testThree", "testTwo", "doMyTest", "testOne"} );
    }


    /**
     *  Test ordering with one bad method name
     */
    public void testOrdering_BadName() {
        try {
            doTestOrdering( new String[]{"testThree", "testTwo", "xx", "testOne"} );
            fail( "Expected exception" );
        }
        catch( final TestInitializationFailedException e ) {
            // expected path
        }
    }


    private void doTestOrdering( final String methodNames[] ) {
        final OrderedTestSuite suite = new OrderedTestSuite( MyTestCase.class, methodNames );
        assertEquals( methodNames.length, suite.testCount() );

        int i;
        for( i = 0; i < methodNames.length; i++ ) {
            assertEquals( methodNames[i], ( (TestCase)suite.testAt( i ) ).getName() );
        }
    }


    /**
     *  Fake TestCase class for testing
     *
     * @version  $Revision: 1.4 $
     */
    public static class MyTestCase extends TestCase {
        /**
         *  Instantiate one
         *
         * @param  name Name of the test
         */
        public MyTestCase( final String name ) {
            super( name );
        }


        /**
         *  Test method
         */
        public void doMyTest() {
        }


        /**
         *  Test method
         */
        public void testOne() {
        }


        /**
         *  Test method
         */
        public void testTwo() {
        }


        /**
         *  Test method
         */
        public void testThree() {
        }


        /**
         *  Test method
         *
         * @param  arg dummy arg
         */
        public void testWithArg( final Object arg ) {
        }


        /**
         *  Test method
         *
         * @return  true
         */
        public boolean testThatReturns() {
            return true;
        }


        /**
         *  Test method
         */
        private void testPrivate() {
        }
    }

}

