/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.trace;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

/**
 * Tests for Trace.
 *
 * @version $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class TraceTest extends TestCase {

    /**
     *
     */
    private class LogEntry {
        private final Thread thread_;
        private final TraceItem traceItem_;
        /** @param item The trace item. */
        public LogEntry( final TraceItem item ) {
            try {
                traceItem_ = (TraceItem)item.clone();
            }
            catch( final CloneNotSupportedException e ) {
                throw new AssertionFailedError("TraceItem not cloneable");
            }
            thread_ = Thread.currentThread();
        }
    }

    /**
     *
     */
    private class LogTraceWriter implements TraceWriter {
        private final List log_;
        /** @param log The list that will collect the log items */
        public LogTraceWriter( final List log ) {
            log_ = log;
        }
        /** @param item The item to be written */
        public void write( final TraceItem item ) {
            log_.add( new LogEntry(item) );
        }
    }

    /**
     * Create a test.
     * @param name The name of the test.
     */
    public TraceTest( final String name ) {
        super(name);
    }

    /**
     * Test that we are logging on a seperate thread when buffering is
     * enabled.
     */
    public void testSecondThreadWhenBuffered() {
        final List log = Collections.synchronizedList(new ArrayList());
        final LogTraceWriter logWriter = new LogTraceWriter(log);
        final TraceChannel channel = new TraceChannel("test");
        channel.addTraceWriter(logWriter);

        Trace.getController().setBufferingEnabled(true);
        Trace.print(channel, "foo");
        Trace.getController().setBufferingEnabled(false);
        Trace.print(channel, "bar");
        Trace.getController().setBufferingEnabled(true);
        Trace.print(channel, "test");
        Trace.flush();

        final Thread currentThread = Thread.currentThread();

        assertEquals( "Number of items", 3, log.size() );
        LogEntry logEntry;

        logEntry = (LogEntry)log.get(0);
        assertEquals( "First item contents", "foo", logEntry.traceItem_.getMessage() );
        assertTrue( "First item thread", currentThread != logEntry.thread_ );

        logEntry = (LogEntry)log.get(1);
        assertEquals( "Second item contents", "bar", logEntry.traceItem_.getMessage() );
        assertTrue( "Second item thread", currentThread == logEntry.thread_ );

        logEntry = (LogEntry)log.get(2);
        assertEquals( "Third item contents", "test", logEntry.traceItem_.getMessage() );
        assertTrue( "Third item thread", currentThread != logEntry.thread_ );
    }
}
