/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.util;

import java.io.File;
import java.io.FileFilter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import junit.framework.TestCase;

/**
 *  Tests For DirectoryWalker
 *
 * @version  $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DirectoryWalkerTest extends TestCase {

    private final String dirA_;
    private final String dirB_;
    private final String dirC_;
    private final String dirD_;
    private final String dirRoot_;
    private final String fileA1_;
    private final String fileA2_;
    private final String fileB_;
    private final String fileD_;


    /**
     *  Instantiate the test
     *
     * @param  name The name of the test
     */
    public DirectoryWalkerTest( final String name ) {
        super( name );

        final String sep = File.separator;

        dirA_ = "directorywalker" + sep + "dira";
        dirB_ = "directorywalker" + sep + "dira" + sep + "dirb";
        dirC_ = "directorywalker" + sep + "dirc";
        dirD_ = "directorywalker" + sep + "dirc" + sep + "dird";
        dirRoot_ = "directorywalker";
        fileA1_ = "directorywalker" + sep + "dira" + sep + "filea1";
        fileA2_ = "directorywalker" + sep + "dira" + sep + "filea2";
        fileB_ = "directorywalker" + sep + "dira" + sep + "dirb" + sep + "fileb";
        fileD_ = "directorywalker" + sep + "dirc" + sep + "dird" + sep + "filed";
    }



    /**
     *  The JUnit setup method
     *
     * @exception  IOException If an error occurs
     */
    public void setUp()
        throws IOException {

        createDir( dirRoot_ );
        createDir( dirA_ );
        createDir( dirB_ );
        createDir( dirC_ );
        createDir( dirD_ );

        createFile( fileA1_ );
        createFile( fileA2_ );
        createFile( fileB_ );
        createFile( fileD_ );
    }


    /**
     *  The teardown method for JUnit
     *
     * @exception  IOException If an error occurs
     */
    public void tearDown()
        throws IOException {

        delete( fileB_ );
        delete( dirB_ );
        delete( fileA1_ );
        delete( fileA2_ );
        delete( dirA_ );
        delete( fileD_ );
        delete( dirD_ );
        delete( dirC_ );
        delete( dirRoot_ );
    }


    /**
     *  Test the constructor with a null starting directory
     */
    public void testConstructor_Null() {
        try {
            new DirectoryWalker( null );
            fail( "expected exception" );
        }
        catch( final NullPointerException e ) {
            // Expected path
        }
    }


    /**
     *  Test the constructor with an empty string as a starting directory
     */
    public void testConstructor_Empty() {
        try {
            new DirectoryWalker( "" );
            fail( "expected exception" );
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }


    /**
     *  Test the good case for getFiles()
     *
     * @exception  Exception If an error occurs
     */
    public void testGetFiles()
        throws Exception {
        final Collection expectedValues = new HashSet( 7 );
        expectedValues.add( new File( fileB_ ) );
        expectedValues.add( new File( fileD_ ) );
        expectedValues.add( new File( fileA1_ ) );
        expectedValues.add( new File( fileA2_ ) );

        final DirectoryWalker walker = new DirectoryWalker( dirRoot_ );
        final FileFilter filter = new AcceptAllFilter();
        assertCollectionsEqual( expectedValues, walker.getFiles( filter ) );
    }


    /**
     *  Test the good case for getDirectories()
     *
     * @exception  Exception If an error occurs
     */
    public void testGetDirectories()
        throws Exception {
        final Collection expectedValues = new HashSet( 7 );
        expectedValues.add( new File( dirA_ ) );
        expectedValues.add( new File( dirB_ ) );
        expectedValues.add( new File( dirC_ ) );
        expectedValues.add( new File( dirD_ ) );
        expectedValues.add( new File( dirRoot_ ) );

        final DirectoryWalker walker = new DirectoryWalker( dirRoot_ );
        final FileFilter filter = new AcceptAllFilter();
        assertCollectionsEqual( expectedValues, walker.getDirectories( filter ) );
    }


    /**
     *  Test the good path of getFilesAndDirectories()
     *
     * @exception  Exception If an error occurs
     */
    public void testGetFilesAndDirectories()
        throws Exception {

        final Collection expectedValues = new HashSet( 11 );
        expectedValues.add( new File( dirA_ ) );
        expectedValues.add( new File( dirB_ ) );
        expectedValues.add( new File( dirC_ ) );
        expectedValues.add( new File( dirD_ ) );
        expectedValues.add( new File( dirRoot_ ) );
        expectedValues.add( new File( fileB_ ) );
        expectedValues.add( new File( fileD_ ) );
        expectedValues.add( new File( fileA1_ ) );
        expectedValues.add( new File( fileA2_ ) );

        final DirectoryWalker walker = new DirectoryWalker( dirRoot_ );
        final FileFilter filter = new AcceptAllFilter();
        assertCollectionsEqual( expectedValues, walker.getFilesAndDirectories( filter ) );
    }


    /**
     *  Test the constructor with a file rather than a directory
     */
    public void testConstructor_File() {
        try {
            new DirectoryWalker( fileA1_ );
            fail( "expected exception" );
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }


    /**
     *  Test the constructor with a directory that doesn't exist
     */
    public void testConstructor_Exists() {
        try {
            new DirectoryWalker( "foo_file" );
            fail( "expected exception" );
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }


    private void delete( final String path )
        throws IOException {

        final File file = new File( path );
        if( file.delete() == false ) {
            throw new IOException( "Unable to delete [" + path + "] in tearDown()" );
        }
    }


    private void createDir( final String path )
        throws IOException {

        if( new File( path ).mkdir() == false ) {
            throw new IOException( "Unable to create dir [" + path + "] in setUp()" );
        }
    }


    private void createFile( final String filename )
        throws IOException {
        final FileOutputStream stream = new FileOutputStream( filename );
        stream.write( 'a' );
        stream.close();
    }


    private void assertCollectionsEqual( final Collection expected, final Collection actual ) {
        if( expected == null ) {
            fail( "expected was null" );
        }
        if( actual == null ) {
            fail( "actual was null" );
        }

        final Collection actualCopy = new ArrayList( actual );

        Object object;
        final Iterator iterator = expected.iterator();
        while( iterator.hasNext() ) {
            object = iterator.next();
            if( actualCopy.remove( object ) == false ) {
                fail( "Object [" + object + "] found in expected but not in actual.  Actual=" + actual );
            }
        }

        if( actualCopy.isEmpty() == false ) {
            fail( "more objects in actual than in expected.  Extras are: " + actualCopy );
        }
    }


    private class AcceptAllFilter implements FileFilter {
    		/** @inheritDoc FileFilter#accept(File) */
        public boolean accept( final File file ) {
            return true;
        }
    }
}

