/*
 * Copyright (c) 2002, 2004 Gargoyle Software Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Gargoyle Software Inc.
 *        (http://www.GargoyleSoftware.com/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 * 4. The name "Gargoyle Software" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact info@GargoyleSoftware.com.
 * 5. Products derived from this software may not be called "GSBase", nor may
 *    "GSBase" appear in their name, without prior written permission of
 *    Gargoyle Software Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARGOYLE
 * SOFTWARE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.gargoylesoftware.base.util;

import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import junit.framework.TestCase;

/**
 * Tests for DocumentUtil
 *
 * @version $Revision: 1.5 $
 * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
 */
public class DocumentUtilTest extends TestCase {

    /**
     * Create a new test
     * @param name The name of the test.
     */
    public DocumentUtilTest( final String name ) {
        super(name);
    }

    /**
     * Test upperCaseDocument() with mixed case inputs.
     * @throws Exception If something bad happens.
     */
    public void testUpperCaseDocument() throws Exception {
        final Document document
            = DocumentUtil.upperCaseDocument( new PlainDocument() );

        add( document, "aBc" );
        assertEquals( "ABC", getText(document) );

        add( document, "d" );
        assertEquals( "ABCD", getText(document) );
    }

    /**
     * Test upperCaseDocument() with null document.
     * @throws Exception If something bad happens.
     */
    public void testUpperCaseDocument_null() throws Exception {
        try {
            DocumentUtil.upperCaseDocument( null );
            fail("Expected exception for null document length");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals("delegate", e.getArgumentName());
        }
    }

    /**
     * Test lengthLimitedDocument() with a negative length.
     * @throws Exception If something bad happens.
     */
    public void testLengthLimitedDocument_negative() throws Exception {
        try {
            DocumentUtil.lengthLimitedDocument( new PlainDocument(), -1 );
            fail("Expected exception for negative length");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test lengthLimitedDocument() with a zero length.
     * @throws Exception If something bad happens.
     */
    public void testLengthLimitedDocument_zero() throws Exception {
        try {
            DocumentUtil.lengthLimitedDocument( new PlainDocument(), -1 );
            fail("Expected exception for zero length");
        }
        catch( final IllegalArgumentException e ) {
            // Expected path
        }
    }

    /**
     * Test lengthLimitedDocument() with a null document.
     * @throws Exception If something bad happens.
     */
    public void testLengthLimitedDocument_null() throws Exception {
        try {
            DocumentUtil.lengthLimitedDocument( null, 4 );
            fail("Expected exception for null document length");
        }
        catch( final DetailedNullPointerException e ) {
            assertEquals( "delegate", e.getArgumentName() );
        }
    }

    /**
     * Test lengthLimitedDocument to ensure that you can't enter more than the
     * specified number of characters.
     * @throws Exception If something bad happens.
     */
    public void testLengthLimitedDocument_4() throws Exception {
        final Document document
            = DocumentUtil.lengthLimitedDocument( new PlainDocument(), 4 );
        add( document, "a" );
        add( document, "b" );
        add( document, "c" );
        add( document, "d" );
        assertEquals( "abcd", getText(document) );
        add( document, "e" );
        assertEquals( "abcd", getText(document) );
    }

    /**
     * Insert the specified string at the end of the document
     * @param document the document
     * @param string The string to insert
     * @throws Exception If something bad happens.
     */
    private void add( Document document, String string ) throws Exception {
        document.insertString( document.getLength(), string, null );
    }

    /**
     * Return the full text of the document.
     * @param document The document
     * @return The content as a string
     * @throws Exception If something bad happens.
     */
    private String getText( final Document document ) throws Exception {
        return document.getText( 0, document.getLength() );
    }
}

